#
# Chum::Analysis::ExtData::NobleDNaseISite
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::ExtData::NobleDNaseISite - Find Noble DNaseI Hypersensitive sites from external data 
(Regulatory Potential Metric)

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::ExtData::NobleDNaseISite;

use vars qw(@ISA @NoDHS_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @NoDHS_PARAMS = qw(ext_file _load);
        foreach my $attr (@NoDHS_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
	if (!defined($self->{'ext_file'})) {
		$self->{'ext_file'} = $ENV{'CHUM_INSTALL_DIR'} . "/external_data/hs-genomic.hg15.sigmoid.predict";
	}
	unless(-e $self->{'ext_file'}) { $self->throw("External DNaseI File could not be found: " . $self->{'ext_file'}); }
        if ($self->{'_load'}) {
		$self->_load();	
	}
	
	return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub _load {
	my $self = shift;
	my %data_array;
	open(NOBLE, $self->{'ext_file'}) || $self->throw("Cannot preload noble file: " . $self->{'ext_file'});
	while (my $line = <NOBLE>) {
		chomp($line);
		if ($line =~ /[0-9]+\-CHR([\S]+)\-([0-9]+)[\S]{2}[\s]+([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]*/) {
			my $chromosome = $1;
			my $vstart = $2;
			my $score = $5;
			
			my $regstart = $vstart;
			$data_array{$chromosome}{int($vstart/1000000)}{$vstart} = $score; 
		}
	}
	close(NOBLE);
	print STDERR "Loaded NOBLE DNaseHS data\n";
	$self->{'_data'} = \%data_array;
}

sub in_dnase1_hs_site() {
	my $self = shift;
	my $chromosome = shift;
	my $start = shift;
	my $end = shift;

	if ($start > $end) {
		my $atmp = $end;
		$end = $start;
		$start = $atmp;
	}
	
	my @values = ();
	my $uchr = uc($chromosome);
	if (!$self->{'_load'} || !defined($self->{'_data'} )) {
		my $data = `grep \'\\\-CHR$uchr\\\-\' $self->{'ext_file'}`;
		my @lines = split ('\n', $data);
		foreach my $line (@lines) {
			if ($line =~ /[0-9]+[\S]+\-([0-9]+)[\S]{2}[\s]+([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]*/) {
				my $regstart = $1;
				my $regend = $regstart + 224;
				#print STDERR $regstart . " " . $regend . "\n";
				if ($start <= $regend && $end >= $regstart) {
					#print STDERR "DNaseI value: " . $4 . "\n";
					push @values, $4;
				}
			}
		}
	} else {
		my $index = int($start / 1000000);
		for my $regstart (keys %{$self->{'_data'}->{$uchr}->{$index}}) {
			my $regend = $regstart + 224;
			if ($start <= $regend && $end >= $regstart) {
				#print STDERR "DNaseI value: " . $self->{'_data'}->{$uchr}->{$index}->{$regstart} . "\n";
				push @values, $self->{'_data'}->{$uchr}->{$index}->{$regstart};
			}
		}
		if (int($start / 1000000) != int(($start - 225) / 1000000)) {
			print STDERR "Noble: Using alt for: " . $start . "\n";
			my $index = int(($start - 225) / 1000000);
			for my $regstart (keys %{$self->{'_data'}->{$uchr}->{$index}}) {
				my $regend = $regstart + 224;
				if ($start <= $regend && $end >= $regstart) {
					#print STDERR "DNaseI value: " . $self->{'_data'}->{$uchr}->{$index}->{$regstart} . "\n";
					push @values, $self->{'_data'}->{$uchr}->{$index}->{$regstart};
				}
			}
		}
	}

	if (scalar(@values) == 0) {
		$self->throw("No fragments found: " . $chromosome . ":" . $start . "-" . $end);
	} else {
		my $total_prob = 0;
		foreach my $value (@values) {
			$total_prob += $value;
		}
		return $total_prob / scalar(@values);
	}

	return 0;
}

1;