#
# Chum::Analysis::ExtData::PhastCons
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::ExtData::PhastCons - Find phast_cons score from external data downloaded from UCSC

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::ExtData::PhastCons;

use vars qw(@ISA @PC_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );
BEGIN {
	@PC_PARAMS = qw(file_hashref _completeness_check phast_cons_dir);
	foreach my $attr (@PC_PARAMS)
			{ $OK_FIELD{$attr}++; }
}

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
	while (@args) {
			my $attr = shift @args;
			my $value = shift @args;
			$self->{$attr} = $value;
	}
	if (!defined($self->{"_completeness_check"})) {
		$self->{"_completeness_check"} = 1;
	}

	if (!defined($self->{'phast_cons_dir'})) {
		$self->{'phast_cons_dir'} = $ENV{'CHUM_INSTALL_DIR'} . "/phast_cons";
	}
	unless (-d $self->{'phast_cons_dir'}) { $self->throw("No phast_cons directory found"); }
	my %file_hash;
	my @files = glob($self->{'phast_cons_dir'} . "/chr*");
	foreach my $file (@files) {
		if ($file =~ /chr([\S]+)_random/i) {
			#ignore randoms
		} elsif ($file =~ /chr([\S]+)/i) {
			%file_hash->{$1} = $file;
		}	
	}
	if ($self->{"_completeness_check"}) {
		if (scalar(keys %file_hash) < 24) {
			print Dumper \%file_hash;
			$self->throw("Missing a phast_cons chromosome file");
		}
	}
	$self->{'file_hashref'} = \%file_hash;

	return $self;
}

sub AUTOLOAD {
	my $self = shift;
	my $attr = $AUTOLOAD;
	$attr =~ s/.*:://;

	$self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
	$self->{$attr} = shift if @_;
	return $self->{$attr};
}

sub average_phast_cons_score() {
	my $self = shift;
	my $chromosome = shift;
	my $start = shift;
	my $end = shift;

	print STDERR "Looking for phast_cons with chr:" . $chromosome . " " . $start . "-" . $end . "\n";

	if ($start > $end) {
		my $atmp = $end;
		$end = $start;
		$start = $atmp;
	}

	my $file = $self->{'file_hashref'}->{$chromosome};
	if (!defined($file)) { $self->throw("Could not find phast_cons file for chromosome: " . $chromosome); }
	unless (-f $file) { $self->throw("Could not find phast_cons file for chromosome: " . $chromosome); }
	my @values;
	my $length = $end - $start + 1;
	my $data = `grep -w \'^$start\' $file -A $length`;
	my @lines = split('\n', $data);
	my $open = 0;
	my $index = 0;
	foreach my $line (@lines) {
		if ($line =~ /([0-9]+)[\s]+([0-9]+)[\s]*/i) {
			if ($1 == $start) {
				$open = 1;
				push @values, $2;
			} else {
				if ($open) {
					if ($1 <= $end) {
						push @values, $2;
					} else {
						$open = 0;
						last;
					}
				}
			}
			$index++;
		} 
	}
	print STDERR "Analyzed $index lines\n";
	my $total = 0;
	print STDERR "Length array: " . scalar(@values) . "\n";
	foreach my $value (@values) {
		$total += $value;
	}
	return $total / $length;
}

1;