#
# Chum::Analysis::ExtData::RegulatoryPotential
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::ExtData::RegulatoryPotential - Find regulatory potential score from external data downloaded from UCSC
(Regulatory Potential Metric)

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::ExtData::RegulatoryPotential;

use vars qw(@ISA @RP_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;
use File::Path qw(mkpath);

@ISA = qw( Bio::Root::Root );
BEGIN {
	@RP_PARAMS = qw(file_hashref _completeness_check regulatory_potential_dir _cache_on _mini_cache _make_cache_files _fork_make_cache_files);
	foreach my $attr (@RP_PARAMS)
		{ $OK_FIELD{$attr}++; }
}

my $HASHSEPERATOR = 1000000;

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
	while (@args) {
		my $attr = shift @args;
		my $value = shift @args;
		$self->{$attr} = $value;
	}
	
	###############################################################
	##SET COMPLETENESS CHECK
	if (!defined($self->{"_completeness_check"})) {
		$self->{"_completeness_check"} = 1;
	}

	###############################################################
	##LOAD CANONICAL FILE LOCATION, IF NOT SET
	if (!defined($self->{'regulatory_potential_dir'})) {
		$self->{'regulatory_potential_dir'} = $ENV{'CHUM_INSTALL_DIR'} . "/regulatory_potential";
	}
	unless (-d $self->{'regulatory_potential_dir'}) { $self->throw("No regulatory potential directory found"); }
	
	###############################################################
	##GET THE FILES FOR ANALYSIS
	my %file_hash;
	my @bedgraph_files = glob($self->{'regulatory_potential_dir'} . "/*bedGraph");
	foreach my $bedgraph_file (@bedgraph_files) {
		if ($bedgraph_file =~ /chr([\S]+).rp[\S]+.bedGraph/i) {
			$file_hash{$1} = $bedgraph_file;
		}	
	}
	if ($self->{"_completeness_check"}) {
		if (scalar(keys %file_hash) != 24) {
			#print Dumper \%file_hash;
			$self->throw("Missing a bedGraph regulatory potential file");
		}
	}
	$self->{'file_hashref'} = \%file_hash;
	
	###############################################################
	##MAKE ALL THE CACHE FILES
	if ($self->{'_make_cache_files'}) {
		for my $chromosome (keys %file_hash) {
			if ($self->{'_fork_make_cache_files'}) {
				unless (fork) {
					$self->_split_and_hash_idify_file($chromosome);
				}
			} else {
				$self->_split_and_hash_idify_file($chromosome);	
			}
		}
	}
	
	###############################################################
	##SET THE CACHE TO ON, IF REQUESTED, THIS IS THE PREFERRED MODE
	if (!defined($self->{"_cache_on"})) {
		$self->{"_cache_on"} = 1;
	}
	if ($self->{'_cache_on'}) {
		print STDERR "REGULATORY POTENTIAL: CACHE ON\n";
		my %cache;
		$self->{'_cache'} = \%cache;
		my %mini_cache;
		$self->{'_mini_cache'} = \%mini_cache;
		$self->_load_cache();
	}

	return $self;
}

sub AUTOLOAD {
	my $self = shift;
	my $attr = $AUTOLOAD;
	$attr =~ s/.*:://;

	$self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
	$self->{$attr} = shift if @_;
	return $self->{$attr};
}

##LOAD ALL THE FILES IN THE CACHE INTO THE CACHE
sub _load_cache {
	my $self = shift;
	my @chrdirectories = glob($self->{'regulatory_potential_dir'} . "/" . $HASHSEPERATOR . "/*");
	foreach my $chrdirectory (@chrdirectories) {
		my @partfiles = glob($chrdirectory . "/*.partfile");
		foreach my $partfile (@partfiles) {
			if ($partfile =~ /[\S]+\/([A-Za-z0-9]+)\/([A-Za-z0-9]+).partfile/g) {
				$self->{'_cache'}->{$HASHSEPERATOR}->{$1}->{$2} = $partfile;
			}
		}
	}
}

##CREATE A CACHE FILE DIRECTORY
sub _split_and_hash_idify_file {
	my $self = shift;
	my $chromosome = shift;
	print STDERR "HASH IDIFY $chromosome\n";
	my $id = 0;
	
	##ONLY DO THIS IF CACHE IS ON
	if ($self->{'_cache_on'}) {
		if(!$self->{'_cache'}->{$HASHSEPERATOR}->{$chromosome}) {
			open(FILE, $self->{'file_hashref'}->{$chromosome}) || $self->throw("Cannot open regulatory potential file: " .  $self->{'file_hashref'}->{$chromosome});
			my $current_hash_value = undef;
			my $current_file_descriptor = undef;
			my $current_index_filename = undef;
			my $ignore_hash_value = undef;
			while (my $line = <FILE>) {
				$id++;
				if ($line =~ /chr$chromosome[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([\S]+)[\s]*/i) {
					my $hash_value = int($1 / $HASHSEPERATOR);
					if (!defined($current_hash_value) || $current_hash_value ne $hash_value) {
						if (defined($current_file_descriptor)) {
							print STDERR "CLOSING $current_index_filename\n";
							close(	$current_file_descriptor ) || $self->throw("Could not close file $current_index_filename");
							$self->{'_cache'}->{$HASHSEPERATOR}->{$chromosome}->{$hash_value} = $current_index_filename;
						}
						if ($ignore_hash_value eq $hash_value) {
							print STDERR "I";
							next; ##Ignore failed hash values	
						}
						$current_hash_value = $hash_value;
						my $index_directory = $self->{'regulatory_potential_dir'} . "/" . $HASHSEPERATOR . "/" . $chromosome . "/";
						unless(-d $index_directory) {
							print STDERR "MAKING $index_directory \n";
							mkpath($index_directory);
						}
						my $index_file = $index_directory . $hash_value . ".partfile";
						unless (-e $index_file) {
							print STDERR "OPENING $index_file\n";
							open(HASHFILE, ">" .$index_file) || $self->throw("Could not open " . $index_file);
							$current_file_descriptor = *HASHFILE;
							$current_index_filename = $index_file
						} else {
							$self->warn("Index file exists for line $id: $line of " . $self->{'file_hashref'}->{$chromosome} . "  doing nothing: " . $index_file);
							$current_file_descriptor = undef;
							$current_index_filename = undef
							$ignore_hash_value = $current_hash_value;
						}
					}
					if (defined($current_file_descriptor)) {
						print $current_file_descriptor $line;
					}
				} else {
					$self->warn("Unknown line in regulatory potential file: " . $line);
				}
			}
			close(FILE);
			if (defined($current_file_descriptor)) {
				print STDERR "CLOSING $current_index_filename\n";
				close(	$current_file_descriptor ) || $self->throw("Could not close file $current_index_filename");	
			}
		}
	}
}

sub average_regulatory_potential_score() {
	my $self = shift;
	my $chromosome = shift;
	my $start = shift;
	my $end = shift;
	
	print STDERR "Looking for average regulatory potential score for chr:" . $chromosome . " " . $start . "-" . $end . "\n";
	
	##########################################################
	##CORRECT ORDER OF START AND END
	if ($start > $end) {
		my $atmp = $end;
		$end = $start;
		$start = $atmp;
	}
	my $length = $end - $start + 1;
	
	##########################################################
	##GET LINES TO ANALYZE DEPENDING ON WHETHER OR NOT THE CACHE IS ON
	my @lines = ();
	if ($self->{'_cache_on'}) {
		
		##a. Make sure we have cache files
		if (!$self->{'_cache'}->{$HASHSEPERATOR} || !$self->{'_cache'}->{$HASHSEPERATOR}->{$chromosome}) {
			$self->_split_and_hash_idify_file($chromosome);
			if (!$self->{'_cache'}->{$HASHSEPERATOR} || !$self->{'_cache'}->{$HASHSEPERATOR}->{$chromosome}) {
				$self->throw("Cannot find phast_cons partfile for HV: $HASHSEPERATOR and chromosome $chromosome");	
			}
		}
		
		##b. Get the hash indicies to use
		my $start_hash_value = int($start / $HASHSEPERATOR);
		my $end_hash_value = int($end / $HASHSEPERATOR);
		my @hash_values = ();
		if (($end_hash_value - $start_hash_value) < 0 || ($end_hash_value - $start_hash_value) > 10) {
			$self->throw("Out of cache range, can only handle " . (10 * $HASHSEPERATOR) . " bps");	
		} else {
			for (my $i = $start_hash_value; $i <= $end_hash_value; $i++) {
				push @hash_values, $i;
			}
		}
		
		##c. Get the files to work with
		my @part_files = ();
		foreach my $hash_value (@hash_values) {
			if (defined($self->{'_cache'}->{$HASHSEPERATOR}->{$chromosome}->{$hash_value})) {
				my %part_file = ('hash_value' => $hash_value, 'part_file' => $self->{'_cache'}->{$HASHSEPERATOR}->{$chromosome}->{$hash_value});
				push @part_files , \%part_file;	
			} else {
				$self->warn("Could not find a part file for HV: " . $hash_value . " and chromosome" . $chromosome);	
			}
		}
		#print Dumper \@part_files;
		
		##d. Load the region from the mini-cache or load from file
		my %new_mini_cache;
		foreach my $part_file (@part_files) {
			if (defined($self->{'_mini_cache'}->{$part_file->{'hash_value'}})) {
				$new_mini_cache{$part_file->{'hash_value'}} = $self->{'_mini_cache'}->{$part_file->{'hash_value'}};
			} else {
				eval {
					open(PARTFILE, $part_file->{'part_file'}) || $self->throw("Cannot open part_file: " . $part_file->{'part_file'});
					my @lines = ();
					while (my $line = <PARTFILE>) {
						push @lines, $line;	
					}
					close(PARTFILE);
					$new_mini_cache{$part_file->{'hash_value'}} = \@lines;
				};
				if ($@) {
					$self->warn($@);
				}
			}
		}
		$self->{'_mini_cache'} = \%new_mini_cache;
		
		##e. Load all lines in mini_cache into lines array for scanning
		foreach my $key (sort {$a <=> $b} keys %new_mini_cache) {
			push @lines, @{$new_mini_cache{$key}};
		}
	} else {
		##a. No cache of data so use grep directly on larger file
		my $file = $self->{'file_hashref'}->{$chromosome};
		my $artstart = int($end / 1000) + 1;
		my $artlength = 2000 + ($end - $start);
		my $grep_cmd = "grep -w \'$artstart\[0-9\]\\{3\\}\' $file -B $artlength -m 1";
		print STDERR "called grep cmd: $grep_cmd\n";
		my $data = `$grep_cmd`;
		@lines = split('\n', $data);
		print STDERR "No caching loaded: " . scalar(@lines) . "\n";
	}
	
	my @values;
	my $open = 0;
	my $index = 0;
	foreach my $line (@lines) {
		if ($line =~ /chr$chromosome[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([\S]+)[\s]*/i) {
			if ($1 == $start) {
				print STDERR "S0:" . $1 . "\n";
				$open = 1;
				push @values, $3;
			} else {
				if ($open) {
					if ($1 <= $end) {
						print STDERR "SC:" . $1 . "\n";
						push @values, $3;
					} else {
						print STDERR "SE:" . $1 . "\n";
						$open = 0;
						last;
					}
				}
			}
			$index++;
		} 
	}
	print STDERR "Analyzed $index lines length $length\n";
	if (scalar(@values) == 0) { $self->throw("Could not analyze region, no data points\n"); }
	my $total = 0;
	print STDERR "Length array: " . scalar(@values) . "\n";
	foreach my $value (@values) {
		print STDERR "+" . $value . "\n";
		$total += $value;
	}
	return $total / $length;
}

1;