#
# Chum::Analysis::Local::Zdna
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::Local::Zdna - Runs local version of zdna algorithm
requires Internet connection to zHUNT algorithm at Oregon State

=head1 SYNOPSIS

=head1 DESCRIPTION



=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::Local::Zdna;

use vars qw(@ISA @ZDNA_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::SeqIO;
use Chum::Analysis::Local::LocalApplicationI;
use File::Temp qw/ tempfile tempdir unlink0/;
use LWP;

@ISA = qw( Chum::Analysis::Local::LocalApplicationI );
BEGIN {
        @ZDNA_PARAMS = qw();
        foreach my $attr (@ZDNA_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
	$self->program("zhunt");
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub run_program {
	my $self = shift;
	my $reference_sequence = shift;
	my $stack_object = shift;
	my $results_file = shift;
	my $meta_input_hashref = shift;
	
	##SAVE REFERENCE SEQUENCE AS TEMP
	my ($tmpfh, $tmpfilename) = tempfile( "zHUNTXXXXXX", SUFFIX => '.fa', DIR => "/tmp/", UNLINK => 1);
        my $seqio = new Bio::SeqIO( -file => ">$tmpfilename",
                                    -format => "Fasta");
        $seqio->write_seq(new Bio::Seq(-seq => $reference_sequence->sequence->seq,
                                       -id => $reference_sequence->id));
	
	
	###RUN ZHUNT
	my $url = 'http://gac-web.cgrb.oregonstate.edu/zDNA/submit';
	my $ua = new LWP::UserAgent;
	$ua->agent("AgentName/0.1 " . $ua->agent);
	my $res = $ua->post($url, Content_Type => 'form-data', Content => [
								userfile        => ["$tmpfilename"],
								submit    => 'Submit Job',
								]);
	unlink0($tmpfh, $tmpfilename);
	
	if ($res->is_success) { print STDERR "zHunt submission success!\n";
	} else { $self->throw("ERROR: zHunt submission failed!");}

	my $job_id;
	if ($res->content =~ m/[\S\s]+JobID[\S]{1}([0-9]+)"[\S\s]+/g) {
		$job_id = $1;
	}
	if (!defined ($job_id)) {
		$self->throw("ERROR: A job ID could not be assigned, failing\n");
	} else {
		print STDERR "zHUNT assigned JOB_ID: " . $job_id . "\n";
	}
	sleep(2); ##wait time in seconds
	
	###GET RESULTS
	my $browser_results = LWP::UserAgent->new;
	my $url_results = 'http://gac-web.cgrb.oregonstate.edu/zDNA/result?JobID=' . $job_id;
	my $response = $browser_results->get( $url_results );
	my $content = $response->content;
	
	###SAVE RESULTS TO OUTPUT FILE
	open(RESULTS, ">$results_file") || $self->throw("ERROR: Could not open result file: $results_file for zHunt");
	print RESULTS $content;
	close(RESULTS);
	
	return 0;
}

sub convert_output_to_gff {
	my $self = shift;
	my $input_file = shift;
	my $output_file = shift;
	my $seqname = shift;

	open(INFILE, $input_file) || $self->throw("Cannot open input file: $input_file\n");
	my $opened = open(OUTFILE, ">$output_file");
	if (!$opened) {
		close(INFILE);
		$self->throw("Cannot open output file: $output_file\n");
	}
	foreach my $line (<INFILE>) {
		chomp($line);
		if ($line =~ /<TR><TD>([0-9]+)<\/TD><TD>([0-9]+)<\/TD><TD>([0-9]+.[0-9]+)<\/TD><TD>([ACTGactgNn-]+)<\/TD><\/TR>/g) {
			my $start = $1;
			my $end = $start + $2;
			my $zscore = $3;
			my $sequence = $4;
			print OUTFILE "$seqname\tzHunt\tmisc_feature\t$start\t$end\t" . $zscore . "\t.\t.\n";
			print STDERR "$seqname\tzHunt\tmisc_feature\t$start\t$end\t" . $zscore . "\t.\t.\n";
		}
	}
	close(INFILE);
	close(OUTFILE);
}

1;