#
# Chum::Analysis::Montham::Montham
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::Montham::Montham - Pattern importance through hamming

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::Montham::Montham;

use vars qw(@ISA @MONTHAM_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Benchmark;
use Bio::Root::Root;
use Bio::Seq;
use Math::VecStat qw(max);
use Data::Dumper;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @MONTHAM_PARAMS = qw(word_length);
        foreach my $attr (@MONTHAM_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
		if (!defined($self->{'word_length'})) {
			$self->{'word_length'} = 9;
		}

        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}


sub montham {
	my $self = shift;
	my $stack_object = shift;

	my $flank_half_length = int(($self->{'word_length'} - 1) / 2); 
	my $core_pattern = $stack_object->reference_sequence->get_sequence_from_flank($flank_half_length)->seq;
	my @variant_patterns = ();
	foreach my $variant_sequence (@{$stack_object->variant_sequences}) {
		my $variant_pattern = $variant_sequence->get_sequence_from_flank($flank_half_length)->seq;
		push @variant_patterns, {'variantseq' => $variant_sequence, 'variantpattern' => $variant_pattern};
	}

	my %comparison_results;
	foreach my $orthologous_sequence (@{$stack_object->orthologous_sequences}) {
		my $rscore = $self->_pairwise_analysis($stack_object->reference_sequence->sequence, $orthologous_sequence, $core_pattern);
		foreach my $variant_pattern (@variant_patterns) {
			my $vscore = $self->_pairwise_analysis($variant_pattern->{'variantseq'}->sequence, $orthologous_sequence, $variant_pattern->{'variantpattern'});
			print STDERR $orthologous_sequence->id . " " . $rscore . ":" . $vscore . "\n";
			$comparison_results{$orthologous_sequence->id}->{$variant_pattern->{'variantpattern'}} = $rscore . ":" . $vscore;
		}
	}
	return \%comparison_results;
}

sub _pairwise_analysis {
	my $self = shift;
	my $seqobj1 = shift;
	my $seqobj2 = shift;
	my $core_pattern = shift;
	$core_pattern = lc($core_pattern);
	print STDERR "Core pattern: " . $core_pattern . "\n";
	if (length($core_pattern) != $self->{'word_length'}) {
		$self->throw("Core pattern " . $core_pattern . " is wrong size: " . $self->{'word_length'});
	}

	my $sequence1 = lc($seqobj1->seq);
	my @chars1 = split '', $sequence1;
	my $sequence_rev1 = lc(reverse($seqobj1->revcom->seq));
	my @chars_rev1 = split '', $sequence_rev1;
	my $length1 = scalar(@chars_rev1);
	
	my $sequence2 = lc($seqobj2->seq);
	my @chars2 = split '', $sequence2;
	my $sequence_rev2 = lc(reverse($seqobj2->revcom->seq));
	my @chars_rev2 = split '', $sequence_rev2;
	my $length2 = scalar(@chars_rev2);

	#print STDERR $sequence1 . "\n" . $sequence2 . "\n";
	
	my @scores_array = ();
	for (my $i = 0; $i < scalar(@chars1) - $self->{'word_length'} + 1; $i++) {
		for (my $j = 0; $j < scalar(@chars2) - $self->{'word_length'} + 1; $j++) {
			my $mismatch_score_A = 0;
			my $mismatch_score_B = 0;
			my $mismatch_score_C = 0;
			my $mismatch_score_D  = 0;
			for (my $k = 0; $k < $self->{'word_length'}; $k++) {
				if ($chars1[$i+$k] ne $chars2[$j+$k]) { $mismatch_score_A++; }
				if ($chars_rev1[$i+$k] ne $chars2[$j+$k]) { $mismatch_score_B++; }	
				if ($chars1[$i+$k] ne $chars_rev2[$j+$k]) { $mismatch_score_C++; }
				if ($chars_rev1[$i+$k] ne $chars_rev2[$j+$k]) { $mismatch_score_D++; }
			}
			my $mismatch_score = 0;
			if ($mismatch_score_A < $mismatch_score_B && $mismatch_score_A < $mismatch_score_C && $mismatch_score_A < $mismatch_score_D) {
				$mismatch_score = $mismatch_score_A;
			} elsif ($mismatch_score_B < $mismatch_score_C && $mismatch_score_B < $mismatch_score_C && $mismatch_score_B < $mismatch_score_D) {
				$mismatch_score = $mismatch_score_B;
			} elsif ($mismatch_score_C < $mismatch_score_A && $mismatch_score_C < $mismatch_score_B && $mismatch_score_D < $mismatch_score_D) {
				$mismatch_score = $mismatch_score_C;
			} else {
				$mismatch_score = $mismatch_score_D;
			}
			#print STDERR $i . " " . $j . " " . substr($sequence1, $i, 9) . " " . substr($sequence2, $j, 9) . " " . $mismatch_score . "\n";
			$scores_array[$i][$j] = $mismatch_score;
		}
	}
	#print Dumper @scores_array;
	
	###score_columns
	my $char_length1 = scalar(@chars1) - $self->{'word_length'};
	my $char_length2 = scalar(@chars2) - $self->{'word_length'};
	
	my %col_score;
	for (my $i = 0; $i < $char_length1; $i++) {
		my $score = 0;
		for (my $j = 0; $j < $char_length2; $j++) {
			$score = $score + $scores_array[$i][$j];
		}
		$col_score{$i} = $score;
	}
	print %col_score->{495} . " " . %col_score->{496} . " " . %col_score->{497} . "\n";
	print STDERR "Column totals calculated\n";
	
	###score_rows
	#my %row_score;
	#for (my $j = 0; $j < $char_length2; $j++) {
	#	my $score = 0;
	#	for (my $i = 0; $i < $char_length1; $i++) {
	#		$score = $score + $scores_array[$i][$j];
	#	}
	#	$row_score{$j} = $score;
	#}
	#print STDERR "Row totals calculated\n";

	###sort to get lowest value words
	#my @col_sorted = sort { $col_score{$a} cmp $col_score{$b} } keys %col_score;
	#my @row_sorted = sort { $row_score{$a} cmp $row_score{$b} } keys %row_score;

	###find lowest value of pattern.
	my $final_col_score = -1;
	for (my $i = 0; $i < $char_length1; $i++) {
		if ($core_pattern eq substr($sequence1, $i, $self->{'word_length'})) {
			print STDERR "Match found col $i! " . substr($sequence1, $i, $self->{'word_length'}) ."\n";
			$final_col_score = %col_score->{$i};
			last;
		} elsif ($core_pattern eq substr($sequence_rev1, $i, $self->{'word_length'})) {
			print STDERR "Match found col $i! " . substr($sequence1, $i, $self->{'word_length'}) . "\n";
			$final_col_score = %col_score->{$i};
			last;	
		}
	}
	
	#my $row_score = -1;
	#for (my $j = 0; $j < $char_length2; $j++) {
	#	if ($core_pattern eq substr($sequence2, $j, $self->{'word_length'})) {
	#		print STDERR "Match found row $j!\n";
	#		$row_score = $row_score{$row_sorted[$j]};
	#		last;
	#	} elsif ($core_pattern eq substr($sequence_rev2, $j, $self->{'word_length'})) {
	#		print STDERR "Match found row $j!\n";
	#		$row_score = $row_score{$row_sorted[$j]};
	#		last;
	#	}
	#}

	#if ($col_score == -1 && $row_score == -1) {
	#	return -1;
	#} elsif ($col_score == -1 && $row_score > 0) {
	#	return $row_score / ($char_length1);
	#} elsif ($row_score == -1 && $col_score > 0) {
	#	return $col_score / ($char_length2);
	#} else {
	#	if (($row_score / $char_length1) < ($col_score / $char_length2)) {
	#		return $row_score / ($char_length1);
	#	} else {
	#		return $col_score / ($char_length2);
	#	}
	#}

	if ($final_col_score == -1) {
		return -1;
	} else {
		print STDERR "$final_col_score $char_length2 : " . ($final_col_score / ($char_length2)) . "\n";
		return $final_col_score / ($char_length2);
	}
}

1;
