#
# Chum::Analysis::Remote::Dan
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::Remote::Dan - Runs EMBOSS:Dan algorithm

=head1 SYNOPSIS

=head1 DESCRIPTION

Uses the Chinook platform (Montgomery et al. Nature Methods, August 2005) to remotely
run Dan against an input sequences.  

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::Remote::Dan;

use vars qw(@ISA @DAN_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Seq;
use Chum::Analysis::Remote::RemoteApplicationI;

@ISA = qw( Chum::Analysis::Remote::RemoteApplicationI );
BEGIN {
        @DAN_PARAMS = qw();
        foreach my $attr (@DAN_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub program {
	my $self = shift;
	return "EMBOSS-Dan";
}

sub get_meta_input_hashref {
	my $self = shift;
	my %meta_hash;
	return \%meta_hash;
}

sub load_data_entry_object_set {
	my $self = shift;
	my $deosets_arrayref = shift;
	my $service_object = shift;
	my $reference_sequence = shift;
	my $stack_object = shift;
	my $meta_input_hashref = shift;

	my $loaded = 0;
	foreach my $deoset (@$deosets_arrayref) {
		my @deos = ();
		if ($deoset->getName eq "dna_sequence") {
			$loaded++;
			my @required_deo_names = @{$deoset->getAllowedDEOTypes()};
			foreach my $required_deo_name (@required_deo_names) {
				if ($required_deo_name eq "DNA_SEQUENCE") {
					my @stacks = ();
					unshift @stacks, new Bio::Seq(	-id => $reference_sequence->id,
													-seq => $reference_sequence->sequence->seq);
					my $index2 = 0;
					foreach my $stack (@stacks) {
						my $deo = $self->chinook_manager->getDEO($required_deo_name);
						$deo->setName("SEQUENCE: " . $index2); ##A unique name for the object
                        $deo->setOrder("-1");   ##The precedence of the object in the set
                        my $attributes = $deo->getAttributes(); ##The collection of attributes
						while ( ( my $key, my $value ) = each(%$attributes) ) {
							if ($key eq "getSequence") { %$attributes->{$key}->{0} = $stack->seq; }
							elsif ($key eq "getName") { %$attributes->{$key}->{0} = "SEQUENCE" . $index2; }
							else { $self->warn("Unknown key $key"); }
						}
						push @deos, $deo;
						$index2++;
					}
				}
			}
		} else {
			$self->throw("Unknown data is required: " . $deoset->getName());
		}
		$deoset->setDEOs(\@deos);
	}
	return $loaded;
}

sub _convert_output_to_gff {
	my $self = shift;
	my $input_file = shift;
	my $output_file = shift;

	open(INFILE, $input_file) || $self->throw("Cannot open input file: $input_file\n");
	my $opened = open(OUTFILE, ">$output_file");
	if (!$opened) {
		close(INFILE);
		$self->throw("Cannot open output file: $output_file\n");
	}

	my $current_seqno = 1;
	foreach my $line (<INFILE>) {
		chomp($line);
		if ($line =~ /([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+([ATCGactgNn]+)/g) {
			my $start = $1;
			my $end = $2;
			my $tm = $3;
			my $gc = $4;
			my $deltaG = $5;
			my $deltaH = $6;
			my $deltaS = $7;
			my $tmProd = $8;
			my $sequence = $9;
			print OUTFILE "SEQ1\tDAN\tmisc_feature\t$start\t$end\t" . $tm . ":" . $gc . "\t.\t.\n";
		}
	}
	close(INFILE);
	close(OUTFILE);
}

1;