#
# Chum::Analysis::Remote::Motifsampler
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::Remote::Motifsampler - Runs motifsampler

=head1 SYNOPSIS

=head1 DESCRIPTION

Uses the Chinook platform (Montgomery et al. Nature Methods, August 2005) to remotely
run Motifsampler against a set of input sequences.  Results are returned as
GFF from the start of the reference input sequence.

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::Remote::Motifsampler;

use vars qw(@ISA @MOTIFSAMPLER_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Bio::Seq;
use Chum::Analysis::Remote::RemoteApplicationI;
use File::Copy;

@ISA = qw( Chum::Analysis::Remote::RemoteApplicationI );
BEGIN {
        @MOTIFSAMPLER_PARAMS = qw();
        foreach my $attr (@MOTIFSAMPLER_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub program {
	my $self = shift;
	return "MotifSampler";

}

sub get_meta_input_hashref {
	my $self = shift;
	my %meta_hash;
	$meta_hash{'background_file'} = '';
	return \%meta_hash;
}

sub _convert_output_to_gff {
	my $self = shift;
	my $input_file = shift;
	my $output_file = shift;
	copy($input_file, $output_file) || $self->throw("Cannot create gff file");
}

sub load_data_entry_object_set {
	my $self = shift;
	my $deosets_arrayref = shift;
	my $service_object = shift;
	my $reference_sequence = shift;
	my $stack_object = shift;
	my $meta_input_hashref = shift;

	if (!defined($meta_input_hashref->{'background_file'})) { $self->throw('ERROR: MotifSampler requires a background_file as input');}
	unless (-e $meta_input_hashref->{'background_file'}) { $self->throw('ERROR: MotifSampler requires a background_file as input');}

	my $loaded = 0;
	foreach my $deoset (@$deosets_arrayref) {
		my @deos;
		if ($deoset->getName() eq "background_file") {
			$loaded++;
			my @required_deo_names = @{$deoset->getAllowedDEOTypes()};
            foreach my $required_deo_name (@required_deo_names) {
				if ($required_deo_name eq "FILE") {
					my $deo = $self->chinook_manager->getDEO($required_deo_name);
					my $file_id = $self->chinook_manager->uploadFile($service_object, $meta_input_hashref->{'background_file'});
					if (!defined($file_id)) {
						$self->throw("ERROR: Could not upload background file to server\n");
                    }

					$deo->setName("BACKGROUND"); ##A unique name for the object
					$deo->setOrder("-1");   ##The precedence of the object in the set
					my $attributes = $deo->getAttributes(); ##The collection of attributes
					while ( ( my $key, my $value ) = each(%$attributes) ) {
							if ($key eq "getMode") { %$attributes->{$key}->{0} = $service_object->getMode(); }
							elsif ($key eq "getFilewireUri") { %$attributes->{$key}->{0} = $service_object->getFilewireUri(); }
							elsif ($key eq "getAbsoluteFileName") { %$attributes->{$key}->{0} = $meta_input_hashref->{'background_file'};  }
							elsif ($key eq "getFileId") { %$attributes->{$key}->{0} = $file_id; }
							elsif ($key eq "getName") { #do nothing 
							}
							else {
									$self->warn("Unknown key $key");
							}
					}
					push @deos, $deo;
				}
			}
		} elsif ($deoset->getName eq "sequences") {
			$loaded++;
			my @required_deo_names = @{$deoset->getAllowedDEOTypes()};
			foreach my $required_deo_name (@required_deo_names) {
				if ($required_deo_name eq "DNA_SEQUENCE") {
					my @stacks = @{$stack_object->orthologous_sequences};
					unshift @stacks, new Bio::Seq(	-id => $reference_sequence->id,
													-seq => $reference_sequence->sequence->seq);

					my $index2 = 0;
					foreach my $stack (@stacks) {
						my $deo = $self->chinook_manager->getDEO($required_deo_name);
						$deo->setName("SEQUENCE: " . $index2); ##A unique name for the object
						$deo->setOrder("-1");   ##The precedence of the object in the set
						my $attributes = $deo->getAttributes(); ##The collection of attributes
						while ( ( my $key, my $value ) = each(%$attributes) ) {
							if ($key eq "getSequence") { %$attributes->{$key}->{0} = $stack->seq; }
							elsif ($key eq "getName") { %$attributes->{$key}->{0} = "SEQUENCE" . $index2; }
							else {
								$self->warn("WARNING: Unknown key $key");
							}
						}

						push @deos, $deo;
						$index2++;
					}
				}
			}
		} else {
			$self->throw("ERROR: Unknown data is required: " . $deoset->getName());
		}
		$deoset->setDEOs(\@deos);
	}
	if ($loaded != 2) { $self->throw("ERROR: The data or background was not correctly loaded"); }
	return $loaded;
}

1;