#
# Chum::Analysis::Remote::Weeder
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::Remote::Weeder - Runs weeder

=head1 SYNOPSIS

=head1 DESCRIPTION

Uses the Chinook platform (Montgomery et al. Nature Methods, August 2005) to remotely
run Weeder against a set of input sequences.  Results are returned as
GFF from the start of the reference input sequence.

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::Remote::Weeder;

use vars qw(@ISA @WEEDER_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Seq;
use Chum::Analysis::Remote::RemoteApplicationI;

@ISA = qw( Chum::Analysis::Remote::RemoteApplicationI );
BEGIN {
        @WEEDER_PARAMS = qw(chinook_man);
        foreach my $attr (@WEEDER_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}


sub program {
	my $self = shift;
	return "weeder";

}

sub get_meta_input_hashref {
	my $self = shift;
	my %meta_hash;
	return \%meta_hash;
}

sub _convert_output_to_gff {
	my $self = shift;
	my $input_file = shift;
	my $output_file = shift;

	open(INFILE, $input_file) || $self->throw("ERROR: Cannot open input file: $input_file\n");
	my $opened = open(OUTFILE, ">$output_file");
	if (!$opened) {
		close(INFILE);
		$self->throw("ERROR: Cannot open output file: $output_file\n");
	}
	my %score_hash;
	my $current_seqno;
	my $current_motif;
	my $current_score;
	my $rev;
	foreach my $line (<INFILE>) {
		chomp($line);

		if ($line =~ />-1sequences([\S]+)[\s]*[\S]*/g) {
			$current_seqno = $1 + 1;
			$rev = 0;
		} elsif ($line =~ /^[\s]{1}([ATCGatcgNn]+)[\s]{1}$/g) {
			$current_motif = $1;
			$current_score = %score_hash->{$current_motif};
        } elsif ($line =~ /[0-9]+\)[\s]{1}([ATCG]+)[\s]{1}([0-9]+[\S]{1}[0-9]+)/g) {
			%score_hash->{$1} = $2;
		}
		elsif ($line =~ /REVCOMP[\s]+>-1sequences([\S]+)[\s]*[\S]*/g) {
			$current_seqno = $1 + 1;
			$rev = 1;
		}
		elsif ($line =~ /[\S\s]{1}([ATCGNatcgn]+)[\S\s]{1}[\s]+position[\s]+([\S]+)[\S]{1}[\s]+[\S]{1}([\S]+)[\S]{1}/g) {
			my $end = $2 + length($1) - 1;
			my $strand = "+";
			if ($rev == 1) {
					$strand = "-";
			}
			print OUTFILE "SEQ" . $current_seqno . "\tWEEDER\tpTFBS\t" . $2 . "\t" . $end . "\t" . $current_score . ":" . $3 . "\t" . $strand . "\t.\n";
		}
	}

	close(INFILE);
	close(OUTFILE);
}

sub load_data_entry_object_set {
	my $self = shift;
	my $deosets_arrayref = shift;
	my $service_object = shift;
	my $reference_sequence = shift;
	my $stack_object = shift;
	my $meta_input_hashref = shift;

	my $loaded = 0;
	foreach my $deoset (@$deosets_arrayref) {
		my @deos;
		if ($deoset->getName eq "sequences") {
			$loaded++;
			my @required_deo_names = @{$deoset->getAllowedDEOTypes()};
			foreach my $required_deo_name (@required_deo_names) {
				if ($required_deo_name eq "DNA_SEQUENCE") {
					my @stacks = @{$stack_object->orthologous_sequences};
					unshift @stacks, new Bio::Seq(	-id => $reference_sequence->id,
													-seq => $reference_sequence->sequence->seq);
					my $index2 = 0;
					foreach my $stack (@stacks) {
						my $deo = $self->chinook_manager->getDEO($required_deo_name);
						$deo->setName("SEQUENCE: " . $index2); ##A unique name for the object
                        $deo->setOrder("-1");   ##The precedence of the object in the set
                        my $attributes = $deo->getAttributes(); ##The collection of attributes
						while ( ( my $key, my $value ) = each(%$attributes) ) {
							if ($key eq "getSequence") { %$attributes->{$key}->{0} = $stack->seq; }
							elsif ($key eq "getName") { %$attributes->{$key}->{0} = "SEQUENCE" . $index2; }
							else { $self->warn("WARN: Unknown key $key"); }
						}
						push @deos, $deo;
						$index2++;
					}
				}
			}
		} else {
			$self->throw("ERROR: Unknown data is required: " . $deoset->getName());
		}
		$deoset->setDEOs(\@deos);
	}
	if ($loaded != 1) { $self->throw("ERROR: The data was not correctly loaded"); }
	return $loaded;
}

1;
