#
# Chum::Neural::Neuron::BendNeuron
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Neural::Neuron::BendNeuron - A neuron for understanding bend results

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Neural::Neuron::BendNeuron;

use vars qw(@ISA @BN_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @BN_PARAMS = qw();
        foreach my $attr (@BN_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
		$self->_loadTraining();
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub think() {
	my $self = shift;
	my $result = shift;
	
	my $positive_uprob = $self->_probabilityAboveScore($result->{'bend'}->{'score'}, $self->{'positives'});
	my $neutral_uprob = $self->_probabilityAboveScore($result->{'bend'}->{'score'}, $self->{'neutrals'});

	my $score = $positive_uprob / ( $positive_uprob + $neutral_uprob);
	if ($score >= $self->{'trained-score'}) {
		return (1, $score - $self->{'trained-score'});
	} else {
		return (0, $score - $self->{'trained-score'});
	}
}

sub _probabilityAboveScore {
	my $self = shift;
	my $score = shift;
	my $results_arrayref = shift;
	my @results = @$results_arrayref;

	my $count = 0;
	foreach my $result (@results) {
		if ($result >= $score) {
			$count++;
		}
	}
	return $count / scalar(@results);
}

sub _probabilityBelowScore {
	my $self = shift;
	my $score = shift;
	my $results_arrayref = shift;
	my @results = @$results_arrayref;

	my $count = 0;
	foreach my $result (@results) {
		if ($result <= $score) {
			$count++;
		}
	}
	return $count / scalar(@results);
}

sub train() {
	my $self = shift;
	my $positiveresults_arrayref = shift;
	my $neutralresultss_arrayref = shift;
	
	my @positives = ();
	my @neutrals = ();
	my @scores = ();
	foreach my $presult (@$positiveresults_arrayref) {
		push @scores, $presult->{'bend'}->{'score'};
		push @positives, $presult->{'bend'}->{'score'};
	}
	foreach my $nresult (@$neutralresultss_arrayref) {
		push @scores, $nresult->{'bend'}->{'score'};
		push @neutrals, $nresult->{'bend'}->{'score'};
	}
	@scores = sort @scores;
	my @diff = ();
	foreach my $score (@scores) {
		my $pscore = $self->_probabilityBelowScore($score, \@positives);
		my $nscore = $self->_probabilityAboveScore($score, \@neutrals);
		push @diff, abs($pscore - $nscore);
	}
	my $minimum_diff;
	my $minimum_index;
	for (my $i = 0; $i < scalar(@diff); $i++) {
		if (!defined($minimum_diff)) {
			$minimum_diff = @diff[$i];
			$minimum_index = $i;
		} 
		if (@diff[$i] < $minimum_diff) {
			$minimum_diff = @diff[$i];
			$minimum_index = $i;
		}
	}
	my $positive_uprob = $self->_probabilityAboveScore(@scores[$minimum_index], \@positives);
	my $neutral_uprob = $self->_probabilityAboveScore(@scores[$minimum_index], \@neutrals);
	$self->{'trained-score'} = $positive_uprob / ( $positive_uprob + $neutral_uprob);
	print STDERR $self->{'trained-score'} . "\n";
	$self->_saveTraining(\@positives, \@neutrals);
}

sub _saveTraining() {
	my $self = shift;
	my $positives_arrayref = shift;
	my $neutrals_arrayref = shift;
	my @positives = @$positives_arrayref;
	my @neutrals = @$neutrals_arrayref;
	open(BN, ">.bend.neuron") || $self->throw("Cannot store trained result\n");
	print BN "T:" . $self->{'trained-score'} . "\n";
	foreach my $presult (@positives) {
		print BN "P:" . $presult . "\n";
	}
	foreach my $nresult (@neutrals) {
		print BN "N:" . $nresult . "\n";
	}
	close(BN);
}

sub _loadTraining() {
	my $self = shift;
	if (-e ".bend.neuron") {
		open(BN, ".bend.neuron") || $self->throw("Cannot load training data\n");
		my @positives = ();
		my @neutrals = ();
		foreach my $line (<BN>) {
			chomp($line);
			if ($line =~ /P:([\S]+)/g) {
				push @positives, $1;
			} elsif ($line =~ /N:([\S]+)/g) {
				push @neutrals, $1;
			} elsif ($line =~ /T:([\S]+)/g) {
				$self->{'trained-score'} = $1;
			}
		}
		$self->{'positives'} = \@positives;
		$self->{'neutrals'} = \@neutrals;
		close(BN);
	}
}

1;