#
# Chum::Result::Interpretter::MotifsamplerInterpretter
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Result::Interpretter::MotifsamplerInterpretter - Interpret MotifSampler result sets

=head1 SYNOPSIS

=head1 DESCRIPTION

Interpret MotifSampler result sets

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut

package Chum::Result::Interpretter::MotifsamplerInterpretter;

use vars qw(@ISA @MINTERPRETTER_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;
use Chum::Result::Interpretter::InterpretterI;
use Chum::Result::MotifPrediction::MotifPredictionXMLReader;
use File::Temp qw/ tempfile tempdir unlink0/;

@ISA = qw(Chum::Result::Interpretter::InterpretterI);
BEGIN {
        @MINTERPRETTER_PARAMS = qw();
        foreach my $attr (@MINTERPRETTER_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->$attr($value);
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;
        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub _preload_data {
	my $self = shift;
	my $mpxmlr = new Chum::Result::MotifPrediction::MotifPredictionXMLReader();
	foreach my $result (@{$self->resultset->results}) {
		my $filename = $result->results_file;
		my ($tmpfh, $tmpfilename) = tempfile( "motifsamplerXXXXXX", SUFFIX => '.xml', DIR => "/tmp/", UNLINK => 1);
		my $matrix_file = undef;
		my $named_supplementary_result_files = $result->named_supplementary_result_files;
		if (defined($named_supplementary_result_files)) {
			my %named_supplementary_result_files_hash = %$named_supplementary_result_files;
			if (defined($named_supplementary_result_files_hash{'matrix_file'})) {
				$matrix_file = 	$named_supplementary_result_files_hash{'matrix_file'};
			}
		}
		
		$self->resultset->analysis_object->convert_output_to_xml($filename, $tmpfilename, $matrix_file);
		$result->results_object($mpxmlr->read($tmpfilename));
		unlink0($tmpfh, $tmpfilename) ;
	}
}

sub calculate_minimum_maximum {
        my $self = shift;

        my %iresults;
        foreach my $result (@{$self->resultset->results}) {
                my $filename = $result->results_file;
                if (!defined($result->results_object)) {
                        $self->throw("ERROR: Cannot find results for " . $filename);
                } else {
                        my @hits = ();
                        foreach my $predicted_motif (@{$result->results_object}) {
                                foreach my $hit (@{$predicted_motif->hits}) {
                                        if ($hit->name eq $result->sequence->id) {
                                                if ($result->sequence->is_start_stop_overlapping($hit->start, $hit->start + length($hit->sequence) - 1)) {
                                                        my %hitlet = ('hit' => $hit, 'score' => $predicted_motif->score);
                                                        push @hits, \%hitlet;
                                                }
                                        }
                                }
                        }
                        $iresults{$filename} = \@hits;
                        print STDERR "Loaded from " . $filename . ": " . scalar(@hits) . "\n";
                }
        }
        my $maximum_score = undef;
        my $minimum_score = undef;
        foreach my $result (@{$self->resultset->results}) {
                my $filename = $result->results_file;
                my $score = 0;
                if (defined($iresults{$filename})) {
                        foreach my $hit (@{$iresults{$filename}}) {
				if ($hit->{'score'} ne "-inf") {
					$score += $hit->{'score'};
				} else {
					$score = "-Inf";	
				}
                        }
                }
		
                if (!defined($maximum_score)) { $maximum_score = $score; }
                elsif ($score ne "-Inf") { if ($maximum_score eq "-Inf" || $score > $maximum_score) { $maximum_score = $score; }}
                if (!defined($minimum_score)) { $minimum_score = $score; }
                elsif ($score ne "-Inf") { if ($minimum_score ne "-Inf") { if ($score < $minimum_score) { $minimum_score = $score; }}}
        }

        return ($minimum_score, $maximum_score);
}

sub calculate_maximum_difference {
        my $self = shift;
        my ($min, $max) = $self->calculate_minimum_maximum();
        return ($min, $max, ($max - $min));
}




#sub maximum {
#	my $self = shift;
#	my $resultset = shift;
#
#	if (!defined($resultset)) {
#		return -1;
#	}
#	#my $variationlocref = $self->get_variation_location($resultset->{'stack'});
#	my @iresults;
#	foreach my $result (@{$resultset->{'results'}}) {
#		my $score = 0;
#		my $hits = 0;
#		foreach my $seqfeature (@{$result->{'seqfeat'}}) {			
#			if ($seqfeature->seq_id eq "-1sequences0" || $seqfeature->seq_id eq "SEQ1") { 
#				if ($self->hit_overlaps_variation($seqfeature, $result->{'seq'})) {
#					my @scores = split(/:/, $seqfeature->score);
#					$hits++;
#					$score += $scores[0];				
#				}
#			}
#		}
#		my %resultlet = ('hits' => $hits, 'score' => $score, 'result' => $result);
#		push @iresults, \%resultlet;
#	}
#
#	##TODO, INTERPRET THE RESULTS USING THE DIFFERENCING METHOD, TAKE THE MAGNITUDE, GIVE RESULTS BACK IN GOOD FORMAT
#	
#	my $canonical_hits = 0;
#	my $canonical_score = 0;
#	my $variant_max_score = 0;
#	my $variant_max_hits = 0;
#
#	foreach my $iresult (@iresults) {
#		if ($iresult->{'result'}->{'type'} eq "CANONICAL") {
#			$canonical_hits = $iresult->{'hits'};
#			$canonical_score = $iresult->{'score'};
#		} elsif ($iresult->{'result'}->{'type'} eq "VARIANT") {
#			if ($iresult->{'score'} > $variant_max_score) {
#				$variant_max_score = $iresult->{'score'};
#				$variant_max_hits = $iresult->{'hits'};
#			}
#		}
#	} 
#	
#	return $canonical_score + $variant_max_score;
#}
#
#sub interpret {
#	my $self = shift;
#	my $resultset = shift;
#
#	if (!defined($resultset)) {
#		return -1;
#	}
#	#my $variationlocref = $self->get_variation_location($resultset->{'stack'});
#	my @iresults;
#	foreach my $result (@{$resultset->{'results'}}) {
#		my $score = 0;
#		my $hits = 0;
#		foreach my $seqfeature (@{$result->{'seqfeat'}}) {			
#			if ($seqfeature->seq_id eq "-1sequences0" || $seqfeature->seq_id eq "SEQ1") { 
#				if ($self->hit_overlaps_variation($seqfeature, $result->{'seq'})) {
#					my @scores = split(/:/, $seqfeature->score);
#					$hits++;
#					$score += $scores[0];				
#				}
#			}
#		}
#		my %resultlet = ('hits' => $hits, 'score' => $score, 'result' => $result);
#		push @iresults, \%resultlet;
#	}
#
#	##TODO, INTERPRET THE RESULTS USING THE DIFFERENCING METHOD, TAKE THE MAGNITUDE, GIVE RESULTS BACK IN GOOD FORMAT
#	
#	my $canonical_hits = 0;
#	my $canonical_score = 0;
#
#	foreach my $iresult (@iresults) {
#		if ($iresult->{'result'}->{'type'} eq "CANONICAL") {
#				$canonical_hits = $iresult->{'hits'};
#				$canonical_score = $iresult->{'score'};
#		}
#	} 
#	if ($canonical_hits == 0 && $canonical_score == 0) {
#		warn "No reference hits detected\n";
#	}
#
#
#	##THIS IS THE MAGNITUDE OF THE DISTANCE
#	my $max_distance = 0;
#	my $processed_variant = 0;
#	foreach my $iresult (@iresults) {
#		if ($iresult->{'result'}->{'type'} eq "VARIANT") {
#			my $variant_hits = $iresult->{'hits'};
#			my $variant_score = $iresult->{'score'};
#			my $t_max_distance = ($variant_score - $canonical_score) * ($variant_score - $canonical_score);
#			if ($t_max_distance > $max_distance) {
#				$max_distance = $t_max_distance;
#			}
#			$processed_variant = 1;
#		}
#	} 
#	if (!$processed_variant) {
#		warn "No variant hits detected";
#	}
#
#	return sqrt($max_distance);
#}

1;