#
# Chum::Result::Interpretter::WeederInterpretter
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Result::Interpretter::WeederInterpretter - Interpret weeder result sets

=head1 SYNOPSIS

=head1 DESCRIPTION

Interpret weeder result sets

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut

package Chum::Result::Interpretter::WeederInterpretter;

use vars qw(@ISA @WINTERPRETTER_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;
use Chum::Result::Interpretter::InterpretterI;
use Chum::Result::MotifPrediction::MotifPredictionXMLReader;
use File::Temp qw/ tempfile tempdir unlink0/;

@ISA = qw(Chum::Result::Interpretter::InterpretterI);
BEGIN {
        @WINTERPRETTER_PARAMS = qw();
        foreach my $attr (@WINTERPRETTER_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->$attr($value);
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;
        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub _preload_data {
	my $self = shift;
	my $mpxmlr = new Chum::Result::MotifPrediction::MotifPredictionXMLReader();
	foreach my $result (@{$self->resultset->results}) {
		my $filename = $result->results_file;
		my ($tmpfh, $tmpfilename) = tempfile( "weederXXXXXX", SUFFIX => '.xml', DIR => "/tmp/", UNLINK => 1);
		$self->resultset->analysis_object->convert_output_to_xml($filename, $tmpfilename);
		$result->results_object($mpxmlr->read($tmpfilename));
		unlink0($tmpfh, $tmpfilename) ;
	}
}

sub calculate_minimum_maximum {
	my $self = shift;
	
	my %iresults;
	foreach my $result (@{$self->resultset->results}) {
		my $filename = $result->results_file;
		if (!defined($result->results_object)) { 
			$self->throw("ERROR: Cannot find results for " . $filename);	
		} else {
			my @hits = ();
			foreach my $predicted_motif (@{$result->results_object}) {
				foreach my $hit (@{$predicted_motif->hits}) {
					if ($hit->name eq $result->sequence->id) {
						if ($result->sequence->is_start_stop_overlapping($hit->start, $hit->start + length($hit->sequence) - 1)) {
							my %hitlet = ('hit' => $hit, 'score' => $predicted_motif->score); 
							push @hits, \%hitlet;
						}
					}
				}	
			}
			$iresults{$filename} = \@hits;
			print STDERR "Loaded from " . $filename . ": " . scalar(@hits) . "\n";
		}
	}
	my $maximum_score = undef;
	my $minimum_score = undef;
	foreach my $result (@{$self->resultset->results}) {
		my $filename = $result->results_file;
		my $score = 0;
		if (defined($iresults{$filename})) {
			foreach my $hit (@{$iresults{$filename}}) {
				$score += $hit->{'score'}; #* $hit->{'hit'}->score;
			}
		}
		if (!defined($maximum_score)) { $maximum_score = $score; }
		elsif ($score > $maximum_score) { $maximum_score = $score; }
		if (!defined($minimum_score)) { $minimum_score = $score; }
		elsif ($score < $minimum_score) { $minimum_score = $score; }
	}
	
	return ($minimum_score, $maximum_score);
}

sub calculate_maximum_difference {
	my $self = shift;
	my ($min, $max) = $self->calculate_minimum_maximum();
	return ($min, $max, ($max - $min));
}

#sub maximum {
#	my $self = shift;
#	my $resultset = shift;
#	if (!defined($resultset)) { return 0; }
#
#	my @iresults;
#	foreach my $result (@{$resultset->{'results'}}) {
#		print "Analyzing: " . $result->{'type'} . "\n";
#		my $score = 0;
#		my $hits = 0;
#		foreach my $seqfeature (@{$result->{'seqfeat'}}) {	
#			if ($seqfeature->seq_id eq "-1sequence0" || $seqfeature->seq_id eq "SEQ1") { 
#				if ($self->hit_overlaps_variation($seqfeature, $result->{'seq'})) {
#					my @scores = split(/:/, $seqfeature->score);
#					$hits++;
#					$score += @scores[0];				
#					print "Overlap score " . @scores[0] . " " . $result->{'type'} .  " " . $seqfeature->start . " " . $seqfeature->end . "\n";
#				}
#			}
#		}
#		my %resultlet = ('hits' => $hits, 'score' => $score, 'result' => $result);
#		push @iresults, \%resultlet;
#	}
#
#	my $canonical_hits = 0;
#	my $canonical_score = 0;
#	my $variant_max_score = 0;
#	my $variant_max_hits = 0;
#
#	foreach my $iresult (@iresults) {
#		if ($iresult->{'result'}->{'type'} eq "CANONICAL") {
#			$canonical_hits = $iresult->{'hits'};
#			$canonical_score = $iresult->{'score'};
#		} elsif ($iresult->{'result'}->{'type'} eq "VARIANT") {
#			if ($iresult->{'score'} > $variant_max_score) {
#				$variant_max_score = $iresult->{'score'};
#				$variant_max_hits = $iresult->{'hits'};
#			}
#		}
#	} 
#	
#	return $canonical_score + $variant_max_score;
#}
#
#sub interpret {
#	my $self = shift;
#	my $resultset = shift;
#	if (!defined($resultset)) { return 0; }
#
#	#my $variationlocref = $self->get_variation_location($resultset->{'stack'});
#
#	my @iresults;
#	foreach my $result (@{$resultset->{'results'}}) {
#		print "Analyzing: " . $result->{'type'} . "\n";
#		my $score = 0;
#		my $hits = 0;
#		foreach my $seqfeature (@{$result->{'seqfeat'}}) {	
#			if ($seqfeature->seq_id eq "-1sequence0" || $seqfeature->seq_id eq "SEQ1") { 
#				if ($self->hit_overlaps_variation($seqfeature, $result->{'seq'})) {
#					my @scores = split(/:/, $seqfeature->score);
#					$hits++;
#					$score += $scores[0];				
#					print "Overlap score " . $scores[0] . " " . $result->{'type'} .  " " . $seqfeature->start . " " . $seqfeature->end . "\n";
#				}
#			}
#		}
#		my %resultlet = ('hits' => $hits, 'score' => $score, 'result' => $result);
#		push @iresults, \%resultlet;
#	}
#
#	my $canonical_hits = 0;
#	my $canonical_score = 0;
#
#	foreach my $iresult (@iresults) {
#		if ($iresult->{'result'}->{'type'} eq "CANONICAL") {
#			$canonical_hits = $iresult->{'hits'};
#			$canonical_score = $iresult->{'score'};
#		}
#	} 
#	print "CANONICAL: " . $canonical_score . " " . $canonical_hits . "\n";
#
#	##THIS IS THE MAGNITUDE OF THE DISTANCE
#	my $max_distance = 0;
#	my $ran_once = 0;
#	my $total_variant_hits = 0;
#	foreach my $iresult (@iresults) {
#		if ($iresult->{'result'}->{'type'} eq "VARIANT") {
#			$ran_once = 1;
#			my $variant_hits = $iresult->{'hits'};
#			$total_variant_hits += $variant_hits;
#			my $variant_score = $iresult->{'score'};
#			print "VARIANT: " . $variant_score . " " . $variant_hits . "\n";
#			my $t_max_distance = ($variant_score - $canonical_score) * ($variant_score - $canonical_score);
#			if ($t_max_distance > $max_distance) {
#				$max_distance = $t_max_distance;
#			}
#		}
#	} 
#	if (!$ran_once) {
#		$max_distance = (0 - $canonical_score) * (0 - $canonical_score);
#	}
#	if ($total_variant_hits == 0 && $canonical_hits == 0) {
#		return -1; ## No hits detected on variant.
#	}
#
#	return sqrt($max_distance);
#}

1;
