#
# Chum::Result::Result
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Result::Result - A result container from any analysis.  

=head1 SYNOPSIS

=head1 DESCRIPTION

Stores output information from an analysis.  Stores information about the type
of analysis run (whether it was the canonical sequence or a variant sequence, including the used sequence
and the filename of the output results.

Example:

my $result = new Chum::Result::Result('type' => 'CANONICAL',
				      'sequence' => $stack_object->reference_sequence,
				      'results_file' => "foo");
					
Usually many Chum::Result::Result objects are stored in a Chum::Result::ResultSet object

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Result::Result;

use vars qw(@ISA @RESULT_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @RESULT_PARAMS = qw(type sequence results_file named_supplementary_result_files_hashref results_object);
        foreach my $attr (@RESULT_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->$attr($value);
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

=head2 type

 Usage   : $result->type;
	the type of analysis result can either be CANONICAL or VARIANT.  CANONICAL results
	are from reference sequences derived from a genome.  VARIANT results are from
	sequences containing a variant as derived artificially or from a population genetics-based source.
 Returns : string
 Args    : optional string argument can be passed to set the value. i.e. $result->type("CANONICAL");

=cut

sub type {  
	my $self = shift;
	if(@_) {
		my $type = shift;
		if (uc($type) ne "CANONICAL" &&
		    uc($type) ne "VARIANT") {
			$self->throw("ERROR: type can only be either CANONICAL (reference) or VARIANT");	
		} else {
			$self->{'type'} = uc($type);	
		}
	}
	return $self->{'type'};
}

=head2 type

 Usage   : $result->sequence;
	the Chum::VariantSequence sequence object used to run the analysis
 Returns : Chum::VariantSequence object
 Args    : optional Chum::VariantSequence argument can be passed to set the value. i.e. $result->sequence($variant_sequence_object);

=cut

sub sequence {
	my $self = shift;
	if(@_) {
		my $sequence = shift;
		if (!$sequence->isa("Chum::VariantSequence")) {
			$self->throw("ERROR: sequence can only be a Chum::VariantSequence object");
		} else {
			$self->{'sequence'} = $sequence;	
		}
	}
	return $self->{'sequence'};
}

=head2 results_file

 Usage   : $result->results_file;
	the location of the raw output from this analysis as a filename
 Returns : string
 Args    : optional string argument can be passed to set the value. i.e. $result->results_file("/tmp/results.out");

=cut

sub results_file {
	my $self = shift;
	if(@_) {
		my $results_file = shift;
		if(-f $results_file) {
			$self->{'results_file'} = $results_file;
		} else {
			$self->throw("ERROR: results file does not exist");	
		}
	}
	return $self->{'results_file'};
}

=head2 named_supplementary_result_files

 Usage   : $result->named_supplementary_result_files;
	the location of the supplementary output files from this analysis as a filenames
 Returns : hashref
 Args    : optional hashref argument can be passed to set the value. i.e. $result->named_supplementary_result_files(\%other_output_files);

=cut

sub named_supplementary_result_files {
	my $self = shift;
	if(@_) {
		my $named_supplementary_result_files_hashref = shift;
		my %named_supplementary_result_files_hash = %$named_supplementary_result_files_hashref;
		foreach my $key (keys %named_supplementary_result_files_hash) {
			unless(-f $named_supplementary_result_files_hash{$key}) {
				$self->throw("ERROR: results file does not exist");	
			}
		}
		$self->{'named_supplementary_result_files_hashref'} = $named_supplementary_result_files_hashref;
	}
	return $self->{'named_supplementary_result_files_hashref'};
}

=head2 results_object

 Usage   : $result->results_object;
	results object (can be anything to store any results from an embedded analysis).  this is used for analyses that
	are not run externally of Chum.
 Returns : object
 Args    : optional object argument can be passed to set the value. i.e. $result->results_object($features);

=cut

sub results_object {
	my $self = shift;
	if(@_) {
		$self->{'results_object'} = shift;
	}
	return $self->{'results_object'};
}

1;