#
# Chum::Result::ResultSet
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Result::ResultSet - The results from a set of analyses

=head1 SYNOPSIS

=head1 DESCRIPTION

Stores information about results.  The input Chum::Stack object, the program ran, and
the set of Chum::Result::Result objects

Example:
my $resultset = new Chum::Result::ResultSet(	'analysis_object' => $myapplication,
						'stack' => $stack_object,
						'program' => "program"
						'results' => \@results);
						
Typically Chum::Result::ResultSet objects are analyzed by the appropriate interpretter object to
yield quantitative results for a particular set of runs.

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut

package Chum::Result::ResultSet;

use vars qw(@ISA @RESULTSET_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @RESULTSET_PARAMS = qw(analysis_object stack program results);
        foreach my $attr (@RESULTSET_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->$attr($value);
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

=head2 analysis_object

 Usage   : $resultset->analysis_object;
	the Chum::Result::ApplicationI object that was used to run the analyses
 Returns : Chum::Result::ApplicationI  object
 Args    : optional Chum::Result::ApplicationI  argument can be passed to set the value. i.e. $resultset->analysis_object($myapplication);

=cut

sub analysis_object {
	my $self = shift;
	if(@_) {
		my $analysis_object = shift;
		if (!$analysis_object->isa("Chum::Analysis::ApplicationI")) {
			$self->throw("ERROR: analysis_object must be a Chum::Analysis::ApplicationI object");
		} else {
			$self->{'analysis_object'} = $analysis_object;
		}
	}
	return $self->{'analysis_object'};	
}

=head2 stack

 Usage   : $resultset->stack;
	the Chum::Stack object that was used for this set of analyses
 Returns : Chum::Stack object
 Args    : optional Chum::Stack argument can be passed to set the value. i.e. $resultset->stack($stack_object);

=cut

sub stack { 
	my $self = shift;
	if(@_) {
		my $stack = shift;
		if (!$stack->isa("Chum::Stack")) {
			$self->throw("ERROR: stack must be a Chum::Stack object");
		} else {
			$self->{'stack'} = $stack;
		}
	}
	return $self->{'stack'};
}

=head2 program

 Usage   : $resultset->program;
	the program used to run these sets of analyses
 Returns : string
 Args    : optional string argument can be passed to set the value. i.e. $resultset->program("myprogram");

=cut

sub program {
	my $self = shift;
	if(@_) {
		my $program = shift;
		$self->{'program'} = $program;
	}
	return $self->{'program'};
}

=head2 results

 Usage   : $resultset->results;
	the results for various analyses (typically of different variant sequences)
 Returns : arrayref of Chum::Result::Result objects
 Args    : optional arrayref of Chum::Result::Result objects argument can be passed to
	set the value. i.e. $resultset->results(\@results);

=cut

sub results {
	my $self = shift;
	if(@_) {
		my $results = shift;
		if (@$results) {
			foreach my $result (@$results) {
				if (!$result->isa("Chum::Result::Result")) {
					$self->throw("ERROR: result must be a Chum::Result::Result object");
				}
			}
		}
		$self->{'results'} = $results;
	}
	return $self->{'results'};
}

1;
