#
# Chum::Stackbuilder
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Stackbuilder - Builds a stack of orthologous sequences around a source of variation

=head1 SYNOPSIS

=head1 DESCRIPTION

Builds a stack of orthologous sequences around a source of variation

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Stackbuilder;

use vars qw(@ISA @STACKBUILDER_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::EnsEMBL::Registry;
use Bio::LocatableSeq;
use Bio::Root::Root;
use Bio::Tools::GFF;
use Data::Dumper;
use Chum::Stack;
use Chum::VariationTools;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @STACKBUILDER_PARAMS = qw(_memconserve _registry _upstream _downstream _species _binomials);
        foreach my $attr (@STACKBUILDER_PARAMS)
                { $OK_FIELD{$attr}++; }
}

my @DEFAULT_SPECIES = ("mouse", "rat", "chimp", "rabbit", "elephant",
		       "macaque", "cow", "dog", "dasypus", "echinops",
		       "human", "oryzias", "takifugu", "tetraodon");
my $DEFAULT_UPSTREAM_BP = 500; # 5'
my $DEFAULT_DOWNSTREAM_BP = 500; # 3'
my $DEFAULT_PROMOTER_SIZE_BP = 6000;

=head2 new

 Function: Instantiates a Stackbuilder
 Usage   : my $stackbuilder = new Stackbuilder(); or
	   my $stackbuilder = new Stackbuilder(	'_memconserve' => 1, ##Conserve memory by reducing embedded object sizes (optional)
						'_registry' => "Bio::EnsEMBL::Registry", ##The EnsEMBL Registry for fetching orthologues (optional)
						'_upstream' => 100, ##The upstream sequence to fetch around a variant (optional)
						'_downstream' => 100, ##The downstream sequence to fetch around a variant (optional)
						'_species' => $species_arrayref,   ##The species to get orthologues for (optional)
						'_binomials' => $binomials_hashref); ##Hashref of species binomial names (optional)
 Returns : Chum::Stackbuilder

=cut

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
	if (!defined($self->{'_memconserve'})) {
		$self->{'_memconserve'} = 0;
	}
	if (!defined($self->{'_downstream'})) {
		$self->{'_downstream'} = $DEFAULT_DOWNSTREAM_BP;
	} elsif ($self->{'_downstream'} < 0) {
		$self->throw("Negative (Downstream) 3' sequence requested");
	}
	if (!defined($self->{'_upstream'})) {
		$self->{'_upstream'} = $DEFAULT_UPSTREAM_BP;
	} elsif ($self->{'_upstream'} < 0) {
		$self->throw("Negative (Upstream) 5' sequence requested");
	}
	if (!defined($self->{'_registry'})) {
		$self->{'_registry'} = "Bio::EnsEMBL::Registry";
		$self->{'_registry'}->load_all();	
	}
	if (!defined($self->{'_species'})) { ##The species to get 
		$self->{'_species'} = \@DEFAULT_SPECIES;	
	}
	return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;
        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}



sub generate_promoter_variants_stacks_for_ensg {
	my $self = shift;
	my $ensembl_gene_id = shift;

	##GET THE TARGET GENE###################################################
	my $ga = $self->{'_registry'}->get_adaptor("human", "core", "Gene");
	my $gene = $ga->fetch_by_stable_id($ensembl_gene_id);
	if (!defined($gene)) {
        	$self->throw("Could not find EnsEMBL gene\n");
	}
	$gene = $gene->transform('toplevel');

	##LOAD THE VARIATION ADAPTOR############################################
	my $vfa = $self->{'_registry'}->get_adaptor("human", "variation", "VariationFeature");
	if (!defined($vfa)) {
        	$self->throw("Could not get VariationFeatureAdaptor");
	}
	
	##GET THE ORTHOLOGOUS REGIONS###########################################
	my @stack_objects;
	my $binomialsref = $self->_get_species_binomials();
	my %binomials = %$binomialsref;
	
	my @slices = @{$self->_get_slices($gene, 'promoter')};
	foreach my $slice (@slices) {

        	##GET ALL VARIATION ON SLICE MEETING MAF CRITERIA###############
        	my @variation_features = @{$vfa->fetch_all_by_Slice($slice)};

        	foreach my $variation_feature (@variation_features) {
			if (defined($variation_feature)) {
				my $variation = $variation_feature->variation;

				##CHECK IF VARIATION HAS A MINOR ALLELE FREQUENCY WITHIN SPECIED RANGE###########
				if ($self->{'_rangecheck'}) {
					my @alleles = @{$variation->get_all_Alleles()};
					my $skip = 0;
					foreach my $allele (@alleles) {
					     	if (!defined($allele->frequency)) { $skip = 1; }
					}
					if ($skip) { next; }

					my $process = 0;
					foreach my $allele (@alleles) {
					  	if ($allele->frequency >= $self->{'minmaf'} && $allele->frequency <= $self->{'maxmaf'}) { 
							$process = 1; 
						}
					}
					if (!$process) { next; }
				}

				##PROCESS THE VARIATION#################################
				my $stack = $self->_generate_stack_for_variation_feature($variation_feature, \%binomials);
				if (defined($stack)) {
					push @stack_objects, $stack;
				}
			}
        	}
	}

	return \@stack_objects;
}

=head2 generate_stack_from_variation_name_source

 Function: Gets a Chum::Stack object for a variation name and source database (recognized by EnsEMBL)
 Usage   : $stackbuilder->generate_stack_from_variation_name_source('rs999', 'dbSNP');
 Throws  : Exception if the variation object cannot be uniquely mapped to the genome
 Returns : Chum::Stack
 Args	 : [1] String (variation_name, i.e. rs999)
	   [2] String (variation_source, i.e. dbSNP)

=cut

sub generate_stack_from_variation_name_source {
	my $self = shift;
	my $variation_name = shift;
	my $variation_source = shift;
	
	my $vtools = new Chum::VariationTools();
	my $variation_object = $vtools->get_variation_object_from_ensembl_for_name_source($variation_name, $variation_source, $self->{'_registry'});
	if (!defined($variation_object)) { $self->throw("Could not retrieve a Variation object from EnsEMBL"); }	
	my $variation_features_arrayref = $vtools->get_variation_features_for_ensembl_variation_object($variation_object, $self->{'_registry'});
	if (!defined($variation_features_arrayref)) { $self->throw("Could not retrieve a VariationFeature object from EnsEMBL"); }
	my @variation_feature = @$variation_features_arrayref;
	return $self->generate_stack_from_variation_feature_object($variation_feature[0]);
}

=head2 generate_stack_from_variation_name_source

 Function: Gets a Chum::Stack object for a variation_feature object
 Usage   : $stackbuilder->generate_stack_from_variation_feature_object($variation_feature);
 Returns : Chum::Stack
 Args	 : [1] Bio::EnsEMBL::Variation::VariationFeature

=cut

sub generate_stack_from_variation_feature_object {
	my $self = shift;
	my $variation_feature = shift;

	my $binomialsref = $self->_get_species_binomials();
	return $self->_generate_stack_for_variation_feature($variation_feature, $binomialsref);
}

=head2 generate_stacks_from_variation_file

 Function: From a specially formatted GFF file, assemble Chum::Stack objects
 Usage   : $stackbuilder->generate_stacks_from_variation_file('variation.gff');
	   The variation GFF file must be in the format
		VARIANT-1	CHROMOSOME	VARIANTID	100000	100001	.	+	.	T
		REFERENCE-1	CHROMOSOME	VARIANTID	100000	100001	.	+	.	C
		VARIANT-2	CHROMOSOME	VARIANTID	200000	200001	.	+	.	-
		VARIANT-2	CHROMOSOME	VARIANTID	200000	200001	.	+	.	A
		REFERENCE-2	CHROMOSOME	VARIANTID	200000	200001	.	+	.	G
		etc...
		
 Returns : arrayref of Chum::Stack objects
 Args	 : String (filename) of variation GFF file.

=cut

sub generate_stacks_from_variation_file {
	my $self = shift;
	my $variation_file = shift;
	my $binomialsref = $self->_get_species_binomials();
	if (!defined($variation_file)) { $self->throw("No variation GFF file specied"); }
	unless (-f $variation_file) { $self->throw("Variation GFF file, does not exist.  File name: " . $variation_file); }
	my $gffio = Bio::Tools::GFF->new(-file => $variation_file, -gff_version => 2);

	##Parse the variation GFF file
	my %variation_hash;
	while(my $feature = $gffio->next_feature()) {
		if ($feature->seq_id() =~ /REFERENCE-([0-9]+)/) {
			my $id = $1;
			$variation_hash{$id}{'reference'} = $feature;	
		} elsif ($feature->seq_id() =~ /VARIANT-([0-9]+)/) {
			my $id = $1;
			my @array = ();
			if (defined($variation_hash{$1}{'variants'})) {
				@array = @{$variation_hash{$id}{'variants'}};
			}
			push @array, $feature;
			$variation_hash{$id}{'variants'} = \@array;
		} else {
			$self->throw("Variation GFF file, does not match expected format");	
		}
	}
	$gffio->close();
	
	##Build the stacks for the associated Bio::SeqFeature objects in the GFF file
	my @stacks = ();
	for my $key (keys %variation_hash) {
		print STDERR $variation_hash{$key}{'reference'}->seq_id();
		push @stacks, $self->_generate_stack_for_seqfeature($variation_hash{$key}{'reference'}, $variation_hash{$key}{'variants'}, $binomialsref);
	}
	return \@stacks;
}


###############################################################################################
###############################################################################################
## INTERNAL SUBROUTINES
###############################################################################################
###############################################################################################


sub _get_slices {
	my $self = shift;
	my $gene = shift;

        my @retslices = ();
        my $sa = $self->{'_registry'}->get_adaptor("human", "core", "Slice");

        my @exons = @{$gene->get_all_Exons()};

        ## BUILD NON-OVERLAPPING SET OF CODING SEQUENCES, FROM THIS WE WILL DERIVE NON-CODING REGIONS ##
        my @exonsort = sort { $a->start <=> $b->start or $a->end <=> $b->end } @exons;
        my @regions = ();
        my $feature = undef;
        for( my $i = 0; $i < (scalar(@exonsort)); $i++) {
                if (!defined($feature)) {
                        if ($i == (scalar(@exonsort) - 1)) {
                                $feature = Bio::EnsEMBL::Feature->new();
                                $feature->{'start'} = $exonsort[$i]->start();
                                $feature->{'end'} = $exonsort[$i]->end();

                                push @regions, $feature;
                                $feature = undef;
                        } elsif ($exonsort[$i]->end() >= $exonsort[$i+1]->start() && $exonsort[$i]->start() <= $exonsort[$i+1]->end()) {
                                my $start = $exonsort[$i]->start() < $exonsort[$i+1]->start() ? $exonsort[$i]->start() : $exonsort[$i+1]->start();
                                my $end = $exonsort[$i]->end() > $exonsort[$i+1]->end() ? $exonsort[$i]->end() : $exonsort[$i+1]->end();
                                $feature = Bio::EnsEMBL::Feature->new();
                                $feature->{'start'} = $start;
                                $feature->{'end'} = $end;
                        } else {
                                $feature = Bio::EnsEMBL::Feature->new();
                                $feature->{'start'} = $exonsort[$i]->start();
                                $feature->{'end'} = $exonsort[$i]->end();

                                push @regions, $feature;
                                $feature = undef;
                        }
                } else {
                        if ($exonsort[$i]->end() >= $feature->start() && $exonsort[$i]->start() <= $feature->end()) {
                                my $start = $exonsort[$i]->start() < $feature->start() ? $exonsort[$i]->start() : $feature->start();
                                my $end = $exonsort[$i]->end() > $feature->end() ? $exonsort[$i]->end() : $feature->end();
                                $feature = Bio::EnsEMBL::Feature->new();
                                $feature->{'start'} = $start;
                                $feature->{'end'} = $end;
                        } else {
                                push @regions, $feature;
                                $feature = undef;

                                $i--;
                        }

                        if ($i == (scalar(@exonsort) - 1)) {
                                push @regions, $feature;
                                $feature = undef;
                        }
                }
        }

        ##BUILD THE NONCODING SLICE REGIONS FOR THE APPROPRIATE STRAND #####################
        my $start = 0;
        foreach my $feature (@regions) {
                if ($start == 0 && $gene->strand eq "-1") { $start = $feature->start - $DEFAULT_PROMOTER_SIZE_BP; }
                elsif ($start == 0 && $gene->strand eq "1") { $start = $feature->start - $DEFAULT_PROMOTER_SIZE_BP; }
                elsif ($start == 0) { die("Unknown strand " . $gene->strand . "\n")};
                my $slice = $sa->fetch_by_region('chromosome', $gene->slice->seq_region_name, $start, $feature->start - 1, $gene->strand());
                if (!defined($slice)) { die ("Cannot fetch slice " . $gene->slice->seq_region_name . " " . $start . " " . ($feature->start - 1) . " " . $gene->strand() . "\n"); }
                push @retslices, $slice;
                $start = $feature->end + 1;
        }
        if ($gene->strand eq "-1") { push @retslices, $sa->fetch_by_region('chromosome', $gene->slice->seq_region_name, $start, $start + $DEFAULT_PROMOTER_SIZE_BP, $gene->strand()); }
        elsif ($gene->strand eq "1") { push @retslices, $sa->fetch_by_region('chromosome', $gene->slice->seq_region_name, $start, $start + $DEFAULT_PROMOTER_SIZE_BP, $gene->strand()); }
        else { die("Unknown strand " . $gene->strand . "\n")};

        return \@retslices;
}

=head2 _generate_stack_for_variation_feature

 Title	 : _generate_stack_for_variation_feature
 Usage   : my $stack = $stackbuilder->_generate_stack_for_variation_feature($variation_feature, $binomial_hashref, $method);
 Function: For an EnsEMBL VariationFeature, generate a Chum::Stack object
 Returns : a Chum::Stack object
 Args    : [1] Bio::EnsEMBL::Variation::VariationFeature
	   [2] hashref (species binomial)
	   [3] method (String, optional - what method to use in compara for obtaining orthologous sequences)

=cut

sub _generate_stack_for_variation_feature {
	my $self = shift;
	my $variation_feature = shift;
	my $binomialsref = shift;
	my $method = shift;

	my $vtools = new Chum::VariationTools();
	my $slice = $self->_get_slice_for_variation_feature($variation_feature);
	my $reference_sequence = new Bio::LocatableSeq(-seq => $slice->seq, -id => "Homo sapiens", -start => $slice->start, -end => $slice->end, -strand => $slice->strand);
	my ($referenceseq, $variantseqsref) = $vtools->get_variant_sequences_for_variation_feature($variation_feature, $reference_sequence, $self->{'_upstream'}, $self->{'_downstream'});
	if (!defined($variantseqsref) || !defined($referenceseq)) {
		$self->warn("No variant sequence or reference sequence could be create for variation: " . $variation_feature->id);
		return undef; 
	}
	my @variantseqs = @$variantseqsref;

	my $orthologous_sequences_arrayref = $self->_get_orthologous_sequences_for_slice($slice, $binomialsref, $method);
	my @orthologous_sequences = ();
	if (!defined($orthologous_sequences_arrayref)) {
		$self->warn("No orthologues found around variation slice: " . $variation_feature->id);
	} else {
		@orthologous_sequences = @$orthologous_sequences_arrayref;
	}

	my $stackobj = undef;
	if (!$self->{'_memconserve'}) {
		$stackobj = new Chum::Stack(	'reference_sequence' => $referenceseq, 
						'orthologous_sequences' => \@orthologous_sequences, 
						'variant_sequences' => \@variantseqs,
						'variant_ref_obj' => $variation_feature); 
	} else {
		$stackobj = new Chum::Stack(	'reference_sequence' => $referenceseq, 
						'orthologous_sequences' => \@orthologous_sequences, 
						'variant_sequences' => \@variantseqs,
						'variant_ref_obj' => $variation_feature->variation->name); 
	}
	return $stackobj;
}

=head2 _generate_stack_for_seqfeature

 Title	 : _generate_stack_for_seqfeature
 Usage   : my $stack = $stackbuilder->_generate_stack_for_seqfeature($reference_feature, $variation_features_arrayref, $binomial_hashref, $method);
 Function: For Bio::SeqFeature::Generic (reference object and variants arrayref), generate a Chum::Stack object
 Returns : a Chum::Stack object
 Args    : [1] Bio::SeqFeature::Generic (reference allele feature)
	   [2] arrayref (Bio::SeqFeature::Generic variant allele features)
	   [3] hashref (binomial species names)
	   [3] method (String, optional - what method to use in compara for obtaining orthologous sequences)

=cut

sub _generate_stack_for_seqfeature {
	my $self= shift;
	my $reference_feature = shift;
	my $variant_feature_arrayref = shift;
	my $binomialsref = shift;
	my $method = shift;
	
	my $vtools = new Chum::VariationTools();
	my $slice = $self->_get_slice_for_seqfeature($reference_feature);
	my $reference_sequence = new Bio::LocatableSeq(-seq => $slice->seq, -id => "Homo sapiens", -start => $slice->start, -end => $slice->end, -strand => $slice->strand);
	my ($referenceseq, $variantseqsref) = @{$vtools->get_variant_seqs_from_gff($reference_feature, $variant_feature_arrayref, $reference_sequence, $self->{'_upstream'}, $self->{'_downstream'})};
	my @variantseqs = @$variantseqsref;

	my $orthologous_sequences_arrayref = $self->_get_orthologous_sequences_for_slice($slice, $binomialsref, $method);
	my @orthologous_sequences = ();
	if (!defined($orthologous_sequences_arrayref)) {
		$self->warn("No orthologues found around variation slice: " . $slice->seq_region_name . ":" . $slice->start . ":" . $slice->end);
	} else {
		@orthologous_sequences = @$orthologous_sequences_arrayref;
	}
	
	my $stackobj = new Chum::Stack(	'reference_sequence' => $referenceseq, 
					'orthologous_sequences' => \@orthologous_sequences, 
					'variant_sequences' => \@variantseqs,
					'variant_ref_obj' => $reference_feature->seq_id);
	
	return $stackobj;
}

=head2 _get_slice_for_variation_feature

 Title	 : _get_slice_for_variation_feature
 Usage   : my $slice = $stackbuilder->_get_slice_for_variation_feature($variation_feature);
 Function: For an EnsEMBL VariationFeature, get a surrounding genomic slice
 Returns : a Bio::EnsEMBL::Slice feature
 Args    : [1] Bio::EnsEMBL::Variation::VariationFeature

=cut

sub _get_slice_for_variation_feature {
	my $self = shift;
	my $variation_feature = shift;
	$variation_feature = $variation_feature->transform('toplevel');
	my $sa = $self->{'_registry'}->get_adaptor("human", "core", "Slice");
	my $slice = undef;
	
	if ($variation_feature->strand == 1) {
		$slice = $sa->fetch_by_region(	'chromosome',
						$variation_feature->slice->seq_region_name,
						$variation_feature->start - $self->{'_upstream'},
						$variation_feature->end + $self->{'_downstream'},
						$variation_feature->strand);
	} elsif ($variation_feature->strand == -1) {
		$slice = $sa->fetch_by_region(	'chromosome',
						$variation_feature->slice->seq_region_name,
						$variation_feature->start - $self->{'_downstream'},
						$variation_feature->end + $self->{'_upstream'},
						$variation_feature->strand);
	} else {
		$self->throw("Cannot create slice: strand is invalid for variation feature - " . $variation_feature->strand);
	}
	return $slice;
}

=head2 _get_slice_for_seqfeature

 Title	 : _get_slice_for_seqfeature
 Usage   : my $slice = $stackbuilder->_get_slice_for_seqfeature($sequence_feature);
 Function: For an Bio::SeqFeature::Generic, get a surrounding genomic slice
 Returns : a Bio::EnsEMBL::Slice feature
 Args    : [1] Bio::SeqFeature::Generic object

=cut

sub _get_slice_for_seqfeature {
	my $self = shift;
	my $sequence_feature = shift;
	
	if (!defined($sequence_feature)) { $self->throw("No sequence feature defined for obtaining surrounding slice"); }
	elsif (!$sequence_feature->isa("Bio::SeqFeature::Generic")) { $self->throw("Defined sequence_feature is not a Bio::SeqFeature::Generic object"); } 
	
	##DETERMINE CHROMOSOME, (UCSC or ENSEMBL format)
	my $chromosome = undef;
	if ($sequence_feature->source_tag =~ /chr([\S]+)/g) {
		$chromosome = $1;
	} elsif ($sequence_feature->source_tag =~ /([0-9XY]+)/g) {
		$chromosome = $sequence_feature->source_tag;
	} else {
		$self->throw("No chromosome could be found for obtaining slice");
	}
	
	##OBTAIN SLICE FROM ENSEMBL
	my $sa = $self->{'_registry'}->get_adaptor("human", "core", "Slice");
	my $slice = undef;
	if ($sequence_feature->strand == 1) {
		$slice = $sa->fetch_by_region(	'chromosome',
						$chromosome,
						$sequence_feature->start - $self->{'_upstream'},
						$sequence_feature->end + $self->{'_downstream'}, 
						$sequence_feature->strand);
	} elsif ($sequence_feature->strand == -1) {
		$slice = $sa->fetch_by_region(	'chromosome',
						$chromosome,
						$sequence_feature->start - $self->{'_downstream'},
						$sequence_feature->end + $self->{'_upstream'}, 
						$sequence_feature->strand);
	} else {
		$self->throw("Strand is not 1 or -1 for sequence feature, strand: " . $sequence_feature->strand);
	}
	
	return $slice;
}

=head2 _get_orthologous_sequences_for_slice

 Title	 : _get_orthologous_sequences_for_slice
 Usage   : my @sequences = @{$stackbuilder->_get_orthologous_sequences_for_slice($slice, $binomialhashref, 'BLASTZ_NET')};
 Function: For an EnsEMBL Slice, get all the orthologous sequences defined in the compara database for a particular method.
 Returns : an arrayref of Bio::LocatableSeq objects
 Args    : [1] Bio::EnsEMBL::Slice
	   [2] hashref (species binomials)
	   [3] String (method, optional)

=cut

sub _get_orthologous_sequences_for_slice {
	my $self = shift;
	my $slice = shift;
	my $binomialsref = shift;
	my $method = shift;
	my %binomials = %$binomialsref;
	if (!defined($method)) { $method = 'BLASTZ_NET'; }
	
	######################################################CONNECT TO ENSEMBL COMPARA
	my $dafa = $self->{'_registry'}->get_adaptor("compara",'compara','DnaAlignFeature');
	if (!defined($dafa)) { $self->throw("Could not get DNA Align Feature Adaptor\n"); }

	my @orthologous_sequences = ();
	for my $key ( keys %binomials ) {
		my @dna_align_features = @{$dafa->fetch_all_by_Slice($slice, $binomials{$key}, undef, $method)};
		foreach my $dna_align_feature (@dna_align_features) {
			$dna_align_feature = $dna_align_feature->transform('toplevel');
			my $restricted_feature = $dna_align_feature->restrict_between_positions( $slice->start, $slice->end, "SEQ");
			if (!defined($restricted_feature)) {  next; }
			
			#######################################REMOVE GAPS
			my $hitseq = @{$restricted_feature->alignment_strings}[1];
			$hitseq =~ s/-//g; 
			
			######################################CORRECT STRAND
			if ($dna_align_feature->hstrand ne $slice->strand) {
				my $hitseqobj = new Bio::Seq(-id => "tmp", -seq => $hitseq);
				my $revhitseqobj = $hitseqobj->revcom->seq;
				$hitseq = $revhitseqobj;
			}
			
			######################################BUILD HIT SEQUENCE
			my $hitseqobj = new Bio::LocatableSeq(	-seq => $hitseq, 
								-id => $binomials{$key}, 
								-start => $restricted_feature->hstart,
								-end => $restricted_feature->hend,
								-strand => $slice->strand);
			push @orthologous_sequences, $hitseqobj;
		}       
	}	
	return \@orthologous_sequences;
}

=head2 _get_species_binomials

 Title	 : _get_species_binomials
 Usage   : my $binomial_hashref = $stackbuilder->_get_species_binomials();
 Function: Obtain all the binomial species names from EnsEMBL Compara database
 Returns : an hashref of Strings (species binomial names)
 Args    : none
 
=cut

sub _get_species_binomials {
	my $self = shift;
	if (!defined($self->{'binomials'})) {
		my %binomials;
		foreach my $target_species (@{$self->{'_species'}}) {
			my $hca = $self->{'_registry'}->get_adaptor($target_species, "core", "MetaContainer");
			if (!defined($hca)) { die("Could not get $target_species MetaContainer Adaptor\n"); }
			my $hg_binomial = $hca->get_Species->binomial;
			$binomials{$target_species} = $hg_binomial;
		}
		$self->{'binomials'} = \%binomials;
	}
	return $self->{'binomials'};
}

1;
