package Chum::Utils::AlleleFrequencyFinder;

use vars qw(@ISA @AFF_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Bio::Seq;
require Chum::DB::Oreganno;
use Data::Dumper;
use LWP;
require XML::Twig;

@ISA = qw( Bio::Root::Root );

BEGIN {
	@AFF_PARAMS = qw();
	foreach my $attr (@AFF_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
	while (@args) {
		my $attr = shift @args;
		my $value = shift @args;
		$self->$attr($value);
    }
	
    $self->_load_genotype_file();
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub _load_genotype_file() {
	my $self = shift;
	my $file = $ENV{'CHUM_INSTALL_DIR'} . "/external_data/genotype-file.txt";
	if (-f $file) {
		eval {
			my %GENOTYPE_HASH;
			open(GF, $file) || die("Could not open genotype file: " . $file);
			while (my $line = <GF>) {
				chomp($line);
				if ($line =~ /##[\S\s]*/g) {
					#ignore comment
				} elsif ($line =~ /([\S]+)\/\/([\S\s]+)/g) {
					my $id = $1;
					my $allele_line = $2;
					my @allele_tokens = split /:/, $allele_line;
					my $strand = $allele_tokens[scalar(@allele_tokens) - 1];
					my @alleles = ();
					for (my $i = 0; $i < (scalar(@allele_tokens) - 2); $i=$i+2) {
						if ($allele_tokens[$i] =~ /[ACTG]*/gi) {
							if ($allele_tokens[$i+1] =~ /[0-9]*|[0-9]*\.[0-9]+|VR/g) {
								my %allele_frequency_hashref = ('allele' => $allele_tokens[$i],
												'strand' => $strand,
												'frequency' => $allele_tokens[$i+1]);
								push @alleles, \%allele_frequency_hashref;
							} else {
								warn("Not a recognized allele frequency: " . $allele_tokens[$i+1]);
								next;
							}
						} else {
							warn("Not a recognized allele: " . $allele_tokens[$i]);
							next;
						}
					}
					if (scalar(@alleles) > 0) {
						my $population = $allele_tokens[scalar(@allele_tokens) - 2];
						if ("" eq $population) {
							$population = "User defined";	
						}
						my %genotype_population = ( 	'population' => $population,
										'population_location_id' => "User defined",
										'samples' => 100, ##Not supported
										'alleles' => \@alleles);
						$GENOTYPE_HASH{$id} = \%genotype_population;
					}
				} else {
					die("Undefined line: " . $line);	
				}
			}
			close(GF);
			$self->{'_genotype_precache_hashref'} = \%GENOTYPE_HASH;
		}; if ($@) {
			$self->warn($@);
		}
	}
}

sub get_allele_frequencies_by_genotype_population {
	my $self = shift;
	my $stack_object = shift;
	my $database = shift;
	if (!defined($database)) {
		$database = "DBSNP";	
	}
	my $variation_name = $stack_object->get_variation_name;
	print STDERR "Analyzing variation: $variation_name\n";
	if ($variation_name =~ /rs[0-9]+/g || $variation_name =~ /ss[0-9]+/g) {  ##Can use dbSNP ID
		if (uc($database) eq "DBSNP") {
			print STDERR "LOADING FROM DBSNP\n";
			return $self->get_allele_frequencies_by_genotype_population_from_dbsnp_file($variation_name);
			#return $self->get_allele_frequencies_by_genotype_population_from_dbsnp($variation_name);
		#} elsif (uc($database) eq "ENSEMBL") {
		#	print STDERR "LOADING FROM ENSEMBL\n";
		#	return $self->get_allele_frequencies_by_genotype_population_from_ensembl($variation_name);
		} else {
			die("No database selected to get allele frequencies");	
		}
	} elsif ($variation_name =~ /OREG[0-9]+/g) {	##Can use ORegAnno ID
		return $self->get_allele_frequencies_by_genotype_population_from_oreganno($variation_name, $database);
	} else { 
		$self->throw("ERROR: No variation name found, failing\n");
	}
}

sub get_daf_from_orthologue {
	my $self = shift;
	my $stack_object = shift;
	
	###############################################SET UP PARAMETERS
	
	##ANCESTRAL SPECIES TO USE TO CALCULATE DAF
	my $orthogue_id = undef;
	if (@_) {
		$orthogue_id = shift;
	} else {
		$self->throw("No ortholog id specified");
	}
	
	##GENOTYPES/POPULATIONS THE SNP HAS BEEN SAMPLED IN
	my $genotype_populations_arrayref = undef;
	if(@_) {
		$genotype_populations_arrayref = shift;
	}
	if (!defined($genotype_populations_arrayref) || scalar(@$genotype_populations_arrayref) == 0) {
		$genotype_populations_arrayref = $self->get_allele_frequencies_by_genotype_population($stack_object);
		if (!defined($genotype_populations_arrayref) || scalar(@$genotype_populations_arrayref) == 0) {
			$self->throw("SNP is not genotyped, cannot calculate DAF");	
		}
	}
	
	##THE TOTAL GENOTYPE FREQUENCY CALCULATED
	my $total_allele_frequency_hashref = undef;
	if (@_) {
		$total_allele_frequency_hashref = shift;
	}
	if (!defined($total_allele_frequency_hashref) || scalar(@{$total_allele_frequency_hashref->{'alleles'}}) == 0) {
		$total_allele_frequency_hashref = $self->get_total_allele_frequency_from_genotype_populations($genotype_populations_arrayref);
		if (!defined($total_allele_frequency_hashref) || scalar(@{$total_allele_frequency_hashref->{'alleles'}}) == 0) {
			$self->throw("SNP is not genotyped - no totals can be calculated, cannot calculate DAF");	
		}
	}
	
	##DETERMINE IF WE HAVE TO REVERSE COMPLEMENT THE OUTPUT OR NOT
	#my @db_alleles = ();
	#my $rev_com_required = 1;
	#foreach my $allele (@{$total_allele_frequency_hashref->{'alleles'}}) {
	#	if ($allele->{'allele'} =~ /^[ACTGactg\-]+$/g) {
	#		push @db_alleles, $allele;
	#	}
	#}
	#my @variant_sequences = @{$stack_object->variant_sequences};
	#my @ref_alleles = ( lc($stack_object->reference_sequence->allele_sequence->seq),
	#		lc($variant_sequences[0]->allele_sequence->seq));
	#my @rev_ref_alleles = ( lc($stack_object->reference_sequence->allele_sequence->revcom->seq),
	#		    lc($variant_sequences[0]->allele_sequence->revcom->seq));
	#my $ref_match = 0;
	#my $rev_match = 0;
	#foreach my $ref_allele (@ref_alleles) {
	#	foreach my $db_allele (@db_alleles) {
	#		if ($ref_allele eq lc($db_allele->{'allele'})) {
	#			$ref_match++;
	#		}
	#	}
	#}
	#foreach my $rev_ref_allele (@rev_ref_alleles) {
	#	foreach my $db_allele (@db_alleles) {
	#		if ($rev_ref_allele eq lc($db_allele->{'allele'})) {
	#			$rev_match++;
	#		}
	#	}
	# }
	#if ($rev_match > $ref_match) {  ##More of the alleles from the reference/variant seq are matched by rev-com of the obtained db_alleles
	#	$rev_com_required = 0;
	#}
	
	###############################################ACTUAL FUNCTION
	
	##a. Get chimp orthologous sequence in stack object if available, if not throw
	my $orthologue_obj = $stack_object->get_orthologous_sequence_by_id($orthogue_id);
	if (!defined($orthologue_obj)) {
		$self->throw("ERROR: No chimp sequence found");
	} else {
		my $human_obj = $stack_object->reference_sequence->sequence;
		
		##b. Get position of variant
		my $vstart = $stack_object->reference_sequence->start;
		my $vend = $stack_object->reference_sequence->end;
		if (($vend - $vstart) != 0) {
			$self->throw("ERROR: This script can only detect DAF frequencies for SNPs");
		}
		
		##c. Get an alignment of orthologue and human, select site where variant is located.
		require Chum::Utils::ClustalWAligner;
		my $clustalwaligner = new Chum::Utils::ClustalWAligner();
		my $aln_obj = $clustalwaligner->run_align($human_obj, $orthologue_obj);
		
		foreach my $seq ( $aln_obj->each_seq() )  {
			if ($seq->id eq "reference") {
				my $subseq = $seq->subseq(1,$vend+1);
				my $c = $subseq =~ tr/[.]//;
				$vstart += $c;
				$vend += $c;
			}
		}
		
		my $orthologue_allele_sequence = undef;
		my $reference_allele_sequence = undef;
		print STDERR "START: " . $vstart . "\tEND:" . $vend . "\n";
		foreach my $seq ( $aln_obj->each_seq() )  {
			print STDERR $seq->id . ":" . $seq->seq . "\n";
			if ($seq->id eq $orthogue_id) {
				$orthologue_allele_sequence = $seq->subseq($vstart,$vend);	
			}
			if ($seq->id eq "reference") {
				$reference_allele_sequence = $seq->subseq($vstart,$vend);	
			}
		}
		
		print STDERR "Ref seq: " . $reference_allele_sequence . "\tOrtho seq: " . $orthologue_allele_sequence . "\n";
		
		##d. Find DAF, for each population
		if (defined($orthologue_allele_sequence)) {
			if ($orthologue_allele_sequence =~ /[ACTGactg]{1}/g) {
				my $daf = undef;
				$daf->{'ancestral_allele'} = $orthologue_allele_sequence;
				
                                my $ancestral_allele_found = 0;
				my @total_alleles = @{$total_allele_frequency_hashref->{'alleles'}};
				for (my $i = 0; $i < scalar(@total_alleles); $i++) {
					my $allele = $total_alleles[$i]->{'allele'};
					if ($total_alleles[$i]->{'strand'} == -1) {
						my $seq_obj = new Bio::Seq(-id => "foo", -seq => $total_alleles[$i]->{'allele'});
						$allele = uc($seq_obj->revcom->seq);
                                                $total_alleles[$i]->{'allele'} = $allele;
					}
                                        if (uc($allele) ne uc($orthologue_allele_sequence)) {
                                                $ancestral_allele_found = 1;       
                                        }
                                }
                                if ($ancestral_allele_found) {
                                        for (my $i = 0; $i < scalar(@total_alleles); $i++) {
                                                my $allele = $total_alleles[$i]->{'allele'};
                                                if (uc($allele) ne uc($orthologue_allele_sequence) && scalar(@total_alleles) == 2) {
                                                        $daf->{'frequency'}->{'ALL'}->{'allele'} = $total_alleles[$i];
                                                        $daf->{'frequency'}->{'ALL'}->{'samples'} = $total_allele_frequency_hashref->{'samples'};
                                                } 
                                        }
                                }
				
				foreach my $population (@$genotype_populations_arrayref) {
                                        my $ancestral_allele_found = 0;
                                        my @total_alleles = @{$population->{'alleles'}};
                                        for (my $i = 0; $i < scalar(@total_alleles); $i++) {
                                                my $allele = $total_alleles[$i]->{'allele'};
                                                if ($total_alleles[$i]->{'strand'} == -1) {
                                                	my $seq_obj = new Bio::Seq(-id => "foo", -seq => $total_alleles[$i]->{'allele'});
                                                	$allele = uc($seq_obj->revcom->seq);
                                                      $total_alleles[$i]->{'allele'} = $allele;
                                                }
                                                if (uc($allele) ne uc($orthologue_allele_sequence)) {
                                                        $ancestral_allele_found = 1;       
                                                }
                                        }
                                        if ($ancestral_allele_found) {
                                                for (my $i = 0; $i < scalar(@total_alleles); $i++) {
                                                        my $allele = $total_alleles[$i]->{'allele'};
                                                        if (uc($allele) ne uc($orthologue_allele_sequence) && scalar(@total_alleles) == 2) {
                                                                $daf->{'frequency'}->{$population->{'population_location_id'}}->{'allele'} = @total_alleles[$i];
                                                                $daf->{'frequency'}->{$population->{'population_location_id'}}->{'samples'} = $population->{'samples'};
                                                        } 
                                                }
                                        }
				}
				
				if (!defined($daf->{'frequency'})) {
					$self->throw("Could not find an allele with a frequency that matches the orthologous sequence for DAF calculation, is SNP fully genotyped?");		
				} else {
					return $daf;	
				}
			} else {
				$self->throw("Could not find an orthologous site sequence for DAF analysis detected char: " . $orthologue_allele_sequence);		
			}
		} else {
			$self->throw("Could not find an orthologous site sequence for DAF analysis");	
		}
		
		
		##OLD FUNCTION THAT CALCULATES DAF FOR TOTAL ALLELES
		#if (defined($orthologue_allele_sequence)) {
		#	if ($orthologue_allele_sequence =~ /[ACTGactg]{1}/g) {
		#		my $daf = undef;
		#		
		#		my $reference_in_freq = 0;
		#		my @total_alleles = @{$total_allele_frequency_hashref->{'alleles'}};
		#		if (scalar(@total_alleles) == 1) {
		#			$self->throw("Only one allele, likely not a SNP");	
		#		}
		#		foreach my $allele_frequency (@total_alleles) {
		#			print STDERR "Testing " . $allele_frequency->{'allele'} . " against " . $orthologue_allele_sequence . "\n";
		#			if (uc($allele_frequency->{'allele'}) eq uc($orthologue_allele_sequence)) {
		#				$daf = $allele_frequency;
		#			}
		#			if (uc($allele_frequency->{'allele'}) eq uc($reference_allele_sequence)) {
		#				$reference_in_freq = 1;	
		#			}
		#		}
		#		if (!defined($daf) && !$reference_in_freq) {
		#			##try again with rev complement
		#			foreach my $allele_frequency (@{$total_allele_frequency_hashref->{'alleles'}}) {
		#				my $seq_obj = new Bio::Seq(-id => "foo", -seq => $allele_frequency->{'allele'});
		#				my $allele_rev = uc($seq_obj->revcom->seq);
		#				
		#				print STDERR "Testing " . $allele_rev . " against " . $orthologue_allele_sequence . "\n";
		#				if ($allele_rev eq uc($orthologue_allele_sequence)) {
		#					$daf = $allele_frequency;
		#				}
		#			}
		#		}
		#		
		#		if (!defined($daf)) {
		#			$self->throw("Could not find an allele with a frequency that matches the orthologous sequence for DAF calculation, is SNP fully genotyped?");	
		#		} else {
		#			return $daf;	
		#		}
		#	} else {
		#		$self->throw("Could not find an orthologous site sequence for DAF analysis detected char: " . $orthologue_allele_sequence);		
		#	}
		#} else {
		#	$self->throw("Could not find an orthologous site sequence for DAF analysis");	
		#}
	}
}

sub get_total_allele_frequency_from_genotype_populations() {
	my $self = shift;
	my $genotype_populations_arrayref = shift;
	if (!defined($genotype_populations_arrayref)) { die("No genotypes have been entered to get total allele frequencies"); }
	my @genotype_populations = @$genotype_populations_arrayref;
	my %total_allele;
	my $total_population_size = 0;
	my $strand = 1;
	foreach my $genotype_population (@genotype_populations) {
		$total_population_size += $genotype_population->{'samples'};
		foreach my $allele (@{$genotype_population->{'alleles'}}) {
			if (!defined($total_allele{$allele->{'allele'}})) { $total_allele{$allele->{'allele'}} = 0; }
			if ($allele->{'frequency'} ne "VR") {
				$total_allele{$allele->{'allele'}} +=  $allele->{'frequency'} * $genotype_population->{'samples'};
			} else {
				$total_allele{$allele->{'allele'}} += 0; ##do nothing, very rare allele	
			}
			if ($allele->{'strand'} != 1) { ##If one allele is rev for a population, all will be the same here.
				$strand = -1;	
			}
		}
	}
	if ($total_population_size == 0) { die("No samples found in genotype populations"); }
    
	my @alleles = ();
	foreach my $allele (keys %total_allele) {
		my %allele_hashref = (  'allele' => $allele,
					'strand' => $strand,
		                        'frequency' => ($total_allele{$allele} / $total_population_size));
	        push @alleles, \%allele_hashref;
	}
	my %genotype_population = ( 	'population' => "ALL",
		                        'population_location_id' => "ALL",
					'samples' => $total_population_size,
					'alleles' => \@alleles);
	return \%genotype_population;
}

##############################################################
## DATABASE SPECIFIC ROUTINES
##############################################################

#sub _get_root_document_from_dbsnp {
#	my $self = shift;
#	my $dbsnp_id = shift;
#	
#	my ($id) = $dbsnp_id =~ m/rs([0-9]+)$/;
#	if (!defined($id)) { $self->throw("dbSNP ID must be of the format \"rs[0-9]+\""); }
#
#	##a. Fetch EUTILS response
#	my $browser_results = LWP::UserAgent->new;
#	my $url_results = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=snp&id=" . $id . "&report=GENXML";
#	my $response = $browser_results->get( $url_results );
#	print STDERR $response->content;
#	my ($content) = $response->content =~ m/[\s]*([\S]+[\S\s]+[\S]+)[\s]*/;
#	
#	my $root = undef;
#	eval {
#		my $twig = new XML::Twig;
#		$twig->parse($content);
#		$root = $twig->root;
#	}; if ($@) {
#		if ($content =~ /[\S\s]*No[\s]{1}genotype[\s]{1}data[\S\s]*/i) { $self->throw("No genotype data found"); }
#		else {
#			$self->throw($@);   
#		}
#	}
#	if (!defined($root)) {
#		$self->throw("Could not get data from dbSNP at NCBI, check connection");
#	} else {
#		return $root;
#	}
#}

sub get_allele_frequencies_by_genotype_population_from_dbsnp_file() {
	my $self = shift;
	my $dbsnp_id = shift;
	my $root = undef;
	if (@_) {
		$root = shift;	
	}
	if (!defined($root) || scalar(@$root) == 0) {
		print STDERR "INFO: Parsing local file\n";
		my $twig = new XML::Twig;
		$twig->parsefile("/home/smontgom/sNPeGEn/pipeline/11c_build_frequency_files/snp.genotypes.from.dbsnp.txt");
		$root = $twig->root;
	}
	if (!defined($root) || !$root->isa("XML::Twig::Elt")) { die("No root has been defined of type XML::Twig::Elt"); }

	my @genotype_populations = ();
	my %populations;
	foreach my $population ($root->children('Population')) {
		$populations{$population->att('popId')} = $population;
	}
	my $snp_info = undef;
	foreach my $a_snp_info ($root->children('SnpInfo')) {
		my $a_id = lc("rs" . $a_snp_info->att('rsId'));
		if ($a_id eq lc($dbsnp_id)) {
			$snp_info = $a_snp_info;
		}
	}
	if (!defined($snp_info)) {
		$self->throw("No genotype data found");	
	}
	
	my $orientation = "fwd";
	foreach my $ss_loc ($snp_info->children('SnpLoc')) {
		if ($ss_loc->att('genomicAssembly') =~ /[\S]+reference/g) {	
			$orientation = $ss_loc->att('rsOrientToChrom');
		}
	}
	print STDERR "Using orientation: " . $orientation . "\n";
	
	foreach my $ss_info ($snp_info->children('SsInfo')) {
		my $orientationToRs = $ss_info->att('ssOrientToRs');
		foreach my $bypop ($ss_info->children('ByPop')) {
			my $population_id = $bypop->att('popId');
			my $sample_size = $bypop->att('sampleSize');
			my @alleles = ();
			foreach my $allele_frequency ($bypop->children('AlleleFreq')) {
				my $allele = $allele_frequency->att('allele');
				if ($orientationToRs eq "rev") {
					if ($allele =~ /^([ACTGactg\-]+)$/g) {
						my $seq = new Bio::Seq(-seq => $allele, -id => "allele");
						$allele = $seq->revcom->seq;
					}	
				}
				my $frequency = $allele_frequency->att('freq');
				my $strand = 1;
				if ($orientation eq "rev") {
					$strand = -1;
				}
				my %allele_frequency_hashref = ('allele' => $allele,
								'strand' => $strand,
				                                'frequency' => $frequency);
				push @alleles, \%allele_frequency_hashref;
			}
			my %genotype_population = ( 'population' => $population_id,
			                            'population_location_id' => $populations{$population_id}->att('locPopId'),
			                            'samples' => $sample_size,
			                            'alleles' => \@alleles);
			push @genotype_populations, \%genotype_population;
		}
	}
	return \@genotype_populations;
	
}

#sub get_allele_frequencies_by_genotype_population_from_dbsnp() {
#	my $self = shift;
#	my $dbsnp_id = shift;
#	my $root = undef;
#	if (@_) {
#		$root = shift;	
#	}
#	if (!defined($root) || scalar(@$root) == 0) {
#		$root = $self->_get_root_document_from_dbsnp($dbsnp_id);
#	}
#	if (!defined($root) || !$root->isa("XML::Twig::Elt")) { die("No root has been defined of type XML::Twig::Elt"); }
#	
#	print STDERR "Trying to get Allele Frequency from dbSNP: $dbsnp_id\n";
#
#	my @genotype_populations = ();
#	my %populations;
#	foreach my $population ($root->children('Population')) {
#		$populations{$population->att('popId')} = $population;
#	}
#	
#	my $snp_info = $root->first_child('SnpInfo');
#	foreach my $ss_info ($snp_info->children('SsInfo')) {
#		my $orientation = $ss_info->att('ssOrientToRs');
#		foreach my $bypop ($ss_info->children('ByPop')) {
#			my $population_id = $bypop->att('popId');
#			my $sample_size = $bypop->att('sampleSize');
#			my @alleles = ();
#			foreach my $allele_frequency ($bypop->children('AlleleFreq')) {
#				my $allele = $allele_frequency->att('allele');
#				my $frequency = $allele_frequency->att('freq');
#				if ($orientation eq "rev") {
#					if ($allele =~ /^([ACTGactg\-]+)$/g) {
#						my $seq = new Bio::Seq(-seq => $allele, -id => "allele");
#						$allele = $seq->revcom->seq;
#					}		 
#				}
#				my %allele_frequency_hashref = ('allele' => $allele,
#				                                'frequency' => $frequency);
#				push @alleles, \%allele_frequency_hashref;
#			}
#			my %genotype_population = ( 'population' => $population_id,
#			                            'population_location_id' => $populations{$population_id}->att('locPopId'),
#			                            'samples' => $sample_size,
#			                            'alleles' => \@alleles);
#			push @genotype_populations, \%genotype_population;
#		}
#	}
#	return \@genotype_populations;
#}

sub get_allele_frequencies_by_genotype_population_from_oreganno {
	my $self = shift;
	my $variation_name = shift;
	my $database = shift;

	print STDERR "Trying to get Allele Frequency from ORegAnno\n";
	my $genotype_populations_arrayref = undef;
	eval {
		my $oreganno = new Chum::DB::Oreganno();
		my $record = $oreganno->get_record_from_id($variation_name);
		if (!defined($record)) {
			$self->throw("No record defined\n");
		} else {
			my @variations = @{$record->{'record'}->{'variationSet'}->{'variation'}};
			if (scalar(@variations) == 1) {
				my $variation = $variations[0];
				if (defined($variation->{'regulatoryVariationCrossReferenceSet'})) {
					my @rvxrefs = @{$variation->{'regulatoryVariationCrossReferenceSet'}->{'regulatoryVariationCrossReference'}};
					if (scalar(@rvxrefs) == 1) {
						my $rvxref = $rvxrefs[0];
						if ($rvxref->{'source'} eq 'dbSNP') {
							if (defined($rvxref->{'snpId'})) {
								if (uc($database) eq "DBSNP") { 
									$genotype_populations_arrayref =  $self->get_allele_frequencies_by_genotype_population_from_dbsnp_file($rvxref->{'snpId'});
								} elsif (uc($database) eq "ENSEMBL") {
									$genotype_populations_arrayref =  $self->get_allele_frequencies_by_genotype_population_from_ensembl($rvxref->{'snpId'});
								} else {
									die("No database selected to get allele frequencies");	
								}
								if (!defined($genotype_populations_arrayref)) {
									$self->throw("No genotypes defined for: " . $variation_name);
								}
							}
						}
					}
				}
			} else {
				$self->throw("No dbSNP id defined for ORegAnno, must try precache, precache has failed");
			}
		}
	};
	if ($@ || !defined($genotype_populations_arrayref)) {
		if (uc($database) ne "ENSEMBL") {
			print STDERR "Trying to get Allele Frequency from precache data\n";
			if (defined($self->{'_genotype_precache_hashref'}{$variation_name})) {
				my @genotype_populations = ();
				push @genotype_populations, $self->{'_genotype_precache_hashref'}{$variation_name};
				return \@genotype_populations;
			} else {
				print STDERR "FAILED\n";
				$self->throw("Threw the following exception... " . $@);	
			}
		}
	}
	return $genotype_populations_arrayref;
}

#sub get_allele_frequencies_by_genotype_population_from_ensembl() {
#	my $self = shift;
#	my $dbsnp_id = shift;
#	my @genotype_populations = ();
#
#	if (!defined($self->{'va'})) {
#		$self->{'va'} = Bio::EnsEMBL::Registry->get_adaptor("human", "variation", "Variation");
#	}
#	my $variation = $self->{'va'}->fetch_by_name($dbsnp_id,'dbSNP');
#	my $pgref = $variation->get_all_PopulationGenotypes();
#	if (defined($pgref)) {
#		my @pgs = @$pgref;
#		my %population;
#		foreach my $pg (@pgs) {
#			my $population_name = $pg->population()->name();
#			my $allele1 = uc($pg->allele1());
#			my $allele2 = uc($pg->allele2());
#			if (!defined($population{$population_name}{$allele1})) {
#				$population{$population_name}{$allele1} = 0;
#			}
#			if (!defined($population{$population_name}{$allele2})) {
#				$population{$population_name}{$allele2} = 0;
#			}
#			$population{$population_name}{$allele1} += ($pg->frequency() / 2);
#			$population{$population_name}{$allele2} += ($pg->frequency() / 2);
#		}
#		
#		foreach my $pop (keys %population) {
#			my $size = 0;
#			if ($pop eq "CSHL-HAPMAP:HapMap-CEU") {
#				$size = 60
#			} elsif ($pop eq "CSHL-HAPMAP:HapMap-HCB") {
#				$size = 45
#			} elsif ($pop eq "CSHL-HAPMAP:HapMap-JPT") {
#				$size = 44;
#			} elsif ($pop eq "CSHL-HAPMAP:HapMap-YRI") {
#				$size = 60;
#			} elsif ($pop eq "PERLEGEN:AFD_CHN_PANEL") {
#				$size = 24;
#			} elsif ($pop eq "PERLEGEN:AFD_AFR_PANEL") {
#				$size = 23;
#			} elsif ($pop eq "PERLEGEN:AFD_EUR_PANEL") {
#				$size = 24;
#			} else {
#				warn "Unsupported population " . $pop . " for " . $dbsnp_id;
#			}
#			my $pop_hash = $population{$pop};
#			my @alleles = keys %$pop_hash;
#			
#			my %genotype_population = ( 'population' => $pop,
#			                            'population_location_id' => undef,
#			                            'samples' => $size,
#			                            'alleles' => \@alleles);
#			push @genotype_populations, \%genotype_population;
#		}
#	}
#	return \@genotype_populations;
#}

1;
