package Chum::Utils::GeneFinder;

use vars qw(@ISA @GF_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::EnsEMBL::Registry;
use Bio::Root::Root;
use Chum::DB::Oreganno;
use Chum::Utils::GenomeCoordinateLocator;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );

BEGIN {
	@GF_PARAMS = qw();
	foreach my $attr (@GF_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
	while (@args) {
		my $attr = shift @args;
		my $value = shift @args;
		$self->$attr($value);
    }
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

##within 200kb
sub get_ensembl_transcript_distances_to_stack {
	my $self = shift;
	my $stack_object = shift;
	my $variation_name = $stack_object->get_variation_name;
	print STDERR "Analyzing variation: $variation_name\n";
	if ($variation_name =~ /rs[0-9]+/g || $variation_name =~ /ss[0-9]+/g) {  ##Can use dbSNP ID
		return $self->_get_et_from_dbsnp($stack_object, $variation_name);
	} elsif ($variation_name =~ /OREG[0-9]+/g) {	##Can use ORegAnno ID
		return $self->_get_et_from_oreganno($stack_object, $variation_name);
	} else { ##No ID, must try mapping to the genome using blast
		return $self->_get_et_from_sequence($stack_object, $variation_name);
	}
}

########PRIVATE SUBROUTINES

sub _get_et_from_dbsnp() {
	my $self = shift;
	my $stack_object = shift;
	my $variation_name = shift;
	
	my $va = Bio::EnsEMBL::Registry->get_adaptor("human", "variation", "Variation");
	my $vfa = Bio::EnsEMBL::Registry->get_adaptor("human", "variation", "VariationFeature");
	
	my $variation = $va->fetch_by_name($variation_name,'dbSNP');
	if (!defined($variation)) {
		return $self->_get_et_from_sequence($stack_object, $variation_name);
	} else {
		my @variation_features = @{$vfa->fetch_all_by_Variation($variation)};
		if (scalar(@variation_features) != 1) {
			return $self->_get_et_from_sequence($stack_object, $variation_name);
		} else {
			my $variation_feature = @variation_features[0];
			$variation_feature = $variation_feature->transform('chromosome');
			return $self->_get_transcript_distances(	$variation_feature->seq_region_name, 
														$variation_feature->start, 
														$variation_feature->end, 
														$variation_feature->strand);
		}
	}
}

sub _get_et_from_oreganno() {
	my $self = shift;
	my $stack_object = shift;
	my $variation_name = shift;

	my $oreganno = new Chum::DB::Oreganno();
	my $record = $oreganno->get_record_from_id($variation_name);
	if (!defined($record)) {
		return $self->_get_et_from_sequence($stack_object, $variation_name);
	} else {
		#Doesn't provide distance metric, removed
		#my $transcript_name = $self->_get_et_from_gene_name($record->{'record'}->{'geneName'});
		#if (defined($transcript_name)) {
		#	my @transcripts = ();
		#	push @transcripts, $transcript_name;
		#	return \@transcripts;
		#}

		my @variations = @{$record->{'record'}->{'variationSet'}->{'variation'}};
		if (scalar(@variations) == 1) {
			my $variation = $variations[0];
			##Load from dbSNP, if available
			if (defined($variation->{'regulatoryVariationCrossReferenceSet'})) {
				my @rvxrefs = @{$variation->{'regulatoryVariationCrossReferenceSet'}->{'regulatoryVariationCrossReference'}};
				if (scalar(@rvxrefs) == 1) {
					my $rvxref = $rvxrefs[0];
					if ($rvxref->{'source'} eq 'dbSNP') {
						if (defined($rvxref->{'snpId'})) {
							return $self->_get_et_from_dbsnp($stack_object, $rvxref->{'snpId'});
						}
					}
				}
			}
		}
		return $self->_get_et_from_sequence($stack_object, $variation_name);
	}
}

sub _get_et_from_gene_name() {
	my $self = shift;
	my $gene_name = shift;
		
	my $ta = Bio::EnsEMBL::Registry->get_adaptor("human", "core", "Transcript");

	my @transcripts = @{$ta->fetch_all_by_external_name($gene_name)};
	if (scalar(@transcripts) == 1) {
		return $transcripts[0];
	} else {
		return undef;
	}
}

sub _get_et_from_sequence() {
	my $self = shift;
	my $stack_object = shift;
	my $variation_name = shift;

	my $blastdbdir = $ENV{'BLASTDB'} . "/";
	my $gcl = new Chum::Utils::GenomeCoordinateLocator('blast_db' => $blastdbdir . "hg17.fa"); ##Should be set from config
	my $reference_sequence = $stack_object->reference_sequence;
	my $variation_size = length($reference_sequence->allele_sequence->seq);
	
	##Get at least an 50mer to map
	my $nmersize = int((50 - $variation_size + 1) / 2);
	my $start_index = $reference_sequence->start - $nmersize;
	if ($start_index < 0) {
		$start_index = 0;
	}

	my $end_index = $reference_sequence->end + $nmersize;
	if ($end_index > length($reference_sequence->sequence->seq)) {
		$end_index = length($reference_sequence->sequence->seq);
	}

	print STDERR "Start : $start_index End: $end_index\n";

	my $subsequence = substr($reference_sequence->sequence->seq, $start_index, ($end_index - $start_index));
	print STDERR $subsequence . "\n";

	my $hits_ref = $gcl->locate_sequence($variation_name, $subsequence);
	if (defined($hits_ref)) {
		my @hits = @$hits_ref;
		if (scalar(@hits) == 0) {
			return undef;
		}
		return $self->_get_transcript_distances(	$hits[0]->{'chromosome'}, 
													$hits[0]->{'start'}, 
													$hits[0]->{'end'}, 
													$hits[0]->{'strand'});
	} else {
		return undef;
	}
}

sub _get_transcript_distances() {
	my $self = shift;
	my $seq_region_name = shift;
	my $start = shift;
	my $end = shift;
	my $strand = shift;

	my $sa = Bio::EnsEMBL::Registry->get_adaptor("human", "core", "Slice");
	my $ta = Bio::EnsEMBL::Registry->get_adaptor("human", "core", "Transcript");
	my $ga = Bio::EnsEMBL::Registry->get_adaptor("human", "core", "Gene");
	print $seq_region_name . " " .$start . " " . $end . "\n";
	my $slice = $sa->fetch_by_region('toplevel', $seq_region_name, $start - 100000, $end + 100000, "1"); ##Look in 200kb for genes
	my @transcripts = @{$ta->fetch_all_by_Slice($slice)};
	my $midpoint = int(($start + $end) / 2);
	my %transcript_distance_hash;
	foreach my $transcript (@transcripts) {
		$transcript = $transcript->transform('chromosome');
		my $start_distance = abs($transcript->start - $midpoint);
		my $end_distance = abs($transcript->end - $midpoint);
		my $external_name = $transcript->external_name;
		if (!defined($external_name)) {$external_name = "unknown";}
		
		my $gene = $ga->fetch_by_transcript_stable_id($transcript->stable_id);
		my $genename;
		if (defined($gene)) {
			$genename = $gene->stable_id;
		}

		if ($start_distance < $end_distance) {
			%transcript_distance_hash->{$genename . ":" . $transcript->stable_id . ":" . $external_name} = $start_distance;
		} else {
			%transcript_distance_hash->{$genename . ":" . $transcript->stable_id . ":" . $external_name} = $end_distance;
		}	
	}
	return \%transcript_distance_hash;
}

1;
