#
# Chum::Utils::RSNPAnalysisXMLFileReader
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Utils::RSNPAnalysisXMLFileReader - Loads in a *.rsnpa.xml file from
an external analysis run.

=head1 SYNOPSIS

=head1 DESCRIPTION

Loads in a *.rsnpa.xml file from an external analysis run.  These files have the format:
<regulatory-polymorphism-analysis-file>
	<id>id from dbSNP or ORegAnno</id>
	<seqstack>
		<sequence name="rhma">AATCTTTGTTGCTTCTTCTAAACCT</sequence>
		<sequence name="patr">AATCTTCGTTGCTTCTTCTAAACCT</sequence>
		.....	
	</seqstack>
	<variants>
		<left-sequence>GTTG</left-sequence>
		<right-sequence>TTCT</right-sequence>
		<variant>
			<variant-sequence>AATCTTTGTTGATTCTTCTAAACCT</variant-sequence>
			<allele-sequence>A</allele-sequence>
		</variant>
	</variants>
</regulatory-polymorphism-analysis-file>

Usage:
my $rs2749024 = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => "rs2749024.snpa.xml");


=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut

package Chum::Utils::RSNPAnalysisXMLFileReader;

use strict;
use Bio::Seq;
use Data::Dumper;
use Chum::Stack;
use Chum::VariantSequence;
use XML::Twig;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw();

sub new {
    my $class = shift;
    my %args = (-file => undef, @_);
    my $self = bless {}, ref($class) || $class;
    if ($args{-file}) {
        $self->_load_file($args{-file});
    } else {
        $self->_warn_not_loaded();    
    }
    return $self;
}

sub _check_loaded {
    my $self = shift;
    if($self->analysis_file()) {
        return 1;
    } else {
        return 0;    
    }
}

sub _warn_not_loaded {
    my $self = shift;
    warn("No file has been loaded into RSNPAnalysisXMLFileReader object");
}

=head2 _load_file

 Usage   : $rsnpafile->_load_file; 
			##NOTE: Private function, loads the analysis file into this object
 Returns : nothing
 Args    : $analysis_file	

=cut

sub _load_file {
    my $self = shift;
    my $analysis_file = shift;
    if ($self->_check_loaded) { warn "Analysis file already loaded\n"; return 0; }
    my $twig = XML::Twig->new();
    $twig->parsefile($analysis_file);
    my $root = $twig->root;
    $self->{'id'} = $root->first_child_text('id');
    my %sequences;
    foreach my $sequence_element ($root->first_child('seqstack')->children('sequence')) {
        my $name = $sequence_element->att('name');
        my $sequence = $sequence_element->text;
        if (defined $sequences{$name}) {
            warn "Non-unique sequence name attribute found: " . $name .
                 " writing over existing value and continuing";
        }
        $sequences{$name} = $sequence;
    }
    $self->{'sequences'} = \%sequences;
    my $variants_element = $root->first_child('variants');
    $self->{'leftsequence'} = $variants_element->first_child_text('left-sequence');
    $self->{'rightsequence'} = $variants_element->first_child_text('right-sequence');
    my @variants = ();
    foreach my $variant_element ($variants_element->children('variant')) {
        my $variant_sequence = $variant_element->first_child_text('variant-sequence');
        my $allele_sequence = $variant_element->first_child_text('allele-sequence');
		if ($allele_sequence =~ /^[ATCGN\-]+/gi) { ##Ignore non DNA variants
			push @variants, {'variantsequence' => $variant_sequence, 'allelesequence' => $allele_sequence};
		} else {
			warn("Ignoring variant: $allele_sequence");
		}
    }
    $self->{'variants'} = \@variants;
    $self->{'analysisfile'} = $analysis_file;
    return 0;
}

=head2 analysis_file

 Usage   : $rsnpafile->analysis_file
 Returns : string containing location of analysis file
 Args    : none

=cut

sub analysis_file {
    my $self = shift;
    return $self->{'analysisfile'};
}

=head2 sequences_map

 Usage   : $rsnpafile->sequences_map; #Returns hashref
		   $rsnpafile->sequences_map('Homo sapiens'); #Returns string containing sequence

 Returns : either a hashref of the sequences loaded or a specific sequence if arg 1 is specified
 Args    : id sequence_name, string (optional)

=cut

sub sequence_map {
    my $self = shift;
    my $id = shift;
    
    if (!defined($id)) {
        my %hash_copy = %{$self->{'sequences'}};
        return %hash_copy;
    }
    else {
        my %hash_copy = %{$self->{'sequences'}};
        if (!defined($hash_copy{$id})) { warn "$id is not a valid id\n"; }
        else {
            return $hash_copy{$id};    
        }
    }
}

=head2 sequence_arr

 Usage   : $rsnpafile->sequence_arr; #Returns arrayref
 Returns : an arrayref of Bio::Seq objects for the sequences defined in the xml file
 Args    : none

=cut

sub sequence_arr {
	my $self = shift;
	my @sequences = ();
	my %hash_copy = %{$self->{'sequences'}};
	for my $key (keys %hash_copy) {
		push @sequences, new Bio::Seq (	-id => $key,
										-seq => %hash_copy->{$key})

	}
	return \@sequences;
}

=head2 sequence_ids

 Usage   : $rsnpafile->sequence_ids; #Returns arrayref
 Returns : an arrayref of sequence ids for the sequences in the input xml file
 Args    : none

=cut

sub sequence_ids {
    my $self = shift;    
    return keys %{$self->{'sequences'}};
}

=head2 left_flank_sequence

 Usage   : $rsnpafile->left_flank_sequence;
 Returns : an string containing a sequence that 5' flanks the variant
 Args    : none

=cut

sub left_flank_sequence {
    my $self = shift;
    return $self->{'leftsequence'};
}

=head2 right_flank_sequence

 Usage   : $rsnpafile->right_flank_sequence;
 Returns : an string containing a sequence that 3' flanks the variant
 Args    : none

=cut

sub right_flank_sequence {
    my $self = shift;
    return $self->{'rightsequence'};
}

=head2 variants

 Usage   : $rsnpafile->variants; #Returns arrayref
 Returns : an arrayref of hashes containing variant sequences and alleles from the input xml file
 Args    : none

=cut

sub variants {
    my $self = shift;
    return $self->{'variants'};
}

=head2 get_flanked_allele_sequences

 Usage   : $rsnpafile->get_flanked_allele_sequences; #Returns arrayref
 Returns : an arrayref of hashes containing sequences with specified flank around them,
			this is for shortening the length of the reference and variant sequences for more detailed analyses.
			For instance, using 10bp flank around a variant instead of 500bp.
 Args    : $flank (positive integer)
		   $reference_id (string) name of a sequence	

=cut

sub get_flanked_allele_sequences {
    my $self = shift;
    my $flank = shift;
	my $reference_id = shift;

    if (!defined($self->sequence_map($reference_id))) {
		warn "Invalid reference id used"; return 0; 
    }
    if (!defined($flank) || $flank <= 0) { warn "Positive-valued flank must be specified"; return 0; }
    else {
        my $left_index = index(lc($self->sequence_map($reference_id)), lc($self->left_flank_sequence)) + length($self->left_flank_sequence);
        my $right_index = index(lc($self->sequence_map($reference_id)), lc($self->right_flank_sequence));
        if ($left_index < 0 || $right_index < 0) { warn "Cannot find flank sequences in $reference_id"; return 0; }
        else {
            my $left_flank_index = $left_index - $flank;
            if ($left_flank_index < 0) { warn "Flank is too large, shortening"; $left_flank_index = 0; }
            my $left_sequence = substr(lc($self->sequence_map($reference_id)), $left_flank_index, $flank);
            my $right_sequence = substr(lc($self->sequence_map($reference_id)), $right_index , $flank);
            my $reference_sequence = substr(lc($self->sequence_map($reference_id)), $left_flank_index, $flank + $flank +($right_index - $left_index));
			my @flanked_allele_sequences = ();
            push @flanked_allele_sequences, {	'type' => 'reference', 
												'sequence' => $reference_sequence,
												'leftseq' => $left_sequence,
												'rightseq' => $right_sequence,
												'allele' => $self->_get_reference_allele($reference_id)};
            foreach my $variant (@{$self->{'variants'}}) {
                push @flanked_allele_sequences, {'type' => 'variant', 
												 'sequence' => lc($left_sequence) . lc($variant->{'allelesequence'}) . lc($right_sequence), 
												 'leftseq' => $left_sequence,
												 'rightseq' => $right_sequence,
												 'allele' => $variant->{'allelesequence'}};
            }
            return \@flanked_allele_sequences;
        }
    }
}

=head2 get_flanked_sequence_stack_obj

 Usage   : $rsnpafile->get_flanked_sequence_stack_obj; 
			this is for shortening the length of the reference and variant sequences for more detailed analyses.
			For instance, using 10bp flank around a variant instead of 500bp.
 Returns : a Chum::Stack object for input into analysis modules
 Args    : $flank (positive integer)
		   $reference_id (string) name of a sequence	

=cut

sub get_flanked_sequence_stack_obj {
	my $self = shift;
	my $flank = shift;
	my $reference_id = shift;
	my $flanked_allele_sequences_arrayref = $self->get_flanked_allele_sequences($flank, $reference_id);
	if (defined ($flanked_allele_sequences_arrayref)) {
		my @flanked_allele_sequences = @$flanked_allele_sequences_arrayref;
		my $reference_sequence;
		my @variant_sequences = ();
		foreach my $flanked_allele_sequence (@flanked_allele_sequences) {
			if ($flanked_allele_sequence->{'type'} eq 'reference') {
				$reference_sequence = new Chum::VariantSequence(	'id' => $reference_id,
																	'sequence' => new Bio::Seq(-id => "reference", -seq => $flanked_allele_sequence->{'sequence'}),
																	'allele_left_flank_sequence' => new Bio::Seq(-id => "left_flank", -seq => $flanked_allele_sequence->{'leftseq'}),
																	'allele_right_flank_sequence' => new Bio::Seq(-id => "right_flank", -seq => $flanked_allele_sequence->{'rightseq'}),
																	'allele_sequence' => new Bio::Seq(-id => "allele", -seq => $flanked_allele_sequence->{'allele'}));
			} elsif ($flanked_allele_sequence->{'type'} eq 'variant') {
				push @variant_sequences, new Chum::VariantSequence(	'id' => $reference_id,
																		'sequence' => new Bio::Seq(-id => "variant", -seq => $flanked_allele_sequence->{'sequence'}),
																		'allele_left_flank_sequence' => new Bio::Seq(-id => "left_flank", -seq => $flanked_allele_sequence->{'leftseq'}),
																		'allele_right_flank_sequence' => new Bio::Seq(-id => "right_flank", -seq => $flanked_allele_sequence->{'rightseq'}),
																		'allele_sequence' => new Bio::Seq(-id => "allele", -seq => $flanked_allele_sequence->{'allele'}));
			} else {
				warn("Unknown sequence type: " . $flanked_allele_sequence->{'type'});
			}
		}
		my @orthologous_sequences = ();
		foreach my $sequence (@{$self->sequence_arr}) {
			if ($sequence->id ne $reference_id) {
				push @orthologous_sequences, $sequence;
			}
		}
		return new Chum::Stack(	'reference_sequence' => $reference_sequence,
									'orthologous_sequences' => \@orthologous_sequences,
									'variant_sequences' => \@variant_sequences,
									'variation_ref_obj' => $self->{'id'});
	} else {
		return undef;
	}
}

=head2 get_sequence_stack_obj

 Usage   : $rsnpafile->get_sequence_stack_obj; 
 Returns : a Chum::Stack object for input into analysis modules
 Args    : $reference_id (string) name of a sequence	

=cut

sub get_sequence_stack_obj {
	my $self = shift;
	my $reference_id = shift;
	my $reference_sequence;
	my @orthologous_sequences = ();
	foreach my $sequence (@{$self->sequence_arr}) {
		if ($sequence->id eq $reference_id) {
			$reference_sequence = new Chum::VariantSequence(	'id' => $sequence->id,
																'sequence' => new Bio::Seq(-id => "reference", -seq => $sequence->seq),
																'allele_left_flank_sequence' => new Bio::Seq(-id => "left_flank", -seq => $self->left_flank_sequence),
																'allele_right_flank_sequence' => new Bio::Seq(-id => "right_flank", -seq => $self->right_flank_sequence),
																'allele_sequence' => new Bio::Seq(-id => "allele", -seq => $self->_get_reference_allele($reference_id)));
		} else {
			push @orthologous_sequences, $sequence;
		}
	}
	my @variant_sequences;
	foreach my $variant_sequence (@{$self->variants}) {
		push @variant_sequences, new Chum::VariantSequence(	'id' => $reference_id,
																'sequence' => new Bio::Seq(-id => "variant", -seq => $variant_sequence->{'variantsequence'}),
																'allele_left_flank_sequence' => new Bio::Seq(-id => "left_flank", -seq => $self->left_flank_sequence),
																'allele_right_flank_sequence' => new Bio::Seq(-id => "right_flank", -seq => $self->right_flank_sequence),
																'allele_sequence' => new Bio::Seq(-id => "allele", -seq => $variant_sequence->{'allelesequence'}));
	}

	return new Chum::Stack(	'reference_sequence' => $reference_sequence,
								'orthologous_sequences' => \@orthologous_sequences,
								'variant_sequences' => \@variant_sequences,
								'variation_ref_obj' => $self->{'id'});
}

=head2 _get_reference_allele

 Usage   : $rsnpafile->_get_reference_allele; 
			##NOTE: Private function
 Returns : the allele in the reference sequence
 Args    : $reference_id (string) name of a sequence	

=cut

sub _get_reference_allele {
	my $self = shift;
	my $reference_id = shift;
	my $left_index = index(lc($self->sequence_map($reference_id)), lc($self->left_flank_sequence)) + length($self->left_flank_sequence);
	my $right_index = index(lc($self->sequence_map($reference_id)), lc($self->right_flank_sequence));
	if ($left_index < 0 || $right_index < 0) { die "Error: Cannot find flank sequences in $reference_id"; }
	my $allele = substr($self->sequence_map($reference_id), $left_index, ($right_index - $left_index));
	#print STDERR $allele . " " . $self->right_flank_sequence . " " . $self->left_flank_sequence . " " . $right_index . " " .  $left_index . "\n";
	return $allele;
}

1;