package Chum::Utils::RepeatContentFinder;

use vars qw(@ISA @RCF_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::EnsEMBL::Registry;
use Bio::Root::Root;
use Chum::Utils::GenomeCoordinateLocator;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );

BEGIN {
	@RCF_PARAMS = qw();
	foreach my $attr (@RCF_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
	while (@args) {
		my $attr = shift @args;
		my $value = shift @args;
		$self->$attr($value);
    }
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub get_repeat_content_chr_start_end {
	my $self = shift;
	my $chromosome = shift;
	my $start = shift;
	my $end = shift;
	my $flank = shift;
	
	return $self->_get_repeat_content_for_chr_start_end(  	$chromosome,
								$start,
								$end,
								"0",
								$flank );
	
}

sub get_repeat_content {
	my $self = shift;
	my $stack_object = shift;
	my $flank = shift;

	my $blastdbdir = $ENV{'BLASTDB'} . "/";
	my $gcl = new Chum::Utils::GenomeCoordinateLocator('blast_db' => $blastdbdir . "hg17.fa"); ##Should be set from config
	my $reference_sequence = $stack_object->reference_sequence;
	my $variation_size = length($reference_sequence->allele_sequence->seq);
	
	##Get at least an 50mer to map
	my $nmersize = int((50 - $variation_size + 1) / 2);
	my $start_index = $reference_sequence->start - $nmersize;
	if ($start_index < 0) {
		$start_index = 0;
	}

	my $end_index = $reference_sequence->end + $nmersize;
	if ($end_index > length($reference_sequence->sequence->seq)) {
		$end_index = length($reference_sequence->sequence->seq);
	}

	my $vstart = $reference_sequence->start - $start_index;
	my $vend = $reference_sequence->end - $start_index;

	print STDERR "Start : $start_index End: $end_index\n";

	my $subsequence = substr($reference_sequence->sequence->seq, $start_index, ($end_index - $start_index));
	print STDERR $subsequence . "\n";

	my $hits_ref = $gcl->locate_sequence($stack_object->get_variation_name, $subsequence);
	if (defined($hits_ref)) {
		my @hits = @$hits_ref;
		if (scalar(@hits) == 0) {
			$self->throw("Could not get repeat content, could not find coordinates");
		}
		return $self->_get_repeat_content_for_hit_and_variant(	$hits[0],
									$flank,
									$vstart,
									$vend);

	} else {
		$self->throw("Could not get repeat content, could not find coordinates");
	}
}


################################ PRIVATE SUBROUTINES

sub _get_repeat_content_for_hit_and_variant() {
	my $self = shift;
	my $hit = shift;
	my $flank = shift;
	my $vstart = shift;
	my $vend = shift;
	
	print STDERR "V: " . $vstart . "\t" . $vend . "\n";
	my $truestart = $hit->{'start'} < $hit->{'end'} ? $hit->{'start'} : $hit->{'end'};
	print STDERR "TSTART: " . $truestart . "\n";
	return $self->_get_repeat_content_for_chr_start_end(  	$hit->{'chromosome'},
								$truestart + $vstart,
								$truestart + $vend,
								$hit->{'strand'},
								$flank );
	
}

sub _get_repeat_content_for_chr_start_end {
	my $self = shift;
	my $seq_region_name = shift;
	my $start = shift;
	my $end = shift;
	my $strand = shift;
	my $flank = shift;

	print STDERR "S: " . $start . "\t" . $end . "\t" . $strand . "\n";
	print STDERR "FLANK: " . $flank . "\n";

	my $sa = Bio::EnsEMBL::Registry->get_adaptor("human", "core", "Slice");
	print STDERR $seq_region_name . " " .$start . " " . $end . "\n";
	my $slice = $sa->fetch_by_region('toplevel', $seq_region_name, $start - $flank, $end + $flank, "1"); ##Look in flank for repeats
	my $length = $end - $start + 1 + (2 * $flank);
	my $repeats_ref = $slice->get_all_RepeatFeatures();
	my %repeat_type_length_hash;
	my %repeat_events_long;
	my %repeat_events_short;
	my $in_repeat = 0;
	foreach my $repeat (@$repeats_ref) {
		$repeat = $repeat->transform('chromosome');
		
		print STDERR "Found repeat " . $repeat->display_id . "\t" . $repeat->start . "\t" . $repeat->end . "\n";
		my $rlength = $repeat->end - $repeat->start + 1;
		if (defined($repeat_type_length_hash{$repeat->display_id})) {
			$repeat_type_length_hash{$repeat->display_id} += $rlength;
		} else {
			$repeat_type_length_hash{$repeat->display_id} = $rlength;
		}
		if ($rlength > 1000) {
			$repeat_events_long{$repeat->start . " " . $repeat->end} = 1;
		} else {
			$repeat_events_short{$repeat->start . " " . $repeat->end} = 1;
		}
		if ($repeat->start <= $end && $repeat->end >= $start) {
			$in_repeat = 1;
		}

	}
	my %return_hash = ( 	'in_repeat' => $in_repeat,
				'events_long' => scalar(keys %repeat_events_long),
				'events_short' => scalar(keys %repeat_events_short),
				'length' => $length, 
				'repeats' => \%repeat_type_length_hash );
	return \%return_hash;
}
