package Chum::Utils::StackRandomizer;

use vars qw(@ISA @SR_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;
use Chum::VariantSequence;
use Chum::Stack;

@ISA = qw( Bio::Root::Root );

BEGIN {
	@SR_PARAMS = qw();
	foreach my $attr (@SR_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
	while (@args) {
		my $attr = shift @args;
		my $value = shift @args;
		$self->$attr($value);
    }
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub generate_new_random_stack_from_dinucleotide_composition() {
	my $self = shift;
	my $stack_object = shift;

	my $new_reference_sequence = $self->_randomize_variant_sequence_object_with_dnt_composition_leaving_variant_in_place($stack_object->reference_sequence);
	my @variant_sequences = @{$stack_object->variant_sequences};
	my @new_variant_sequences = ();
	foreach my $variant_sequence_object (@variant_sequences) {
		push @new_variant_sequences, new Chum::VariantSequence(	'id' => $variant_sequence_object->id,
																	'sequence' => new Bio::Seq(-id => "reference", -seq => $new_reference_sequence->allele_left_flank_sequence->seq . $variant_sequence_object->allele_sequence->seq() . $new_reference_sequence->allele_right_flank_sequence->seq),
																	'allele_left_flank_sequence' => new Bio::Seq(-id => "left_flank", -seq => $new_reference_sequence->allele_left_flank_sequence->seq),												
																	'allele_right_flank_sequence' => new Bio::Seq(-id => "right_flank", -seq => $new_reference_sequence->allele_right_flank_sequence->seq),
																	'allele_sequence' => new Bio::Seq(-id => "allele", -seq => $variant_sequence_object->allele_sequence->seq()));
	}
	return new Chum::Stack(	'reference_sequence' => $new_reference_sequence,
							'orthologous_sequences' => $stack_object->orthologous_sequences,
							'variant_sequences' => \@new_variant_sequences,
							'variation_ref_obj' => $stack_object->variation_ref_obj );
}

sub _randomize_variant_sequence_object_with_dnt_composition_leaving_variant_in_place() {
	my $self = shift;
	my $variant_sequence_object = shift;

	my ($a_l_ref, $c_l_ref, $g_l_ref, $t_l_ref) = $self->_get_sequence_dnt_composition($variant_sequence_object->allele_left_flank_sequence);
	my ($a_r_ref, $c_r_ref, $g_r_ref, $t_r_ref) = $self->_get_sequence_dnt_composition($variant_sequence_object->allele_right_flank_sequence);

	my @a;
	my @c; 
	my @g;
	my @t;
	push @a, @$a_l_ref; push @a, @$a_r_ref;
	push @c, @$c_l_ref; push @c, @$c_r_ref;
	push @g, @$g_l_ref; push @g, @$g_r_ref;
	push @t, @$t_l_ref; push @t, @$t_r_ref;

	my $length_left = length($variant_sequence_object->allele_left_flank_sequence->seq); 
	my $length_right = length($variant_sequence_object->allele_right_flank_sequence->seq);
	my $new_sequence = $self->_get_random_sequence_from_dnts($length_left + $length_right , \@a, \@c, \@g, \@t);

	my $left = substr($new_sequence, 0, $length_left);
	my $right = substr($new_sequence, $length_right);

	return new Chum::VariantSequence(	'id' => $variant_sequence_object->id,
										'sequence' => new Bio::Seq(-id => "reference", -seq => $left . $variant_sequence_object->allele_sequence->seq() . $right),
										'allele_left_flank_sequence' => new Bio::Seq(-id => "left_flank", -seq => $left),												
		                                'allele_right_flank_sequence' => new Bio::Seq(-id => "right_flank", -seq => $right),
										'allele_sequence' => new Bio::Seq(-id => "allele", -seq => $variant_sequence_object->allele_sequence->seq()));
}

sub _get_sequence_dnt_composition() {
	my $self = shift;
	my $seq_obj = shift;

	my @a_dnts;
	my @c_dnts;
	my @g_dnts;
	my @t_dnts;

	my $forward_sequence = uc($seq_obj->seq);
	for (my $i = 0; $i < length($forward_sequence); $i++) {
		my $dnt = substr($forward_sequence, $i, 2);
		if (length($dnt) == 2) {
			if ($dnt =~ /A[\S]{1}/g) { push @a_dnts, $dnt; }
			elsif ($dnt =~ /C[\S]{1}/g) { push @c_dnts, $dnt; }
			elsif ($dnt =~ /G[\S]{1}/g) { push @g_dnts, $dnt; }
			elsif ($dnt =~ /T[\S]{1}/g) { push @t_dnts, $dnt; }
		}
	}
	
	my $reverse_sequence = uc($seq_obj->revcom->seq);
	for (my $i = 0; $i < length($reverse_sequence); $i++) {
		my $dnt = substr($reverse_sequence, $i, 2);
		if (length($dnt) == 2) {
			if ($dnt =~ /A[\S]{1}/g) { push @a_dnts, $dnt; }
			elsif ($dnt =~ /C[\S]{1}/g) { push @c_dnts, $dnt; }
			elsif ($dnt =~ /G[\S]{1}/g) { push @g_dnts, $dnt; }
			elsif ($dnt =~ /T[\S]{1}/g) { push @t_dnts, $dnt; }
		}
	}

	return \@a_dnts, \@c_dnts, \@g_dnts, \@t_dnts;
}

sub _get_random_sequence_from_dnts() {
	my $self = shift;
	my $length = shift;
	my $a_ref = shift;
	my $c_ref = shift;
	my $g_ref = shift;
	my $t_ref = shift;

	my @a = @$a_ref;
	my @c = @$c_ref;
	my @g = @$g_ref;
	my @t = @$t_ref;

	if ($length < 2) { $self->throw("Cannot get random sequence for length less than 2"); }

	my @nts = ( 'A', 'C', 'G', 'T');
	my $starting_nt = @nts[rand(@nts)];
	my $return_sequence = $starting_nt;
	my $last_char = $starting_nt;
	while (length($return_sequence) <= $length) {
		#print STDERR ".$last_char\t$return_sequence\t" . length($return_sequence) . ":" . $length . "\n";
		if ($last_char eq 'A') {
			if (scalar(@a) == 0) {
				@a = @$a_ref;
			} 
			my $index = int(rand(@a));
			my $dnt = splice(@a, $index, 1);
			$last_char = substr($dnt, 1, 1);
			$return_sequence .= $last_char;			
		} elsif ($last_char eq 'C') {
			if (scalar(@c) == 0) {
				@c = @$c_ref;
			} 
			my $index = int(rand(@c));
			my $dnt = splice(@c, $index, 1);
			$last_char = substr($dnt, 1, 1);
			$return_sequence .= $last_char;
		} elsif ($last_char eq 'G') {
			if (scalar(@g) == 0) {
				@g = @$g_ref;
			} 
			my $index = int(rand(@g));
			my $dnt = splice(@g, $index, 1);
			$last_char = substr($dnt, 1, 1);
			$return_sequence .= $last_char;
		} elsif ($last_char eq 'T') {
			if (scalar(@t) == 0) {
				@t = @$t_ref;
			} 
			my $index = int(rand(@t));
			my $dnt = splice(@t, $index, 1);
			$last_char = substr($dnt, 1, 1);
			$return_sequence .= $last_char;
		} else {
			$self->throw("Undefined character in sequence: $last_char\n");
		}
	}
	return substr($return_sequence, 0, $length);
}