#
# Chum::VariationTools
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::VariationTools - Tools for working on variation

=head1 SYNOPSIS

=head1 DESCRIPTION

Miscellaneous tools for extracting variation information and sequences

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::VariationTools;

use vars qw(@ISA @VARIATIONTOOLS_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Bio::Seq;
use Data::Dumper;
use Chum::VariantSequence;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @VARIATIONTOOLS_PARAMS = qw();
        foreach my $attr (@VARIATIONTOOLS_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

=head2 get_variation_object_from_ensembl_for_name_source

 Title	 : get_variation_object_from_ensembl_for_name_source
 Usage   : my $variation = $vtools->get_variation_object_from_ensembl_for_name_source('rs999', 'dbSNP', $registry);
 Function: Get a variation object from EnsEMBL
 Returns : Bio::EnsEMBL::Variation::Variation object
 Args    : [1] String (variation name, i.e. 'rs999')
	   [2] String (variation source, i.e. 'dbSNP')
	   [3] Bio::EnsEMBL::Registry
 
=cut

sub get_variation_object_from_ensembl_for_name_source {
	my $self = shift;
	my $variation_name = shift;
	my $variation_source = shift;
	my $registry = shift;
	
	if (!defined($variation_name)) { $self->throw("No variation name has been defined"); 
	} elsif (!defined($variation_source)) { $self->throw("No variation source has been defined"); 
	} elsif (!defined($registry)) { $self->throw("No registry (Bio::EnsEMBL::Registry) has been defined"); 
	} 

	##LOAD THE VARIATION ADAPTOR############################################
	my $va = $registry->get_adaptor("human", "variation", "Variation");
	if (!defined($va)) {
		$self->throw("Could not get VariationAdaptor");
	}
	my $variation = $va->fetch_by_name($variation_name, $variation_source);
	if (!defined($variation)) {
		$self->throw("Could not find variation: " . $variation_name . " in source: " . $variation_source);
	}
	return $variation;
}

=head2 get_variation_features_for_ensembl_variation_object

 Title	 : get_variation_features_for_ensembl_variation_object
 Usage   : my @variation_features = @{$vtools->get_variation_features_for_ensembl_variation_object($variation, $registry)};
 Function: Get variation features object from EnsEMBL for a Bio::EnsEMBL::Variation::Variation object
 Returns : arrayref of Bio::EnsEMBL::Variation::VariationFeature objects
 Args    : [1] Bio::EnsEMBL::Variation::Variation
	   [2] Bio::EnsEMBL::Registry
 
=cut

sub get_variation_features_for_ensembl_variation_object {
	my $self = shift;
	my $variation = shift;
	my $registry = shift;
	my $allow_multiple_variation_features = shift;
	
	if (!defined($variation)) { $self->throw("No variation (Bio::EnsEMBL::Variation::Variation object defined");
	} elsif (!$variation->isa("Bio::EnsEMBL::Variation::Variation")) { $self->throw("variation is not a Bio::EnsEMBL::Variation::Variation object"); 
	} elsif (!defined($registry)) { $self->throw("No registry (Bio::EnsEMBL::Registry) has been defined"); }
	if (!defined($allow_multiple_variation_features)) {
		$allow_multiple_variation_features = 0; 	
	}

	##LOAD THE VARIATION FEATURE ADAPTOR####################################
	my $vfa = $registry->get_adaptor("human", "variation", "VariationFeature");
	if (!defined($vfa)) {
		$self->throw("Could not get VariationFeatureAdaptor");
	}
	
	##GET VARIATION FEATURES################################################
	my @variation_features = @{$vfa->fetch_all_by_Variation($variation)};
	if (!@variation_features) {
		$self->throw("Could not find variation_features for: " . $variation->name . " in source: " . $variation->source);
	} elsif (scalar(@variation_features) > 1) {
		if (!$allow_multiple_variation_features) {
			$self->throw("The following variation doesn't map uniquely to the genome.  variation: " . $variation->name  . " in source: " . $variation->source);
		} 
	}
	return \@variation_features;
}

sub get_alleles_from_variation_feature {
	my $self = shift;
	my $variation_feature = shift; ##An EnsEMBL Variation object
	if (!defined($variation_feature)) {
		$self->throw("Variation feature is undefined\n");
	}
	my @alleles = split (/\//, $variation_feature->allele_string);
	return \@alleles;
}

######## GENERATES ALTERNATIVE SEQUENCES WITH VARIATION IN PLACE FROM AN ENSEMBL VARIATION
sub get_variant_sequences_for_variation_feature {
	my $self = shift;
	my $variation_feature = shift; ##An EnsEMBL Variation object
	my $refseq = shift; ##A Bio::LocatableSeq
	my $halfwidth = shift;

	my @variant_seqs;
	my $reference_sequence;
	my @alleles = @{$self->get_alleles_from_variation_feature($variation_feature)};

	foreach my $allele (@alleles) {
		my $startseq;
		my $endseq;
		if (($variation_feature->end - $variation_feature->start) ==  -1) {
			$startseq = substr($refseq->seq, 0, $halfwidth + 1);
			$endseq = substr($refseq->seq, $halfwidth + ($variation_feature->end - $variation_feature->start) + 2);
		} elsif (($variation_feature->end - $variation_feature->start) >= 0) {
			$startseq = substr($refseq->seq, 0, $halfwidth);
			$endseq = substr($refseq->seq, $halfwidth + ($variation_feature->end - $variation_feature->start) + 1);
		} else {
			$self->throw("COMPLEX VARIATION SIZE DETECTED: " . $variation_feature->display_name);
		}

		my $varseq;
		if ($allele eq "-") {
			$varseq = $startseq . $endseq;			
		} elsif (index($allele, '-') == -1) {
			$varseq = $startseq . $allele . $endseq;
		} else {
			$self->throw("COMPLEX VARIATION ALLELE DETECTED: " . $variation_feature->display_name);
		}

		if ($varseq =~ /[^ACGTactg]/) {
			return undef; #not DNA sequence
		}

		if (uc($varseq) ne uc($refseq->seq)) {
			##Ignore reference sequence here
			push @variant_seqs, new Chum::VariantSequence('id' => "VARIANT",
								      'sequence' => new Bio::Seq(-id => "sequence", -seq => $varseq),
								      'allele_sequence' => new Bio::Seq(-id => "allele", -seq => $allele),
								      'allele_left_flank_sequence' => new Bio::Seq(-id => "left_flank", -seq => $startseq),
								      'allele_right_flank_sequence' => new Bio::Seq(-id => "right_flank", -seq => $endseq));
		} else {
			$reference_sequence = new Chum::VariantSequence('id' => "REFERENCE",
									'sequence' => new Bio::Seq(-id => "sequence", -seq => $varseq),
									'allele_sequence' => new Bio::Seq(-id => "allele", -seq => $allele),
									'allele_left_flank_sequence' => new Bio::Seq(-id => "left_flank", -seq => $startseq),
									'allele_right_flank_sequence' => new Bio::Seq(-id => "right_flank", -seq => $endseq));	
		}
	}
	return ($reference_sequence, \@variant_seqs);
}

######## GENERATES ALTERNATIVE SEQUENCES WITH VARIATION IN PLACE FROM A PROPERLY-FORMATTED GFF ANNOTATION

=head2 get_alleles_from_seqfeatures

 Title	 : get_alleles_from_seqfeatures
 Usage   : my @alleles = @{$vtools->get_alleles_from_seqfeatures($reference_feature, $variant_features_arrayref)};
 Function: Get alleles (a String arrayref) from Bio::SeqFeature::Generic objects for reference and variants.
	   Each Bio::SeqFeature::Generic must have the allele, specified as an annotation called 'allele'
 Returns : arrayref of String (alleles)
 Args    : [1] Bio::SeqFeature::Generic (reference allele annotation)
	   [2] arrayref of Bio::SeqFeature::Generic objects (variants allele annotation)
 
=cut

sub get_alleles_from_seqfeatures {
	my $self = shift;
	my $reference_feature = shift; ## Bio::SeqFeature::Generic
	my $variant_features_arrayref = shift; ##arrayref of Bio::SeqFeature::Generic objects

	my @alleles =();
	my $refanno = $reference_feature->annotation();
	my @reference_annotations = $refanno->get_Annotations('allele');
	if (scalar(@reference_annotations) == 0) {
		$self->warn("No alleles found for feature: " . 	$reference_feature->seq_id());
	} else {
		push @alleles, $reference_annotations[0]->value;
	}
	
	foreach my $variant_feature (@$variant_features_arrayref) {
		my $varanno = $variant_feature->annotation();
		my @variant_annotations = $varanno->get_Annotations('allele');
		if (scalar(@variant_annotations) == 0) {
			$self->warn("No alleles found for feature: " . 	$variant_feature->seq_id());
		} else {
			push @alleles, $variant_annotations[0]->value;
		}
	}
	return \@alleles;
}

sub get_variant_seqs_from_gff {
	my $self = shift;
	my $reference_feature = shift; ##io::SeqFeature::Generic
	my $variant_features_arrayref = shift; ##Bio::SeqFeature::Generic arrayref
	my $refseq = shift; ##A Bio::LocatableSeq
	my $upstream = shift;
	my $downstream = shift;

	my @alleles = @{$self->get_alleles_from_gff($reference_feature, $variant_features_arrayref)};
	print Dumper @alleles;
	print "HERE\n";
	$self->throw();
	my @variant_seqs = ();
#	foreach my $allele (@alleles) {
#		my $startseq;
#		my $endseq;
#		if (($variant_feature->end - $variant_feature->start) ==  -1) {
#			$startseq = substr($refseq->seq, 0, $halfwidth + 1);
#			$endseq = substr($refseq->seq, $halfwidth + ($variant_feature->end - $variant_feature->start) + 2);
#		} elsif (($variant_feature->end - $variant_feature->start) >= 0) {
#			$startseq = substr($refseq->seq, 0, $halfwidth);
#			$endseq = substr($refseq->seq, $halfwidth + ($variant_feature->end - $variant_feature->start) + 1);
#		} else {
#			$self->throw("COMPLEX VARIATION SIZE DETECTED: " . $variant_feature->seq_id);
#		}
#
#		my $varseq;
#		if ($allele eq "-") {
#			$varseq = $startseq . $endseq;			
#		} elsif (index($allele, '-') == -1) {
#			$varseq = $startseq . $allele . $endseq;
#		} else {
#			$self->throw("COMPLEX VARIATION ALLELE DETECTED: " . $variant_feature->seq_id);
#		}
#
#		if (uc($varseq) ne uc($refseq->seq)) {
#			##Ignore reference sequence here
#			push @variant_seqs, new Chum::VariantSeq('seq' => $varseq, 'allele' => $allele, 'startseq' => $startseq);
#		}
#    }
	return \@variant_seqs;
}

sub generate_allele_string {
	my $self = shift;
	my $alleles_ref = shift;
	my @alleles = @$alleles_ref;

	my $allele_string = "";
	for (my $i = 0; $i < scalar(@alleles); $i++) {
		if ($i < (scalar(@alleles) - 1)) {
			$allele_string .= $alleles[$i] . "/";
		} else {
			$allele_string .= $alleles[$i];
		}
	}

	return $allele_string
}

1;
