#!/usr/bin/perl
use strict;

use Chum::Statistics::AnalysisResult;
use Chum::Utils::ExpectancyManager;
use Data::Dumper;

############################################################CONSTANTS
##
my $STARSON = 1;
my $SPECCHAR = "NA";
my $NORMALIZE_ALL_EXPECTANCIES = 1;
my $EXPECTANCY_LOADER = undef;
if ($NORMALIZE_ALL_EXPECTANCIES) { $EXPECTANCY_LOADER = new Chum::Utils::ExpectancyManager(); }

############################################################SETUP PROGRAM
##
my $groups_file;
my $raw_results_directory;
my $transcript_chromosome_map;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-g/) {
        $groups_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-d/) {
        $raw_results_directory = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-t/) {
        $transcript_chromosome_map = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($groups_file)) { die("No input group file defined"); }
unless (-e $groups_file) { die("Input group file does not exist"); }
if (!defined($raw_results_directory)) { die("No raw results directory defined"); }
unless (-d $raw_results_directory) { die("Raw results directory does not exist"); }
if (!defined($transcript_chromosome_map)) { die("No transcript chromosome map file defined"); }
unless (-e $transcript_chromosome_map) { die("Transcript chromosome map file does not exist"); }

############################################################LOAD CHROMOSOME MAP
##
my %TRANSCRIPT_CHROMOSOME_MAP;
open(TCM, $transcript_chromosome_map) || die ("Could not open transcript chr map: " . $transcript_chromosome_map);
while (my $line = <TCM>) {
    chomp($line);
    if ($line =~ /([\S]+)[\s]+([\S]+)/g) {
	$TRANSCRIPT_CHROMOSOME_MAP{$1} = $2;
    }
}
close(TCM);

############################################################OPEN GROUP FILE
##
my %GROUPS;
open(GF, $groups_file) || die ("Could not open group file: " . $groups_file);
while (my $line = <GF>) {
    chomp($line);
    if ($line =~ /([\S]+):([\S\s]+)/g) {
	my $transcript_stable_id = $1;
        my $split_line = $2;
        my @tokens = split /[\s]+/, $split_line;
        foreach my $token (@tokens) {
            if ($token =~ /[\S]+/g) {    
                $GROUPS{$token} = $transcript_stable_id;
            }
        }
    }
}
close(GF);

############################################################RUN TESTS
##
my @results = @{&loadFiles(\%GROUPS, $raw_results_directory)};

my $index = 1;
my %tests = (	"transfac.difference" => $index++,
		"weeder.difference" => $index++,
		"weeder.maximum" => $index++,
		"motifsampler.difference" => $index++,
		"motifsampler.maximum" => $index++,
		"bend.difference" => $index++,
		"curve.difference" => $index++,
		"gc.difference" => $index++,
		"temperature.difference" => $index++,
                "transition.transversion" => $index++,
		"distance" => $index++,
		"ln.distance" => $index++,
		"repeat.density" => $index++,
		"in.repeat" => $index++,
		"repeat.short.events" => $index++,
		"repeat.long.events" => $index++,
		"maf" => $index++,
		"daf" => $index++,
		"coexpression.jaspar.difference" => $index++,
		"in.cpg" => $index++,
		###"nhgri.dnase" => $index++,
		"noble.dnase" => $index++,
		"phastcons" => $index++,
		"rp" => $index++,
		"alignment.distance" => $index++);

############################################################OUTPUT DATA
##

##HEADER DATA
print "id" . "\t" . "transcript" . "\t" . "case";
foreach my $test (sort { $tests{$a} <=> $tests{$b} } keys %tests) {
	print "\t" . $test;
}
print "\n";

##DATA DATA
foreach my $result (@results) {
    if ($result->isNotSNP()) {
	print STDERR "Skipping " . $result->{'id'} . "\n";  ##ENSURE THAT WE SKIP NON-SNPS
	next;  ##PROBABLY NOT A SNP
    }
   
    my $transcript_stable_id = $GROUPS{$result->{'id'}}; 
    my $closest_gene = $result->{'distance'}->{$transcript_stable_id}->{'gene'};
    my $closest_distance = $result->{'distance'}->{$transcript_stable_id}->{'distance'};
    my $closest_ln_distance = $result->{'distance'}->{$transcript_stable_id}->{'ln_distance'};
    
    if (abs($closest_distance) > 2001) {
        warn($result->{'id'} . " is outside range for " . $transcript_stable_id . " with: " . $closest_distance);
	next; ##NOT WITHIN SEARCH REGION WE ARE INTERESTED IN
    } elsif (!defined($closest_gene)) {
	warn("Could not find closest gene for: " . $transcript_stable_id . " in " . $result->{'id'});
	next;
    }
    my $OUTPUT_LINE = "";
    $OUTPUT_LINE .= $result->{'id'} . "\t" . $GROUPS{$result->{'id'}};
    
    if ($result->{'id'} =~ /OREG[0-9]+/) {
	$OUTPUT_LINE .= "\tPOSITIVE\t";
    } elsif ($result->{'id'} =~ /rs[0-9]+/) {
	$OUTPUT_LINE .= "\tNEUTRAL\t";
    } else {
        die("Unknown id: " . $result->{'id'});   
    }
    
    if (defined($tests{'transfac.difference'})) {
	if ($result->{'transfac'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'transfac'}->{'score'} . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    if (defined($tests{'weeder.difference'})) {
	if ($result->{'weeder'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'weeder'}->{'score'} . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    if (defined($tests{'weeder.maximum'})) {
	if ($result->{'weeder'}->{'max_score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'weeder'}->{'max_score'} . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    if (defined($tests{'motifsampler.difference'})) {
	if ($result->{'motifsampler'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'motifsampler'}->{'score'} . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    if (defined($tests{'motifsampler.maximum'})) {
	if ($result->{'motifsampler'}->{'max_score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'motifsampler'}->{'max_score'} . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    if (defined($tests{'bend.difference'})) {
	if ($result->{'bend'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'bend'}->{'score'} . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    if (defined($tests{'curve.difference'})) {
	if ($result->{'curve'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'curve'}->{'score'} . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    if (defined($tests{'gc.difference'})) {
	if ($result->{'gc'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'gc'}->{'score'} . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    if (defined($tests{'temperature.difference'})) {
	if ($result->{'temperature'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'temperature'}->{'score'} . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    
    if (defined($tests{'transition.transversion'})) {
	if ($result->{'transition-transversion'} ne "UNDEFINED" && $result->{'transition-transversion'} ne "") {
	    $OUTPUT_LINE .=  $result->{'transition-transversion'} . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
		
    ###DISTANCE
    if (defined($tests{'distance'})) {
	if (defined($closest_distance)) {
	    $OUTPUT_LINE .=  $closest_distance . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    if (defined($tests{'ln.distance'})) {
	if (defined($closest_ln_distance)) {
	    $OUTPUT_LINE .=  $closest_ln_distance . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    
    my $repeat_density = 0;
    if ($result->{'repeat'} != 0) {
	    $repeat_density = $result->{'repeat'}->{'repeat_length'} / $result->{'repeat'}->{'search_length'};
    }
    my $CHROMOSOME = $TRANSCRIPT_CHROMOSOME_MAP{$transcript_stable_id};
    my $expectancy_adjustment_in_repeat_value = 0;
    my $expectancy_adjustment_short_event = 0;
    my $expectancy_adjustment_long_event = 0;
    if ($NORMALIZE_ALL_EXPECTANCIES) {
	$expectancy_adjustment_in_repeat_value = $EXPECTANCY_LOADER->get_in_repeat_expectancy_value($CHROMOSOME, (0 - $closest_distance));
	$expectancy_adjustment_short_event = $EXPECTANCY_LOADER->get_short_event_expectancy_value($CHROMOSOME, (0 - $closest_distance));
	$expectancy_adjustment_long_event = $EXPECTANCY_LOADER->get_long_event_expectancy_value($CHROMOSOME, (0 - $closest_distance));
    }

		
    if (defined($tests{'repeat.density'})) {
	$OUTPUT_LINE .=  $repeat_density . "\t";
    }
    if (defined($tests{'in.repeat'})) {
	my $adjvalue = $result->{'repeat'}->{'in_repeat'} - $expectancy_adjustment_in_repeat_value;
	$OUTPUT_LINE .=  $adjvalue . "\t";
    }
    if (defined($tests{'repeat.short.events'})) {
	my $adjvalue = $result->{'repeat'}->{'short_events'} - $expectancy_adjustment_short_event;
	$OUTPUT_LINE .=  $adjvalue . "\t";
    }
    if (defined($tests{'repeat.long.events'})) {
	my $adjvalue = $result->{'repeat'}->{'long_events'} - $expectancy_adjustment_long_event;
	$OUTPUT_LINE .=  $adjvalue . "\t";
    }
    

    ###MAF
    my $printed = 0;
    if (defined($tests{'maf'})) {
	if (defined($result->{'minor_allele_frequency'}) && $result->{'minor_allele_frequency'} != 0) {
	    if (defined($result->{'minor_allele_frequency'}->{'ALL'})) {
		if (defined($result->{'minor_allele_frequency'}->{'ALL'}->{'frequency'})) {
		    if (defined($tests{'maf'})) {
		        $OUTPUT_LINE .=  $result->{'minor_allele_frequency'}->{'ALL'}->{'frequency'} . "\t";
		        $printed = 1;
		    }
		}
	    }
	}
    }
    unless($printed) {
	if (defined($tests{'maf'})) {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }

    if (defined($tests{'daf'})) {
	if ($result->{'daf'} ne "UNDEFINED" && defined($result->{'daf'})) {
	    if (defined($result->{'daf'}->{'ALL'} )) {
		$OUTPUT_LINE .=  $result->{'daf'}->{'ALL'} . "\t";
	    } else {
		$OUTPUT_LINE .=  $SPECCHAR . "\t";
	    }
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    
    
    
    if (defined($tests{'coexpression.jaspar.difference'})) {
	if ($result->{'coexpression-jaspar'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'coexpression-jaspar'}->{'score'} . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    
    my $expectancy_adjustment_in_cpg = 0;
    my $expectancy_adjustment_nhgri = 0;
    my $expectancy_adjustment_noble = 0;
    my $expectancy_adjustment_phastcons = 0;
    my $expectancy_adjustment_regulatory_potential = 0;
    if ($NORMALIZE_ALL_EXPECTANCIES) {
	$expectancy_adjustment_in_cpg = $EXPECTANCY_LOADER->get_in_cpg_expectancy_value($CHROMOSOME, (0 - $closest_distance));
	$expectancy_adjustment_nhgri = $EXPECTANCY_LOADER->get_nhgri_expectancy_value($CHROMOSOME, (0 - $closest_distance));
	$expectancy_adjustment_noble = $EXPECTANCY_LOADER->get_noble_expectancy_value($CHROMOSOME, (0 - $closest_distance));
	$expectancy_adjustment_phastcons = $EXPECTANCY_LOADER->get_phastcons_expectancy_value($CHROMOSOME, (0 - $closest_distance));
	$expectancy_adjustment_regulatory_potential = $EXPECTANCY_LOADER->get_regulatory_potential_expectancy_value($CHROMOSOME, (0 - $closest_distance));
    }
    if (defined($tests{'in.cpg'})) {
	if ($result->{'cpg'}->{'score'} ne "UNDEFINED") {
	    my $adjvalue = $result->{'cpg'}->{'score'} - $expectancy_adjustment_in_cpg;
	    $OUTPUT_LINE .=  $adjvalue . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    if (defined($tests{'nhgri.dnase'})) {
	if ($result->{'nhgri-dnase'}->{'score'} ne "UNDEFINED") {
	    my $adjvalue = $result->{'nhgri-dnase'}->{'score'} - $expectancy_adjustment_nhgri;
	    $OUTPUT_LINE .=  $adjvalue . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    if (defined($tests{'noble.dnase'})) {
	if ($result->{'noble-dnase'}->{'score'} ne "UNDEFINED") {
	    my $adjvalue = $result->{'noble-dnase'}->{'score'} - $expectancy_adjustment_noble;
	    $OUTPUT_LINE .=  $adjvalue . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    if (defined($tests{'phastcons'})) {
	if ($result->{'phastcons'}->{'score'} ne "UNDEFINED") {
	    my $adjvalue = $result->{'phastcons'}->{'score'} - $expectancy_adjustment_phastcons;
	    $OUTPUT_LINE .=  $adjvalue . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    if (defined($tests{'rp'})) {
	if ($result->{'rp'}->{'score'} ne "UNDEFINED") {
	    my $adjvalue = $result->{'rp'}->{'score'} - $expectancy_adjustment_phastcons;
	    $OUTPUT_LINE .=  $adjvalue . "\t";
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR . "\t";
	}
    }
    if (defined($tests{'alignment.distance'})) {
	if ($result->{'alignment_distance'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'alignment_distance'};
	} else {
	    $OUTPUT_LINE .=  $SPECCHAR;
	}
    }
    
    chomp($OUTPUT_LINE);
    $OUTPUT_LINE .=  "\n";
    print $OUTPUT_LINE;
}
exit(0);


############################################################PRIVATE SUBROUTINES
##
sub loadFiles {
	my $groupshashref = shift;
        my $raw_results_directory = shift;
        
	my @results = ();
	foreach my $id (keys %$groupshashref) {
		print STDERR "loading file: " . $raw_results_directory . $id . ".results";
		push @results, new Chum::Statistics::AnalysisResult( 'file' => $raw_results_directory . $id . ".results");
	}
	return \@results;
}
