#!/usr/bin/perl

use strict;

################################################# IMPORTS ##########################
use Bio::EnsEMBL::Registry;
use Bio::Seq;
use Chum::Utils::RSNPAnalysisXMLFileReader;
use Data::Dumper;

##TRANSFAC ANALYSIS IMPORTS
##
use Chum::Analysis::Local::MatrixScanner;
use Chum::Result::Interpretter::MatrixScannerInterpretter;
require Chum::DB::Transfac;

##WEEDER ANALYSIS IMPORTS
##
use Chum::Analysis::Local::Weeder;
use Chum::Result::Interpretter::WeederInterpretter;

##MOTIFSAMPLER ANALYSIS IMPORTS
##
use Chum::Analysis::Local::Motifsampler;
use Chum::Result::Interpretter::MotifsamplerInterpretter;

##BANANA ANALYSIS IMPORTS##
##
use Chum::Analysis::Local::Banana;
use Chum::Result::Interpretter::BananaInterpretter;

##DAN ANALYSIS IMPORTS##
##
use Chum::Analysis::Local::Dan;
use Chum::Result::Interpretter::DanInterpretter;

##ZDNA IMPORTS##
##
use Chum::Analysis::Local::Zdna;
use Chum::Result::Interpretter::ZdnaInterpretter;

##UTILITY IMPORTS##
##
use Chum::Utils::DistanceCalculator;
use Chum::Utils::GeneFinder;
use Chum::Utils::Kaks;
use Chum::Utils::RepeatContentFinder;
use Chum::Utils::AlleleFrequencyFinder;

##SPECIAL DATA ANALYSES IMPORTS##
##
require Chum::DB::Jaspar;
use Chum::Analysis::ExtData::CpGIsland;
use Chum::Analysis::ExtData::NhgriDNaseISite;
use Chum::Analysis::ExtData::NobleDNaseISite;
use Chum::Analysis::ExtData::PhastCons;
use Chum::Analysis::ExtData::RegulatoryPotential;

###############################################CONSTANTS############################
my $background_file = "/home/smontgom/motifsampler/human_promoters.bg";



####################################################################################
####################################################################################
#########	START PROGRAM
####################################################################################
####################################################################################

###############################################LOAD INPUT PARAMETERS################
my $rsnpa_xml_file;
my $output_file;
my $test_line;
my $ens_gene_id;
my $chromosome;
my $start;
my $end;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnpa_xml_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-o/) {
	$output_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-x/) {
        $test_line = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-g/) {
        $ens_gene_id = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-c/) {
        $chromosome = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-s/) {
        $start = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-e/) {
        $end = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }
if (!defined($output_file)) { die("ERROR: Output file not defined: -o\n"); }
if (!defined($ens_gene_id)) { die("ERROR: Target EnsEMBL gene required -g"); }
Bio::EnsEMBL::Registry->load_all();

my $ens_trans_id = undef;
if ($ens_gene_id =~ /ENST[0-9]+/gi) {
	$ens_trans_id = $ens_gene_id;
	eval {
	    my $ga = Bio::EnsEMBL::Registry->get_adaptor("human", "core", "Gene");
	    my $gene = $ga->fetch_by_transcript_stable_id($ens_gene_id);
	    $ens_gene_id = $gene->stable_id;
	}; if ($@) {
	    warn $@;   
	}
}

###############################################LOAD PREVIOUS RESULTS IF THE EXIST###
my ($results1, $results2, $results3, $results4, $results5, $results6, $results7, $results8, $results9, $results10, $results11, $results12, $results13, $results14, $results15, $results16, $results17, $results18, $results19);
if (-e $output_file) {
    open(INPUT, $output_file) || die("Cannot open existing output file for input\n");
    my $run_number;
    my $run_line;
    my $run_status;
    foreach my $line (<INPUT>) {
	chomp($line);
	if ($line =~ /[\S]{1}TEST([0-9]+):[\S]+:([\S]+)/g || $line =~ /[\S]{1}(DONE)/g) {
	    if (defined($run_number)) { 
		if ($run_number == 1) { $results1 = $run_line; }    	##TRANSFAC ANALYSIS
		elsif ($run_number == 2) { $results2 = $run_line; } 	##WEEDER ANALYSIS
		elsif ($run_number == 3) { $results3 = $run_line; } 	##MOTIFSAMPLER ANALYSIS
		elsif ($run_number == 4) { $results4 = $run_line; } 	##BANANA ANALYSIS
		elsif ($run_number == 5) { $results5 = $run_line; }	##DAN ANALYSIS
		elsif ($run_number == 6) { $results6 = $run_line; }	##DISTANCE ANALYSIS
		elsif ($run_number == 7) { $results7 = $run_line; }	##KAKS ANALYSIS
		elsif ($run_number == 8) { $results8 = $run_line; }	##REPEAT ANALYSIS(NORMALIZED)
		elsif ($run_number == 9) { $results9 = $run_line; }	##MAF ANALYSIS
		elsif ($run_number == 10) { $results10 = $run_line; }	##DAF(CHIMP) ANALYSIS (SAMPLES)
		elsif ($run_number == 11) { $results11 = $run_line; }	##TRANSITION/TRANSVERSION
		elsif ($run_number == 12) { $results12 = $run_line; }	##COEXPRESSION SCANNING ANALYSIS
		elsif ($run_number == 13) { $results13 = $run_line; }	##CPG ISLAND ANALYSIS(RAW)
		elsif ($run_number == 14) { $results14 = $run_line; }	##NHGRI ANALYSIS(RAW)
		elsif ($run_number == 15) { $results15 = $run_line; }	##NOBLE ANALYSIS(RAW)
		elsif ($run_number == 16) { $results16 = $run_line; }	##PHASTCONS ANALYSIS(RAW)
		elsif ($run_number == 17) { $results17 = $run_line; }	##REGULATORY POTENTIAL ANALYSIS(RAW)
		elsif ($run_number == 18) { $results18 = $run_line; }	##ORTHOLOGY DISTANCE
		elsif ($run_number == 19) { $results19 = $run_line; }	##ZDNA ANALYSIS
	    }
	    if ($run_number eq "DONE") { last; }
	    $run_number = $1;
	    $run_line = $line . "\n";
	    $run_status = $2;
	} elsif (defined($run_number)) {
	    $run_line .= $line . "\n";
	}
    }
    close(INPUT);
}

###############################################ANALYZE INPUT FILE##################
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
open (OUTPUT, ">" . $output_file) || die("Cannot open output file\n");

my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');
my $stack_object_f10 = $rsnpa_obj->get_flanked_sequence_stack_obj(10, 'hosa');

###############################################RUN ALL TESTS TOKENS################
my @tokens = split(':', $test_line);
my %IGNORE_TOKENS;
foreach my $token (@tokens) {
    $IGNORE_TOKENS{$token} = 1;
}

####################################################################################
########################TEST 1: TRANSFAC############################################
####################################################################################
if (!defined($IGNORE_TOKENS{"1"})) {
    print STDERR "Running TEST1:TRANSFAC\n";
    $results1 = "";
    eval {
	my $transfac_db = new Chum::DB::Transfac('transfac_dir' => "/home/pubseq/databases/TransFac/current/www/biobase/transfac/7.2/data");
	$transfac_db->matrixset($transfac_db->_find_all_matrixset());
	my $matrix_scanner = new Chum::Analysis::Local::MatrixScanner('matrix_db' => $transfac_db);
	my $resultset = $matrix_scanner->run_variation_analysis($stack_object_f10);
	if (defined($resultset)) {
	    my $msi = new Chum::Result::Interpretter::MatrixScannerInterpretter('resultset' => $resultset);
	    my $score = $msi->interpret();
	    $results1 = $score;
	} else {
	    die("Could not get transfac results" . $rsnpa_xml_file);
	}
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results1 = "#TEST1:TRANSFAC:FAILED\n" . $results1 . "\n";
    } else {
	$results1 = "#TEST1:TRANSFAC:SUCCESS\n" . $results1 . "\n";
    }
}

####################################################################################
#######################TEST2: WEEDER################################################
####################################################################################
if (!defined($IGNORE_TOKENS{"2"})) {
    print STDERR "Running TEST2:WEEDER\n";
    $results2 = "";
    eval {
	my $weeder = new Chum::Analysis::Local::Weeder();
	$weeder->{'binary'} = "/home/smontgom/chinook-internal-server/services/weeder/Weeder1.0.1/weederlaunchersm.pl";
	my $meta_input_hashref;
	$meta_input_hashref->{'-o'} = "HS";
	$meta_input_hashref->{'-r'} = "small";
	$meta_input_hashref->{'-s'} = "";
	$meta_input_hashref->{'-m'} = "";
	$meta_input_hashref->{'-t'} = "500";
	my $resultset = $weeder->run_variation_analysis($stack_object, $meta_input_hashref);
		
	if(defined($resultset)) {
	    my $winterpretter = new Chum::Result::Interpretter::WeederInterpretter('resultset' => $resultset);
	    my ($min, $max, $diff) = $winterpretter->calculate_maximum_difference();
	    $results2 = "VAR:" . $diff . "\nMAX:" . $max;
	} else {
	    die("Could not get weeder results" . $rsnpa_xml_file);
	}
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results2 = "#TEST2:WEEDER:FAILED\n" . $results2 . "\n";
    } else {
	$results2 = "#TEST2:WEEDER:SUCCESS\n" . $results2 . "\n";
    }
}

####################################################################################
##############################TEST3: MOTIFSAMPLER###################################
####################################################################################
if (!defined($IGNORE_TOKENS{"3"})) {
    print STDERR "Running TEST3:MOTIFSAMPLER\n";
    $results3 = "";
    eval {
	my $motifsampler = new Chum::Analysis::Local::Motifsampler();
	$motifsampler->binary("/home/smontgom/motifsampler/MotifSampler");
	my %meta_input_hashref;
	$meta_input_hashref{'background_file'} = $background_file;
	$meta_input_hashref{'-p'} = "0.95";
	$meta_input_hashref{'-n'} = "3";
	$meta_input_hashref{'-r'} = "5";
	my $resultset = $motifsampler->run_variation_analysis($stack_object_f10, \%meta_input_hashref);
	if (defined($resultset)) {
	    my $minterpretter = new Chum::Result::Interpretter::MotifsamplerInterpretter('resultset' => $resultset);
	    my ($min, $max, $diff) = $minterpretter->calculate_maximum_difference();
	    $results3 = "VAR:" . $diff . "\nMAX:" . $max;
	} else {
	    die("Could not get motifsampler results" . $rsnpa_xml_file);
	}
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results3 = "#TEST3:MOTIFSAMPLER:FAILED\n" . $results3 . "\n";
    } else {
	$results3 = "#TEST3:MOTIFSAMPLER:SUCCESS\n" . $results3 . "\n";
    }
}

####################################################################################
##############################TEST4: BANANA#########################################
####################################################################################
if (!defined($IGNORE_TOKENS{"4"})) {
    print STDERR "Running TEST4:BANANA\n";
    $results4 = "";
    eval {	
	my $banana = new Chum::Analysis::Local::Banana();
	my $resultset = $banana->run_variation_analysis($stack_object);
	if (defined($resultset)) {
	    my $binterpretter = new Chum::Result::Interpretter::BananaInterpretter('resultset' => $resultset);
	    my $bscore = $binterpretter->calculate_maximum_difference('bend');
	    my $cscore = $binterpretter->calculate_maximum_difference('curve');
	    $results4 = "BEND:" . $bscore . "\nCURVE:" . $cscore;
	} else {
	    die("Could not get banana results" . $rsnpa_xml_file);
	}
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results4 = "#TEST4:BANANA:FAILED\n" . $results4 . "\n";
    } else {
	$results4 = "#TEST4:BANANA:SUCCESS\n" . $results4 . "\n";;
    }
}

####################################################################################
##############################TEST5: DAN############################################
####################################################################################
if (!defined($IGNORE_TOKENS{"5"})) {
    print STDERR "Running TEST5:DAN\n";
    $results5 = "";
    eval {
	my $dan = new Chum::Analysis::Local::Dan();
	my $resultset = $dan->run_variation_analysis($stack_object);
	if (defined($resultset)) {
	    my $dinterpretter = new Chum::Result::Interpretter::DanInterpretter('resultset' => $resultset);
	    my $gscore = $dinterpretter->calculate_maximum_difference('gc');
	    my $tscore = $dinterpretter->calculate_maximum_difference('tm');
	    $results5 = "GC:" . $gscore . "\nTM:" . $tscore;
	} else {
	    die("Could not get dan results" . $rsnpa_xml_file);
	}
    };
    if ($@) {
    	print STDERR "ERROR: " . $@ . "\n";
    	$results5 = "#TEST5:DAN:FAILED\n" . $results5 . "\n";
    } else {
    	$results5 = "#TEST5:DAN:SUCCESS\n" . $results5 . "\n";
    }
}

####################################################################################
##############################TEST6: DISTANCE#######################################
####################################################################################
my @gene_names = ();
if (!defined($IGNORE_TOKENS{"6"})) {
    print STDERR "Running TEST6:DISTANCE\n";
    $results6 = "";
    eval {
	my $gene_finder = new Chum::Utils::GeneFinder();
	#my $transcript_distance_hashref = $gene_finder->get_ensembl_transcript_distances_to_stack($stack_object);
	my $transcript_distance_hashref = $gene_finder->_get_transcript_distances($chromosome, $start, $end, "1");
	if (defined $transcript_distance_hashref) {
	    foreach my $key (keys %$transcript_distance_hashref) {
		my @ids = split(/:/, $key);
		if (defined($ens_trans_id)) {
			if (uc($ids[1]) eq uc($ens_trans_id)) {
				my $mdistance = abs($transcript_distance_hashref->{$key});
                                if ($mdistance == 0) {  $mdistance = 1; }
                                $results6 .= $key . ":" . $transcript_distance_hashref->{$key} . ":" . log($mdistance) . "\n";
                                if ($ids[0] ne "unknown") {  push @gene_names, $ids[0];}
			}
		} else {
			if ($ids[0] eq $ens_gene_id) {
		    		my $mdistance = abs($transcript_distance_hashref->{$key});
		    		if ($mdistance == 0) {  $mdistance = 1; } 
		    		$results6 .= $key . ":" . $transcript_distance_hashref->{$key} . ":" . log($mdistance) . "\n";
		    		if ($ids[0] ne "unknown") {  push @gene_names, $ids[0];}
			}
		}
	    }
	    if (scalar(@gene_names) == 0) {
		foreach my $key (keys %$transcript_distance_hashref) {
		    my @ids = split(/:/, $key);
		    my $mdistance = abs($transcript_distance_hashref->{$key});
		    if ($mdistance == 0) { $mdistance = 1; } 
		    $results6 .= $key . ":" . $transcript_distance_hashref->{$key} . ":" . log($mdistance) . "\n";
		    if ($ids[0] ne "unknown") {
			push @gene_names, $ids[0];
		    }
		}
	    }
	}
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results6 = "#TEST6:DISTANCE:FAILED\n" . @$ . "\n";
    } else {
	$results6 = "#TEST6:DISTANCE:SUCCESS\n" . $results6;
    }
}

####################################################################################
##############################TEST7: KAKS###########################################
####################################################################################
if (!defined($IGNORE_TOKENS{"7"})) {
    print STDERR "Running TEST7:KAKS\n";
    $results7 = "";
    eval {
	if (defined($IGNORE_TOKENS{"6"})) {
	    ##Get gene names
	    my $gene_finder = new Chum::Utils::GeneFinder();
	    #my $transcript_distance_hashref = $gene_finder->get_ensembl_transcript_distances_to_stack($stack_object);
	    my $transcript_distance_hashref = $gene_finder->_get_transcript_distances($chromosome, $start, $end, "1");
	    if (defined $transcript_distance_hashref) {
		foreach my $key (keys %$transcript_distance_hashref) {
		    my @ids = split(/:/, $key);
		    if ($ids[0] eq $ens_gene_id) {
			if ($ids[0] ne "unknown") {  push @gene_names, $ids[0];}
		    }
		}
		if (scalar(@gene_names) == 0) {
		    foreach my $key (keys %$transcript_distance_hashref) {
			my @ids = split(/:/, $key);
			if ($ids[0] eq $ens_gene_id) {
			    if ($ids[0] ne "unknown") {  push @gene_names, $ids[0];}
			}
		    }
		}
	    }
	}
	my $kaks = new Chum::Utils::Kaks();
	my $kaks_results_hashref = $kaks->get_kaks_species_and_distances_for_gene_names_hashref(\@gene_names);
	if (defined($kaks_results_hashref)) {
	    foreach my $key (keys %$kaks_results_hashref) {
		my $scores_arrayref = $kaks_results_hashref->{$key};
		if (defined($scores_arrayref)) {
		    foreach my $score (@$scores_arrayref) {
			$results7 .= $key . ":" . $score->{'species1'} . ":" . $score->{'species2'} . ":" . $score->{'kaks'} . "\n"; 
		    }
		} else {
		    $results7 .= $key . ":\n";
		}
	    }
	} else { $results7 = "\n"; }
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results7 = "#TEST7:KAKS:FAILED\n" . $@ . "\n";
    } else {
	$results7 = "#TEST7:KAKS:SUCCESS\n" . $results7;
    }
}


####################################################################################
##############################TEST8: REPEAT CONTENT#################################
####################################################################################

if (!defined($IGNORE_TOKENS{"8"})) {
    print STDERR "Running TEST8:REPEAT\n";
    $results8 = "";
    eval {
	my $scan_length = 100;
	my $rcf = new Chum::Utils::RepeatContentFinder();
	my $repeat_content = $rcf->get_repeat_content_chr_start_end($chromosome, $start, $end, $scan_length);
	#my $repeat_content = $rcf->get_repeat_content($stack_object, $scan_length);
	if (defined($repeat_content)) {
	    $results8 .= "LENGTH:" . $repeat_content->{'length'} . "\n";
	    $results8 .= "VAR_IN_REPEAT:" . $repeat_content->{'in_repeat'} . "\n";
	    $results8 .= "LONG_EVENTS:" . $repeat_content->{'events_long'} . "\n";
	    $results8 .= "SHORT_EVENTS:" . $repeat_content->{'events_short'} . "\n";
	    my $repeat_types_hashref = $repeat_content->{'repeats'};
	    foreach my $key (keys %$repeat_types_hashref) {
		$results8 .= $key . ":" . $repeat_types_hashref->{$key} . "\n";			
	    }		
	} else {
	    $results8 = "\n";
	}
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results8 = "#TEST8:REPEAT:FAILED\n";
    } else {
	$results8 = "#TEST8:REPEAT:SUCCESS\n" . $results8;
    }
}

####################################################################################
###############################TEST9: ALLELE FREQUENCY##############################
####################################################################################
my $genotype_populations_arrayref = undef;
my $totals_genotypes_hashref = undef;

if (!defined($IGNORE_TOKENS{"9"})) {
    print STDERR "Running TEST9:ALLELEFREQ\n";
    $results9 = "";
    eval {
	my $aff = new Chum::Utils::AlleleFrequencyFinder();
	$genotype_populations_arrayref = $aff->get_allele_frequencies_by_genotype_population($stack_object);
	$totals_genotypes_hashref = $aff->get_total_allele_frequency_from_genotype_populations($genotype_populations_arrayref);

	##HAVE GENOTYPE INFORMATION
	if (defined($genotype_populations_arrayref) && defined($totals_genotypes_hashref)) {
	#    my @variant_sequences = @{$stack_object->variant_sequences};
	#
	    ##1. determine alleles and if they should be reverse-complemented
	    my @db_alleles = ();
	    foreach my $allele (@{$totals_genotypes_hashref->{'alleles'}}) {
		if ($allele->{'allele'} =~ /^[ACTGactg\-]+$/g) {
		    push @db_alleles, $allele;
		}
	    }
	#    my $rev_com_required = 1;
	#    my @ref_alleles = ( lc($stack_object->reference_sequence->allele_sequence->seq),
	#			lc($variant_sequences[0]->allele_sequence->seq));
	#    my @rev_ref_alleles = ( lc($stack_object->reference_sequence->allele_sequence->revcom->seq),
	#			    lc($variant_sequences[0]->allele_sequence->revcom->seq));
	#    my $ref_match = 0;
	#    my $rev_match = 0;
	#    foreach my $ref_allele (@ref_alleles) {
	#	foreach my $db_allele (@db_alleles) {
	#	    if ($ref_allele eq lc($db_allele->{'allele'})) {
	#		$ref_match++;
	#	    }
	#	}
	#    }
	#    foreach my $rev_ref_allele (@rev_ref_alleles) {
	#	foreach my $db_allele (@db_alleles) {
	#	    if ($rev_ref_allele eq lc($db_allele->{'allele'})) {
	#		$rev_match++;
	#	    }
	#	}
	#    }
	#    if ($rev_match > $ref_match) {  ##More of the alleles from the reference/variant seq are matched by rev-com of the obtained db_alleles
	#	$rev_com_required = 0;
	#    }
    
	    ##2. print out total allele frequency and alleles
	    foreach my $allele (@db_alleles) {
		my $allele_seq;
		if ($allele->{'strand'} == -1) {
		    my $allele_seqobj = new Bio::Seq(   -seq => lc($allele->{'allele'}),
                                                        -id => "tmp");
		    $allele_seq = lc($allele_seqobj->revcom->seq);
		} else {
		    $allele_seq = $allele->{'allele'};
		}
		$results9 .= "ALL:" . $totals_genotypes_hashref->{'samples'} . ":" . uc($allele_seq) . ":" . $allele->{'frequency'} . "\n";
	    }
    
	    ##3. print out all population alleles and frequencies
	    foreach my $population (@$genotype_populations_arrayref) {
		foreach my $allele (@{$population->{'alleles'}}) {
		    my $allele_seq;
		    if ($allele->{'strand'} == -1) {
			my $allele_seqobj = new Bio::Seq(   -seq => lc($allele->{'allele'}),
							    -id => "tmp");
			$allele_seq = lc($allele_seqobj->revcom->seq);
		    } else {
			$allele_seq = $allele->{'allele'};
		    }
		    $results9 .=  $population->{'population_location_id'} .  ":" . $population->{'samples'} . ":" .  uc($allele_seq) . ":" . $allele->{'frequency'} . "\n";
		}
	    }
	} else {
	    $results9 = "\n";
	}
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results9 = "#TEST9:ALLELEFREQ:FAILED\n";
    } else {
	$results9 = "#TEST9:ALLELEFREQ:SUCCESS\n" . $results9;
    }
}

####################################################################################
###############################TEST10: DERIVED ALLELE FREQUENCY#####################
####################################################################################
if (!defined($IGNORE_TOKENS{"10"})) {
    print STDERR "Running TEST10:DAF\n";
    $results10 = "";
    eval {
	my $aff = new Chum::Utils::AlleleFrequencyFinder();
	if (defined($IGNORE_TOKENS{"9"})) {
	    $genotype_populations_arrayref = $aff->get_allele_frequencies_by_genotype_population($stack_object);
	    $totals_genotypes_hashref = $aff->get_total_allele_frequency_from_genotype_populations($genotype_populations_arrayref);
	}
	my $daf = $aff->get_daf_from_orthologue($stack_object, "patr", $genotype_populations_arrayref, $totals_genotypes_hashref);
	if (defined($daf)) {
	    my $daf_frequency = $daf->{'frequency'};
	    foreach my $population (keys %$daf_frequency) {
		$results10 .= "DAF:" . $population . ":" .  $daf->{'frequency'}->{$population}->{'allele'}->{'frequency'} ."\n";  
	    }
	} else {
	    $results10 = "\n";
	}    
    };
    if ($@) {
    	print STDERR "ERROR: " . $@ . "\n";
    	$results10 = "#TEST10:DAF:FAILED\n";
    } else {
	$results10 = "#TEST10:DAF:SUCCESS\n" . $results10;
    }
}

####################################################################################
#############################TEST11: TRANSITION/TRANSVERSION########################
####################################################################################
if (!defined($IGNORE_TOKENS{"11"})) {
    print STDERR "Running TEST11:TRANSITION-TRANSVERSION\n";
    $results11 = "";
    eval {
	my $allele_ref = lc($stack_object->reference_sequence->allele_sequence->seq);
	if ((length $allele_ref) == 1) {
	    my @variant_sequences = @{$stack_object->variant_sequences};
	    my $allele_var;
	    foreach my $variant_sequence (@variant_sequences) {
		my $av = $variant_sequence->allele_sequence->seq;
		if ((length $av) == 1) {
		    $allele_var = lc($av);
		    last;
		}
	    }
	    if (($allele_ref eq 'a' && $allele_var eq 'g') || ($allele_var eq 'a' && $allele_ref eq 'g')) { $results11 = "TRANSITION"; }
	    elsif (($allele_ref eq 't' && $allele_var eq 'c') || ($allele_var eq 't' && $allele_ref eq 'c')) { $results11 = "TRANSITION"; }
	    elsif (($allele_ref eq 'a' && $allele_var eq 'c') || ($allele_var eq 'a' && $allele_ref eq 'c')) { $results11 = "TRANSVERSION"; }
	    elsif (($allele_ref eq 'a' && $allele_var eq 't') || ($allele_var eq 'a' && $allele_ref eq 't')) { $results11 = "TRANSVERSION"; }
	    elsif (($allele_ref eq 'g' && $allele_var eq 'c') || ($allele_var eq 'g' && $allele_ref eq 'c')) { $results11 = "TRANSVERSION"; }
	    elsif (($allele_ref eq 'g' && $allele_var eq 't') || ($allele_var eq 'g' && $allele_ref eq 't')) { $results11 = "TRANSVERSION"; }
	    else {
		#do nothing
	    }
	} 
	$results11 .= "\n";
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results11 = "#TEST11:TRANSITION-TRANSVERSION:FAILED\n";
    } else {
	$results11 = "#TEST11:TRANSITION-TRANSVERSION:SUCCESS\n" . $results11;
    }
}

####################################################################################
#########################TEST 12: COEXPRESSION-OPTIMIZED JASPAR#####################
####################################################################################
if (!defined($IGNORE_TOKENS{"12"})) {
    print STDERR "Running TEST12:COEXPRESSION-JASPAR\n";
    $results12 = "";
    eval {
	my $jaspar = new Chum::DB::Jaspar();
	unless(-e "/home/smontgom/sNPeGEn/pipeline/11a_build_coexpression_matrices/coexpression-matrices-july-2006/" . $ens_gene_id . ".matrix") {
	    die("No matrix file exists for ENSG: " . $ens_gene_id);
	}
	open(MATRIX, "/home/smontgom/sNPeGEn/pipeline/11a_build_coexpression_matrices/coexpression-matrices-july-2006/" . $ens_gene_id . ".matrix");		
	my @factors = ();
	my %modifier_scores;
	foreach my $line (<MATRIX>) {
	    chomp($line);
	    if ($line =~ /([\S]+)[\s]+([\S]+)/g) {
		push @factors, $1;
		    $modifier_scores{$1} = $2;
		}
	    }
	close(MATRIX);

	my $matrixset = $jaspar->_find_pwm_matrixset(\@factors);
	$jaspar->matrixset($matrixset);
	my $matrix_scanner = new Chum::Analysis::Local::MatrixScanner('matrix_db' => $jaspar);
	my $resultset = $matrix_scanner->run_variation_analysis($stack_object_f10);
	if (defined($resultset)) {
	    my $msi = new Chum::Result::Interpretter::MatrixScannerInterpretter('resultset' => $resultset);
	    my $score = $msi->interpret_with_modifiers(\%modifier_scores);
	    print STDERR "SCORE: $score\n";
	    $results12 = $score;
	} else {
	    die("Could not get coexpression-jaspar results" . $rsnpa_xml_file);
	}
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results12 = "#TEST12:COEXPRESSION-JASPAR:FAILED\n" . $results12 . "\n";
    } else {
	$results12 = "#TEST12:COEXPRESSION-JASPAR:SUCCESS\n" . $results12 . "\n";
    }
}

####################################################################################
#########################TEST 13: CPG ISLANDS#######################################
####################################################################################
if (!defined($IGNORE_TOKENS{"13"})) {
    print STDERR "Running TEST13:CPG\n";
    $results13 = "";
    eval {
	my ($chromosome, $start, $end, $strand) = &getHG17Mapping($rsnpa_obj->{'id'});
	if (defined($chromosome) && defined($start) && defined($end) && defined($strand)) {
	    my $cpg_detector = new Chum::Analysis::ExtData::CpGIsland();
	    $results13 = $cpg_detector->in_cpg($chromosome, ($start + $stack_object->reference_sequence->start), ($start + $stack_object->reference_sequence->end));
	} else {
	    die("Cannot find coordinates for CpG island detection\n");
	}
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results13 = "#TEST13:CPG:FAILED\n" . $results13 . "\n";
    } else {
	$results13 = "#TEST13:CPG:SUCCESS\n" . $results13 . "\n";
    }
}

####################################################################################
#########################TEST 14: NHGRI DNASE1 SITE#################################
####################################################################################
if (!defined($IGNORE_TOKENS{"14"})) {
    print STDERR "Running TEST14:NHGRI-DNASEI\n";
    $results14 = "";
    eval {
	my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');
	my ($chromosome, $start, $end, $strand) = &getHG17Mapping($rsnpa_obj->{'id'});
	if (defined($chromosome) && defined($start) && defined($end) && defined($strand)) {
	    my $ndIs_detector = new Chum::Analysis::ExtData::NhgriDNaseISite();
	    $results14 = $ndIs_detector->in_dnase1_hs_site($chromosome, ($start + $stack_object->reference_sequence->start), ($start + $stack_object->reference_sequence->end));
	} else {
	    die("Cannot find coordinates\n");
	}
    };
    if ($@) {
	    print STDERR "ERROR: " . $@ . "\n";
	    $results14 = "#TEST14:NHGRI-DNASEI:FAILED\n" . $results14 . "\n";
    } else {
	    $results14 = "#TEST14:NHGRI-DNASEI:SUCCESS\n" . $results14 . "\n";
    }
}

####################################################################################
#########################TEST 15: NOBLE DNASE1 SITE#################################
####################################################################################
if (!defined($IGNORE_TOKENS{"15"})) {
    print STDERR "Running TEST15:NOBLE-DNASEI\n";
    $results15 = "";
    eval {
	my ($chromosome, $start, $end, $strand) = &getHG15Mapping($rsnpa_obj->{'id'});
	if (defined($chromosome) && defined($start) && defined($end) && defined($strand)) {
	    my $nodIs_detector = new Chum::Analysis::ExtData::NobleDNaseISite();
	    $results15 = $nodIs_detector->in_dnase1_hs_site($chromosome, ($start + $stack_object->reference_sequence->start), ($start + $stack_object->reference_sequence->end));
	} else {
		die("Cannot find coordinates\n");
	}
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results15 = "#TEST15:NOBLE-DNASEI:FAILED\n" . $results15 . "\n";
    } else {
	$results15 = "#TEST15:NOBLE-DNASEI:SUCCESS\n" . $results15 . "\n";
    }
}

####################################################################################
#########################TEST 16: PHAST CONS SCORE##################################
####################################################################################
if (!defined($IGNORE_TOKENS{"16"})) {
    print STDERR "Running TEST16:PHASTCONS\n";
    $results16 = "";
    eval {
	my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');
	my ($chromosome, $start, $end, $strand) = &getHG17Mapping($rsnpa_obj->{'id'});
	if (defined($chromosome) && defined($start) && defined($end) && defined($strand)) {
	    my $pc_detector = new Chum::Analysis::ExtData::PhastCons('phast_cons_dir' => "/home/pubseq/databases/phast_cons/mzPt1Mm5Rn3Cf1Gg2Fr1Dr1",
								     '_cache_on' => 1);
	    $results16 = $pc_detector->average_phast_cons_score($chromosome, ($start + $stack_object->reference_sequence->start), ($start + $stack_object->reference_sequence->end));
	} else {
	    die("Cannot find coordinates\n");
	}
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results16 = "#TEST16:PHASTCONS:FAILED\n" . $results16 . "\n";
    } else {
	$results16 = "#TEST16:PHASTCONS:SUCCESS\n" . $results16 . "\n";
    }
}
####################################################################################
#########################TEST 17: REGULATORY POTENTIAL SCORE########################
####################################################################################
if (!defined($IGNORE_TOKENS{"17"})) {
    print STDERR "Running TEST17:RP\n";
    $results17 = "";
    eval {
	my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');
	my ($chromosome, $start, $end, $strand) = &getHG17Mapping($rsnpa_obj->{'id'});
	if (defined($chromosome) && defined($start) && defined($end) && defined($strand)) {
	    my $rp_detector = new Chum::Analysis::ExtData::RegulatoryPotential('regulatory_potential_dir' => "/home/pubseq/databases/regulatory_potential",
									       '_cache_on' => 1);
	    $results17 = $rp_detector->average_regulatory_potential_score($chromosome, ($start + $stack_object->reference_sequence->start), ($start + $stack_object->reference_sequence->end));
	} else {
	    die("Cannot find coordinates\n");
	}
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results17 = "#TEST17:RP:FAILED\n" . $results17 . "\n";
    } else {
	$results17 = "#TEST17:RP:SUCCESS\n" . $results17 . "\n";
    }
}
####################################################################################
#########################TEST 18: ORTHOLOGUE DISTANCE###############################
####################################################################################
if (!defined($IGNORE_TOKENS{"18"})) {
    print STDERR "Running TEST18:ORTHODIST\n";
    $results18 = "";
    eval {
	my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');
	my $distance_calculator = new Chum::Utils::DistanceCalculator();
	my $distance = $distance_calculator->calculate_distance($stack_object);
	$results18 = $distance;
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results18 = "#TEST18:ORTHODIST:FAILED\n" . $results18 . "\n";
    } else {
	$results18 = "#TEST18:ORTHODIST:SUCCESS\n" . $results18 . "\n";
    }
}

####################################################################################
#########################TEST 19: ZHUNT ANALYSIS####################################
####################################################################################
if (!defined($IGNORE_TOKENS{"19"})) {
    print STDERR "Running TEST19:ZHUNT\n";
    $results19 = "";
    eval {
	my $zhunt = new Chum::Analysis::Local::Zdna();
	my $resultset = $zhunt->run_variation_analysis($stack_object);
	if (defined($resultset)) {
	    my $zinterpretter = new Chum::Result::Interpretter::ZdnaInterpretter('resultset' => $resultset);
	    my $dscore = $zinterpretter->calculate_maximum_difference();
	    my $mscore = $zinterpretter->calculate_maximum_score();
	    $results19 = "DIFF:" . $dscore . "\nMAX:" . $mscore;
	} else {
	    die("Could not get zDNA results" . $rsnpa_xml_file);
	}
    };
    if ($@) {
	print STDERR "ERROR: " . $@ . "\n";
	$results19 = "#TEST19:ZHUNT:FAILED\n" . $results19 . "\n";
    } else {
	$results19 = "#TEST19:ZHUNT:SUCCESS\n" . $results19 . "\n";
    }
}

####################################################################################
####################################################################################
####################################################################################
####################################################################################
####################################################################################
####################################################################################
####################################################################################
####################################################################################
#############################DONE###################################################

print OUTPUT $results1;
print OUTPUT $results2;
print OUTPUT $results3;
print OUTPUT $results4;
print OUTPUT $results5;
print OUTPUT $results6;
print OUTPUT $results7;
print OUTPUT $results8;
print OUTPUT $results9;
print OUTPUT $results10;
print OUTPUT $results11;
print OUTPUT $results12;
print OUTPUT $results13;
print OUTPUT $results14;
print OUTPUT $results15;
print OUTPUT $results16;
print OUTPUT $results17;
print OUTPUT $results18;
print OUTPUT $results19;
print OUTPUT "#DONE";
print STDERR "DONE ANALYSIS\n";
close(OUTPUT);
exit(0);

####################################################################################
################################################################SUBROUTINES
####################################################################################

sub getHG17Mapping() {
	my $id = shift;
	my $mapping_directory = "/home/smontgom/sNPeGEn/pipeline/11b_build_genome_mappings/hg17/";
	my $file = $mapping_directory . $id . ".hg17.mapped";
	unless (-e $file) { die("Cannot find mapping for $file\n"); }
	open(FILE, $file) || die("Cannot open mapping file $file\n");
	my $chromosome;
	my $start;
	my $end;
	my $strand;
	while (my $line = <FILE>) {
		chomp($line);
		if ($line =~ /([\S]+)[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([\S]+)/g) {
			$chromosome = $1;
			$start = $2;
			$end = $3;
			$strand = $4;
		}
	}
	close(FILE);
	return $chromosome, $start, $end, $strand;
}

sub getHG15Mapping() {
	my $id = shift;
	my $mapping_directory = "/home/smontgom/sNPeGEn/pipeline/11b_build_genome_mappings/hg15/";
	my $file = $mapping_directory . $id . ".hg15.mapped";
	unless (-e $file) { die("Cannot find mapping for $file\n"); }
	open(FILE, $file) || die("Cannot open mapping file $file\n");
	my $chromosome;
	my $start;
	my $end;
	my $strand;
	while (my $line = <FILE>) {
		chomp($line);
		if ($line =~ /([\S]+)[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([\S]+)/g) {
			$chromosome = $1;
			$start = $2;
			$end = $3;
			$strand = $4;
		}
	}
	close(FILE);
	return $chromosome, $start, $end, $strand;
}
