#!/usr/bin/perl

use strict;
use Bio::EnsEMBL::Registry;
use Chum::Analysis::Local::CoexpressionScanner;
use Chum::DB::Jaspar;
use Chum::Result::Interpretter::TransfacInterpretter;
use Chum::Utils::StackRandomizer;
use Chum::Utils::RSNPAnalysisXMLFileReader;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
my $flank_length = 10;
my $ensembl_gene_id = undef;
my $samples = 10;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnpa_xml_file = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-w/) {
        $flank_length = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-g/) {
        $ensembl_gene_id = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-s/) {
        $samples = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack = $rsnpa_obj->get_flanked_sequence_stack_obj($flank_length, 'hosa');

## setup analysis
my $cs = new Chum::Analysis::Local::CoexpressionScanner( 
		'coexpression_file' => "/home/obig/Projects/coexpression_db/tmm/ensembl_mapped/coexpression.tmm_links_all_human_ENS30_35c.new.txt",
        'opossum_install_dir' => "/home/smontgom/oPOSSUM/oPOSSUM");
my @coexpressed_genes = @{$cs->_get_coexpressed_genes($ensembl_gene_id, 1)};
print STDERR "Found " . scalar(@coexpressed_genes) . " coexpressed genes\n";
my ($factors_arrayref, $modifier_scores_hashref, $excluded_genes_arrayref) = $cs->_get_factors_for_coexpressed_genes(\@coexpressed_genes);
my $jaspar = new Chum::DB::Jaspar();
my $matrixset = $jaspar->get_matrix_set_for_names($factors_arrayref); 
my $resolving_quantity = &_get_matrixset_resolving_quantity($matrixset, $modifier_scores_hashref);
my $matrices = $matrixset->size;
my $scanner = new Chum::Analysis::Local::Transfac();  
$scanner->{'_matrix_set'} = $matrixset;

## run all samples and output
my $rscore = &_run_stack($scanner, $stack, $modifier_scores_hashref);
print "RESOLVING QUANTITY\t" . $resolving_quantity . "\n";
print "MATRIX COUNT\t" . $matrices . "\n";
print "REFERENCE\t" . $rscore . "\t" . $stack->reference_sequence->sequence->seq  . "\n";
my $stack_randomizer = new Chum::Utils::StackRandomizer();
for (my $i; $i < $samples; $i++) {
	my $test_stack = $stack_randomizer->generate_new_random_stack_from_dinucleotide_composition($stack);
	my $tscore = &_run_stack($scanner, $test_stack, $modifier_scores_hashref);
	print "RANDOM\t" . $tscore . "\t" . $test_stack->reference_sequence->sequence->seq  . "\n";
}
exit(0);

###############################################SUBROUTINES
sub _get_matrixset_resolving_quantity () {
	my $matrixset = shift;
	my $modifier_scores_hashref = shift;
	my %modifier_scores = %$modifier_scores_hashref;
	
	my $matrix_iterator = $matrixset->Iterator();
	my $resolving_quantity = 0;
	while (my $pwm = $matrix_iterator->next())  {
		my $difference = $pwm->max_score - $pwm->min_score;
		print STDERR %modifier_scores->{$pwm->name} . "\t" . $pwm->name . "\n";
		$resolving_quantity += ($difference / %modifier_scores->{$pwm->name});
	}
	return $resolving_quantity;
}

sub _run_stack () {	
	my $scanner = shift;
	my $stack = shift;
	my $modifier_scores_hashref = shift;

	my $resultset = $scanner->run_variation_analysis($stack);
	if (defined($resultset)) {
		my $tinterpretter = new Chum::Result::Interpretter::TransfacInterpretter();
		my $score = $tinterpretter->interpret_with_modifiers($resultset, $modifier_scores_hashref);
		return $score
	} else {
		return "N/A\n";
	}
}
