#!/usr/bin/perl
use strict;
use Bio::Tools::GFF;

my $rsnp_gff_file;
my $transcript_archival_file;
my $transcript_mapping_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnp_gff_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-t/) {
        $transcript_archival_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-m/) {
        $transcript_mapping_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnp_gff_file)) { die("No rSNP GFF file defined"); }
unless (-e $rsnp_gff_file) { die("Input rSNP GFF file does not exist"); }
if (!defined($transcript_archival_file)) { die("No input transcript archival file defined"); }
unless (-e $transcript_archival_file) { die("Input transcript archival file does not exist"); }
if (!defined($transcript_mapping_file)) { die("No input transcript mapping file defined"); }
unless (-e $transcript_mapping_file) { die("Input transcript mapping file does not exist"); }


############################################################OPEN TRANSCRIPT ARCHIVAL FILE
##
my %TRANSCRIPTINFO;
open(TRANS, $transcript_archival_file) || die("Could not open transcript archival file " . $transcript_archival_file);
while (my $line = <TRANS>) {
    chomp($line);
    if ($line =~ /(ENST[0-9]+)[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([\S]+)/g) {
        my $transid = $1;
        my $start = $2;
        my $end = $3;
        my $strand = $4;
        $TRANSCRIPTINFO{$transid}{'start'} = $start;
        $TRANSCRIPTINFO{$transid}{'end'} = $end;
        $TRANSCRIPTINFO{$transid}{'strand'} = $strand;
    }
}
close(TRANS);

############################################################OPEN TRANSCRIPT MAPPING FILE
##
my %TRANSCRIPTMAP;
open(TM, $transcript_mapping_file) || die ("Could not open transcript mapping file: " . $transcript_mapping_file);
while (my $line = <TM>) {
    chomp($line);
    if ($line =~ /(OREG[0-9]+)[\S\s]+:(ENST[\S]+)/g) {
	my $oregid = $1;
        my $transid = $2;
        $TRANSCRIPTMAP{$oregid} = $transid;
    } else {
        close(TM);
        die("Unknown transcript mapping line: " . $line);
    }
}
close(TM);

############################################################PROCESS DISTANCES
##
my %DISTANCES;
my $gffio = Bio::Tools::GFF->new(-file => $rsnp_gff_file, -gff_version => 2);
while(my $feature = $gffio->next_feature()) {
    my $chromosome = $feature->seq_id;
    my $start = $feature->start;
    my $end = $feature->end;
    my $strand = $feature->strand;
    my $oregid = $feature->primary_tag;
    my $transid = $TRANSCRIPTMAP{$oregid};
    my $transcriptinfo = $TRANSCRIPTINFO{$transid};
    print STDERR "Analyzing $oregid:$transid\n";
    if ($transcriptinfo->{'strand'} == -1) {
        $DISTANCES{$oregid} = $start - $transcriptinfo->{'end'};
        
    } elsif ($transcriptinfo->{'strand'} == 1) {
        $DISTANCES{$oregid} = $transcriptinfo->{'start'} - $start;
    
    } else {
        warn("Undefined strand: " . $transcriptinfo->{'strand'} . " for " . $oregid . ":" . $transid);
        next;   
    }
}
$gffio->close; 
#my @files = glob($results_directory . "/*.results");
#foreach my $file (@files) {
#    if ($file =~ /[\S\s]+OREG[0-9]+[\S\s]+/g) {    
#        my $result = new Chum::Statistics::AnalysisResult( 'file' => $file);
#        if (!defined($result->{'bend'}->{'score'}) || length($result->{'bend'}->{'score'}) == 0 || $result->{'bend'}->{'score'} eq "UNDEFINED") {
#            print STDERR "Skipping " . $result->{'id'} . "\n";  ##ENSURE THAT WE SKIP NON-SNPS
#            next;  ##PROBABLY NOT A SNP
#        }
#        my $transcript_stable_id = $TRANSCRIPTMAP{$result->{'id'}}; 
#        my $closest_gene = $result->{'distance'}->{$transcript_stable_id}->{'gene'};
#        my $closest_distance = $result->{'distance'}->{$transcript_stable_id}->{'distance'};
#        $DISTANCES{$result->{'id'}} = $closest_distance;
#    }
#}

my $size = scalar(keys(%DISTANCES));
my $printed = 0;
foreach my $id (sort { $DISTANCES{$a} <=> $DISTANCES{$b} } keys %DISTANCES) {
    print $id . "\t" . $DISTANCES{$id} . "\t";
    my $dens = 1 - ($printed++ / $size);
    print $dens . "\n";
}