#!/usr/bin/perl

use strict;
use Bio::Tools::GFF;
use Data::Dumper;
use SOAP::Lite;

my $gff_file = $ARGV[0];
if (!defined($gff_file)) { die("No GFF file defined"); }
unless(-f $gff_file) { die("The specified GFF file does not exist: " . $gff_file); }

my $transcript_mapping_file = $ARGV[1];
if (!defined($transcript_mapping_file)) { die("No Transcript Mapping file defined"); }
unless(-f $transcript_mapping_file) { die("The specified Transcript Mapping file does not exist: " . $transcript_mapping_file); }
my %TRANSCRIPT_MAP;
open(TRANS, $transcript_mapping_file) || die("Cannot open transcript mapping file");
while (my $line = <TRANS>) {
    chomp($line);
    if ($line =~ /(OREG[0-9]+)[\s]+[\S]+[\s]+[\S]+:([\S]+)/g) {
       $TRANSCRIPT_MAP{$1} = $2;
    } else {
        close(TRANS);
        die("TRANSCRIPTS Undefined line: " . $line);
    }
}
close(TRANS);

my $snp_mapping_file = $ARGV[2];
if (!defined($snp_mapping_file)) { die("No SNP Mapping file defined"); }
unless(-f $snp_mapping_file) { die("The specified SNP Mapping file does not exist: " . $snp_mapping_file); }
my %SNP_MAP;
open(SNPS, $snp_mapping_file) || die("Cannot open SNP mapping file");
while (my $line = <SNPS>) {
    chomp($line);
    if ($line =~ /(OREG[0-9]+)[\s]+([\S]+)/g) {
       $SNP_MAP{$1} = $2;
    }
}
close(SNPS);

my %EVIDENCE_LINK;
my %USED_EVIDENCE_SUBTYPES;
my %GENEIDMAP;
my $gffio = Bio::Tools::GFF->new(-file => $gff_file, -gff_version => 2);
eval {
    my $index = 0;
    while (my $feature = $gffio->next_feature()) {
        my $oreganno_stable_id = $feature->primary_tag();
        print STDERR "Testing " . $oreganno_stable_id . "\n";
        
        ##GET RECORD FROM OREGANNO;
        my $osa = SOAP::Lite
            -> uri('OregannoServerImpl')
            -> proxy('http://www.oreganno.org/oregano/soap/',
                    proxy=>['http'=>'http://wwwcache.sanger.ac.uk:3128'],  timeout => 1000);
        my $response = $osa->fetchRecord(SOAP::Data->name(record_stable_id => SOAP::Data->type(string => $oreganno_stable_id)));
        if ($response->fault) {
            $gffio->close();
            die $response->faultstring;
        } else {
            my $record = $response->result;
            my @evidences = @{$record->{'record'}->{'evidenceSet'}->{'evidence'}};
            my $gene = $record->{'record'}->{'geneName'};
            $GENEIDMAP{$oreganno_stable_id} = $record->{'record'}->{'geneId'};
            foreach my $evidence (@evidences) {
                my $evidence_subtype = $evidence->{'evidenceSubtypeStableId'};
                if (defined($EVIDENCE_LINK{$gene}{$oreganno_stable_id}{$evidence_subtype})) {
                    $EVIDENCE_LINK{$gene}{$oreganno_stable_id}{$evidence_subtype}++;
                } else {
                    $EVIDENCE_LINK{$gene}{$oreganno_stable_id}{$evidence_subtype} = 1;
                }
                $USED_EVIDENCE_SUBTYPES{$evidence_subtype} = 1;
            }
        }
        #if ($index++ == 6) {
        #    last;   
        #}
    }
};
if ($@) {
    print $@ . "\n";
    $gffio->close();
    exit(1);
} else {
    $gffio->close();   
}

my @sorted_subtypes = sort(keys(%USED_EVIDENCE_SUBTYPES));
print "GENE NAME\tOREGANNO STABLE ID";
foreach my $sorted_subtype (@sorted_subtypes) {
    print "\t" .  $sorted_subtype;
}
print "\n";
foreach my $gene (sort(keys(%EVIDENCE_LINK))) {
    my $hashref = %EVIDENCE_LINK->{$gene};
    foreach my $oreganno_id (sort(keys(%$hashref))) {
        print $gene . " EntrezGene:" .$GENEIDMAP{$oreganno_id} . " EnsEMBL:" . $TRANSCRIPT_MAP{$oreganno_id};
        print "\t" . $oreganno_id . " dbSNP:" . $SNP_MAP{$oreganno_id};
        foreach my $evidence_subtype_id (@sorted_subtypes) {    
            if (defined($EVIDENCE_LINK{$gene}{$oreganno_id}{$evidence_subtype_id})) {
                print "\t" . $EVIDENCE_LINK{$gene}{$oreganno_id}{$evidence_subtype_id};   
            } else {
                print "\t";    
            }
        }
        print "\n";
    }
}
exit(0);