#!/usr/bin/perl

use Bio::Graphics;
use Bio::SeqFeature::Generic;
use Chum::Statistics::AnalysisResult;
use Data::Dumper;

my %USE_TRANSCRIPT_FOR_GENE;
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000012061'} = "ENST00000300853";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000104725'} = "ENST00000311854";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000188170'} = "ENST00000335295";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000198668'} = "ENST00000356978";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000034971'} = "ENST00000037502";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000096746'} = "ENST00000265866";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000160868'} = "ENST00000336411";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000022355'} = "ENST00000023897"; 
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000100033'} = "ENST00000334029";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000138294'} = "ENST00000358559";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000163599'} = "ENST00000302823";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000057593'} = "ENST00000261967";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000060138'} = "ENST00000228251";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000182871'} = "ENST00000359759";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000165409'} = "ENST00000298171";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000198380'} = "ENST00000357308";
$USE_TRANSCRIPT_FOR_GENE{'ENSG00000118785'} = "ENST00000237623";

my $gene_group_file;
my $results_directory;
my $output_file = "output.png";
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $gene_group_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-d/) {
        $results_directory = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-o/) {
        $output_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if(!defined($gene_group_file)) { die("No gene group file defined"); }
unless(-f $gene_group_file) { die("Gene group file: " . $gene_group_file . " does not exist"); }
if(!defined($results_directory)) { die("No results directory has been defined"); }
unless (-d $results_directory) { die("Results directory: " . $results_directory . " does not exist"); }

my $error;
my %GENE_GROUP_HASH;
open(GENEGROUP, $gene_group_file) || die("Cannot open gene group file");
while (my $line = <GENEGROUP>) {
    chomp($line);
    if ($line =~ /([\S\s]+):[\s]*([\S\s]*)/g) {
        my $gene = $1;
        my $snp_line = $2;
        my @snps = split /[\s]+/, $snp_line;
        if (defined($GENE_GROUP_HASH{$gene})) {
            $error = "Gene is defined twice in gene group file: " . $gene . "\n";
            last;
        } else {
            $GENE_GROUP_HASH{$gene} = \@snps;
        }
    } else {
        $error = "Undefined line: " . $line . "\n";
        last;
    }
}
close(GENEGROUP);
if (defined($error)) { die($error); }

my $panel = Bio::Graphics::Panel->new(-length => 6000,
                                      -key_style => 'left',
                                      -width => 800,
                                      -pad_left => 100,
                                      -pad_right => 100);
my $track = $panel->add_track(  new Bio::SeqFeature::Generic(-start => 0,
                                                             -end => 6000,
                                                             -display_name => "6kb upstream region"),
                                -glyph => 'arrow',
                                -tick => 2,
                                -fgcolor => 'black',
                                -double => 1,
                                -label => 1);


foreach my $gene (keys %GENE_GROUP_HASH) {
    print STDERR "Processing $gene\n";
    
    ##GET ALL GENE DISTANCES FROM TRANSCRIPTS
    my @snps = @{$GENE_GROUP_HASH{$gene}};
    my %AR_RESULTS_HASH;
    foreach my $snp (@snps) {
        my $analysis_result_file = $results_directory . "/" . $snp . ".results";
        print STDERR "Opening " . $analysis_result_file . "\n";
        my $ar = new Chum::Statistics::AnalysisResult( 'file' => $analysis_result_file);
        $AR_RESULTS_HASH{$snp} = $ar;
    }
    print Dumper @snps;
    
    ##GET THE NUMBER OF TRANSCRIPTS IN THE GROUP
    my %TRANSCRIPT_COUNT_HASH;
    foreach my $snp (keys %AR_RESULTS_HASH) {
        my $transcript_hashref = $AR_RESULTS_HASH{$snp}{'distance'};
        my $hit = 0;
        print STDERR $snp . ":\t";
        foreach my $transcript (keys %$transcript_hashref) {
            if ($transcript_hashref->{$transcript}->{'gene'} eq $gene) {
                print STDERR $transcript . "\t";
                if (!defined($TRANSCRIPT_COUNT_HASH{$transcript})) {
                    $TRANSCRIPT_COUNT_HASH{$transcript} = 1;
                    $hit = 1;
                } else {
                    $TRANSCRIPT_COUNT_HASH{$transcript} += 1;
                    $hit = 1;
                }
            }
        }
        print STDERR "\n";
        if (!$hit) {
            print STDERR "*****Error: could not find matching hit for: " . $snp . "\n";   
        }
    }
    print Dumper \%TRANSCRIPT_COUNT_HASH;
    
    ##FIND THE TRANSCRIPT THAT COVER THE MOST SNPS, AND HAS THE LOWEST DISTANCE VALUES
    my $highest_score;
    my @HIGHEST_SCORING_TRANSCRIPTS = ();
    foreach my $transcript (sort {$TRANSCRIPT_COUNT_HASH{$b} <=> $TRANSCRIPT_COUNT_HASH{$a}} keys %TRANSCRIPT_COUNT_HASH) {
        if (!defined($highest_score)) {
            $highest_score = $TRANSCRIPT_COUNT_HASH{$transcript};
            push @HIGHEST_SCORING_TRANSCRIPTS, $transcript;
        } else {
            if ($TRANSCRIPT_COUNT_HASH{$transcript} != $highest_score) {
                last;   
            } else {
                push @HIGHEST_SCORING_TRANSCRIPTS, $transcript;    
            }
        }
    }
    if ($highest_score != scalar(@snps)) {
        print Dumper @HIGHEST_SCORING_TRANSCRIPTS;
        die("Could not find matches through group: " . $gene . " count: " . $highest_score);   
    }
    
    my %TRANSCRIPTS_IN_REGION_COUNT = ();
    foreach my $transcript (@HIGHEST_SCORING_TRANSCRIPTS) {
        foreach my $snp (keys %AR_RESULTS_HASH) {
            print STDERR "DISTANCE: " . $AR_RESULTS_HASH{$snp}->{'distance'}->{$transcript}->{'distance'} . " for " . $transcript . "\n";
            if ($AR_RESULTS_HASH{$snp}->{'distance'}->{$transcript}->{'distance'} <= 6000) {
                if (!defined($TRANSCRIPTS_IN_REGION_COUNT{$transcript})) {
                    $TRANSCRIPTS_IN_REGION_COUNT{$transcript} = 1;
                } else {
                    $TRANSCRIPTS_IN_REGION_COUNT{$transcript} += 1;
                }
            } else {
                print STDERR $snp . " is outside region at: " . $AR_RESULTS_HASH{$snp}->{'distance'}->{$transcript}->{'distance'} . "\n";
            }
        }
    }
    print Dumper \%TRANSCRIPTS_IN_REGION_COUNT;
    
    my @TRANSCRIPT_TO_USE;
    if (scalar(keys(%TRANSCRIPTS_IN_REGION_COUNT)) > 1) {
        if (defined($USE_TRANSCRIPT_FOR_GENE{$gene})) {
            push @TRANSCRIPT_TO_USE, $USE_TRANSCRIPT_FOR_GENE{$gene};
        } else {
            die("Could not find unique transcript for gene: " . $gene);   
        }
    } elsif (scalar(keys(%TRANSCRIPTS_IN_REGION_COUNT)) == 1) {
        my @keys = keys(%TRANSCRIPTS_IN_REGION_COUNT);
        push @TRANSCRIPT_TO_USE, $keys[0];
    } else {
        die("No transcripts found in region for gene: " . $gene);
    }
    
    print STDERR "TOUSE: \n";
    print Dumper \@TRANSCRIPT_TO_USE;
    if (!defined(@TRANSCRIPT_TO_USE)) {
        die("Could not find transcript to use for gene: " . $gene);   
    } elsif (scalar(@TRANSCRIPT_TO_USE) != 1) {
        die("Could not find transcript to use for gene: " . $gene);   
    }
    my $transcript_to_use = $TRANSCRIPT_TO_USE[0];
    
    ##DRAW GENE TRACKS WITH FEATURES, COLOR THE KNOWNS RED, THE UNKNOWNS BLUE
    
    my $snp_track = $panel->add_track(  -glyph => 'generic',
                                        -fgcolor => sub {
                                            my $feature = shift;
                                            if ($feature->display_name =~ /rs[0-9]+/g) {
                                                return 'blue';   
                                            } elsif ($feature->display_name =~ /OREG[0-9]+/g) {
                                                return 'red';   
                                            } else {
                                                return 'black';   
                                            }
                                        },
                                        -label => 1,
                                        -key => $gene);
    foreach my $snp (keys %AR_RESULTS_HASH) {
        my $snp_feature = Bio::SeqFeature::Generic->new(  -score => 1,
                                                          -start => $AR_RESULTS_HASH{$snp}->{'distance'}->{$transcript_to_use}->{'distance'},
                                                          -end => $AR_RESULTS_HASH{$snp}->{'distance'}->{$transcript_to_use}->{'distance'},
                                                          -display_name => $snp);
        $snp_track->add_feature($snp_feature);
    }
}
open(PNG, ">" . $output_file) || die("Cannot open " . $output_file);
binmode(PNG);
print PNG $panel->png;
close(PNG);
