#!/usr/bin/perl

use strict;
use Data::Dumper;

my $group_file;
my $gene_mapping_file;
my $dnsnp_mapping_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-g/) {
        $group_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-m/) {
        $gene_mapping_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-s/) {
        $dnsnp_mapping_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if(!defined($group_file)) { die("No group file defined"); }
unless(-f $group_file) { die("Group file: " . $group_file . " does not exist"); }
if(!defined($gene_mapping_file)) { die("No gene mapping file defined"); }
unless(-f $gene_mapping_file) { die("Gene mapping file: " . $gene_mapping_file . " does not exist"); }
if(!defined($dnsnp_mapping_file)) { die("No dbSNP mapping file defined"); }
unless(-f $dnsnp_mapping_file) { die("dbSNP mapping file: " . $dnsnp_mapping_file . " does not exist"); }

#################################################OPEN GROUP FILE, CREATE GROUP HASH
my %GROUP_HASH;
my $error = undef;
open(GROUP, $group_file) || die("Cannot open group file " . $group_file);
while (my $line = <GROUP>) {
    chomp($line);
    if ($line =~ /^([\S]+):[\s]+([\S\s]+)/g) {
        my $group_id = $1;
        my $group_line = $2;
        my @group_ids = split /[\s]+/, $group_line;
        if (defined($GROUP_HASH{$group_id})) {
            $error = "Group id " . $group_id . " already defined";
            last;    
        }
        $GROUP_HASH{$group_id} = \@group_ids;
    }                       
}
close(GROUP);
if (defined($error)) { die($error); }

#################################################GET ALL GENE MAPPINGS
my %GENE_HASH;
open(GENE, $gene_mapping_file) || die("Cannot open gene mapping file: " . $gene_mapping_file);
while (my $line = <GENE>) {
    chomp($line);
    my @tokens = split '\t', $line;
    my $oreganno_id;
    my $ensembl_id;
    my $ncbi_id;
    foreach my $token (@tokens) {
        if ($token =~ /ORegAnno:(OREG[0-9]+)/gi) { $oreganno_id = $1;
        } elsif ($token =~ /EnsEMBL:([\S]+)/gi) { $ensembl_id = $1;
        } elsif ($token =~ /EntrezGene:([0-9]+)/gi) { $ncbi_id = $1;
        } else { $error = "Unknown token in gene mapping file: " . $token; last; }
    }
    $GENE_HASH{$oreganno_id}->{'ncbi'} = $ncbi_id;
    $GENE_HASH{$oreganno_id}->{'ensembl'} = $ensembl_id;
}
close(GENE);
if (defined($error)) { die($error); }

#################################################GET ALL DBSNP MAPPINGS
my %DBSNP_HASH;
open(DBSNP, $dnsnp_mapping_file) || die("Cannot open dbSNP mapping file: " . $dnsnp_mapping_file);
while (my $line = <DBSNP>) {
    chomp($line);
    if ($line =~ /(OREG[0-9]+)[\s]+(rs[0-9]+)/gi) {
        $DBSNP_HASH{$1} = $2;
    } elsif ($line =~ /(OREG[0-9]+)/gi) {
        #ignore, no mapping   
    } else {
        $error = "Undefined line: " . $line;
        last;
    }
}
close(DBSNP);
if (defined($error)) { die($error); }

#################################################CREATE NON-REDUNDANT GENE GROUPS
my %GENE_GROUPS_HASH;
foreach my $group (keys %GROUP_HASH) {
    my $gene_name = $group;
    my $is_oreganno_id = 0;
    if ($group =~ /(OREG[0-9]+)/gi) {
	$is_oreganno_id = 1;
        $gene_name = $GENE_HASH{$group}{'ensembl'};    
	if (!defined($gene_name)) {
		die("No EnsEMBL gene name for " . $group);
	}
    }
    print STDERR "Processing " . $group . " " . $gene_name . "\n";
    if (!defined($GENE_GROUPS_HASH{$gene_name})) {
        ##NOT DEFINED, PUT IN GROUP IDS
        my @ids = @{$GROUP_HASH{$group}};
        if ($is_oreganno_id) {
            my $dbsnp_id = $DBSNP_HASH{$group};
            my $match = 0;
            if (defined($dbsnp_id)) {
                for (my $i = 0; $i < scalar(@ids); $i++) {
                    if ($ids[$i] eq $dbsnp_id) {
                        $ids[$i] = $group;
                        $match = 1;
                    }
                }
            }
            if (!$match) {
                push @ids, $group;
            }
        }
        $GENE_GROUPS_HASH{$gene_name} = \@ids;
    } else {
        ##ALREADY DEFINED, GET NON-REDUNDANT LIST OF IDS
        print STDERR $group . " is already defined\n";
	my @existing_ids = @{$GENE_GROUPS_HASH{$gene_name}};
        my %tmp_id_hash;
        foreach my $existing_id (@existing_ids) {
            $tmp_id_hash{$existing_id} = 1;
        }
        my @new_ids = @{$GROUP_HASH{$group}};
        foreach my $new_id (@new_ids) {
            $tmp_id_hash{$new_id} = 1;
        }
        my @ids = ();
        foreach my $key (keys %tmp_id_hash) {
            push @ids, $key; 
        }
        if ($is_oreganno_id) {
            my $dbsnp_id = $DBSNP_HASH{$group};
            my $match = 0;
            if (defined($dbsnp_id)) {
                for (my $i = 0; $i < scalar(@ids); $i++) {
                    if ($ids[$i] eq $dbsnp_id) {
                        $ids[$i] = $group;
                        $match = 1;
                    }
                }
            }
            if (!$match) {
                push @ids, $group;
            }
        }
        $GENE_GROUPS_HASH{$gene_name} = \@ids;
      }
}
#1. Output groups
foreach my $gene_name (keys %GENE_GROUPS_HASH) {
    print $gene_name . ":\t";
    foreach my $id (@{$GENE_GROUPS_HASH{$gene_name}}) {
        print $id . "\t";   
    }
    print "\n";
}
exit(0);