#!/usr/bin/perl -w
use strict;
use XML::Twig;

##############################PROCESS INPUTS##############################
my $input_file = undef;
my $matrix_input_file = undef;
my $output_file = undef;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-h/ || $ARGV[$i] =~ /--help/) {
        print STDERR "motifsamplerToXML.pl usage\n";
        print STDERR "\t-f: specify input file.\n";
        exit(1);
    } elsif ($ARGV[$i] =~ /-o/) {
        $output_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-f/) {
        $input_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-m/) {
        $matrix_input_file = $ARGV[++$i];
    } else {
        print STDERR "ERROR: Undefined option: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($input_file) || !(-f $input_file)) {
    print STDERR "ERROR: Input file is undefined or does not exist, -f: " . $input_file . "\n";
    exit(1);    
}
if (defined($matrix_input_file)) {
    unless (-f $matrix_input_file) {
        print STDERR "ERROR: Matrix input file does not exist, -m: " . $input_file . "\n";
        exit(1);    
    }
}

##############################READ IN INPUT##############################
my @lines = ();
open(INPUT, $input_file) || die("Cannot open input file: " . $input_file . "\n");
while (my $line = <INPUT>) { push @lines, $line; }
close(INPUT);
if (scalar(@lines) == 0) { die("ERROR: Input is empty\n"); }

my @matrixlines = ();
if (defined($matrix_input_file)) {
    open(MATRIX, $matrix_input_file) || die("Cannot open matrix input file: " . $matrix_input_file . "\n");
    while (my $line = <MATRIX>) { push @matrixlines, $line; }
    close(MATRIX);
    if (scalar(@matrixlines) == 0) { die("ERROR: Input matrix file is empty\n"); }
}

##############################TRANSFORM OUTPUT INTO XML##############################
my $root = XML::Twig::Elt->new('motifsampler_output');

my $predicted_motif_tag = undef;
my $current_motif_name = undef;
my @hitsequences = ();
foreach my $line (@lines) {
    chomp($line);
    if ($line =~ /[\S]+id:[\s]+([\S]+)[\s]+consensus:[\s]+([\S]+)[\s]+sequences:[\s]+([0-9]+)
                    [\s]+instances:[\s]+([0-9]+)[\s]+cs:[\s]+([\S]+)[\s]+ic:[\s]+([\S]+)
                    [\s]+ll:[\s]+([\S]+)/xg) {
        my $id = $1;
        my $consensus = $2;
        my $sequences_count = $3;
        my $hit_count = $4;
        my $consensus_score = $5;
        my $information_content = $6;
        my $log_likelihood = $7;
        
        if (defined($predicted_motif_tag)) {
            my $matrix_tag = &writeMatrix($current_motif_name,
                         \@matrixlines,
                        \@hitsequences,
                        $predicted_motif_tag);
            @hitsequences = ();
            $matrix_tag->paste($predicted_motif_tag);
            $predicted_motif_tag->paste($root);
        }
        
        $current_motif_name = $id;
        $predicted_motif_tag = XML::Twig::Elt->new('predicted_motif');
        $predicted_motif_tag->set_att('seq', $consensus);
        $predicted_motif_tag->set_att('score', $log_likelihood);
    } elsif  ($line =~ /([\S]+)[\s]+MotifSampler[\s]+misc_feature[\s]+([0-9]+)[\s]+([0-9]+)[\s]+
                    ([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+id[\s]+[\S]{1}([\S]+)[\S]{2}[\s]+site[\s]+
                    [\S]{1}([\S]+)[\S]{2}/xg) {
        my $name = $1;
        my $start = $2;
        my $end = $3;
        my $score = $4;
        my $strand = $5;
        my $phase = $6;
        my $id = $7;
        my $sequence = $8;
        
        if ($strand eq "-") { $strand = -1; }
        elsif ($strand eq "+") { $strand = 1; }
        
        my $hit_tag = XML::Twig::Elt->new('hit');
        $hit_tag->set_att('name', $name);
        $hit_tag->set_att('start', $start);
        $hit_tag->set_att('score', $score);
        $hit_tag->set_att('strand', $strand);
        $hit_tag->set_att('sequence', $sequence);
        $hit_tag->paste($predicted_motif_tag);
        
        push @hitsequences, $sequence;
    }
}
if (defined($predicted_motif_tag)) {
    my $matrix_tag = &writeMatrix($current_motif_name,
    \@matrixlines,
    \@hitsequences,
    $predicted_motif_tag);
    @hitsequences = ();
    $matrix_tag->paste($predicted_motif_tag);
    $predicted_motif_tag->paste($root);
}

##############################OUTPUT RESULTS##############################
$root->set_pretty_print( 'indented');
if (defined($output_file)) {
    open(OUT, ">" , $output_file) || die("Cannot open output file");
    $root->print( \*OUT );
    close(OUT);
} else {
    $root->print;
}
exit(0);

##############################SUBROUTINES##############################

sub writeMatrix {
    my $current_motif_name = shift;
    my $matrixlines_arrayref = shift;
    my $hitsequences_arrayref = shift;
    my $predicted_motif_tag = shift;
    
    my @matrixlines = @$matrixlines_arrayref;
    my $matrix_tag = undef;
    if (scalar(@matrixlines) > 0) {
        my $open = 0;
        my $index = 1;
        foreach my $line (@matrixlines) {
            chomp($line);
            if ($line =~ /[\S]{1}ID[\s\S]+$current_motif_name/g) {
                if (!$open) {
                    $open = 1;
                    $matrix_tag = XML::Twig::Elt->new('matrix');
                    $matrix_tag->set_att('name', $current_motif_name);
                    $matrix_tag->set_att('type', "PFM");
                } else { $open = 0; }
            } elsif ($line =~ /[\S]{1}ID[\s\S]+[\S]+/g) {
                if ($open) {
                    $open = 0;   
                }
            } elsif ($open) {
                if ($line =~ /([\S]+)[\s]+([\S]+)[\s]+([\S]+)[\s]+([\S]+)/g) {
                    my $index_tag = XML::Twig::Elt->new('index');
                    $index_tag->set_att('position', $index++);
                    $index_tag->set_att('A', $1);
                    $index_tag->set_att('C', $2);
                    $index_tag->set_att('G', $3);
                    $index_tag->set_att('T', $4);
                    $index_tag->paste($matrix_tag);
                }
            }
        }
    }
    if (!defined($matrix_tag)) {
        my @hitsequences = @$hitsequences_arrayref;
        if (scalar(@hitsequences) > 0) {
            $matrix_tag = XML::Twig::Elt->new('matrix');
            $matrix_tag->set_att('name', $current_motif_name);
            $matrix_tag->set_att('type', "PFM");
        
            my $length = length($hitsequences[0]);
            my $count = scalar(@hitsequences);
            for (my $i = 0; $i < $length; $i++) {
                my $As = 0;
                my $Cs = 0;
                my $Gs = 0;
                my $Ts = 0;
                
                foreach my $hitsequence (@hitsequences) {
                    my $char = uc(substr($hitsequence, $i, 1)); 
                    if ($char eq 'A') { $As++;}
                    elsif ($char eq 'C') { $Cs++;}
                    elsif ($char eq 'G') { $Gs++;}
                    elsif ($char eq 'T') { $Ts++;}
                    else { die("Unknown character: " . $char); }
                }
                
                my $index_tag = XML::Twig::Elt->new('index');
                $index_tag->set_att('position', $i+1);
                $index_tag->set_att('A', $As / $count);
                $index_tag->set_att('C', $Cs / $count);
                $index_tag->set_att('G', $Gs / $count);
                $index_tag->set_att('T', $Ts / $count);
                $index_tag->paste($matrix_tag);
            }
        }
    }
    return $matrix_tag;
}