#!/usr/bin/perl

###MAPS AN OREGANNO ID (RECORD) TO THE HUMAN GENOME, ALLOWS FOR ENSEMBL TO ENTREZ MAPPING IF -g 1 SET

###FOR THIS SCRIPT TO WORK WE HAVE SET UP A BLAST DATABASE CALLED HG17.FA USING THE BLASTDB
###ENVIRONMENT VARIABLE AND BLAST2.2.6

use strict;

$ENV{'BLASTDB'} = "/home/smontgom/databases/blast";
$ENV{'ENSEMBL_REGISTRY'} = "/home/smontgom/perllibs/ensembl/cisred-ensembl-config/ensembl-config";

use lib "/home/smontgom/perllibs/DBI-1.45/lib";
use lib "/home/smontgom/perllibs/bioperl/bioperl-live/blib/lib";
use lib "/home/smontgom/perllibs/ensembl/ensembl-30/ensembl/modules";
use lib "/home/smontgom/perllibs/ensembl/ensembl-30/ensembl-compara/modules";
use lib "/home/smontgom/perllibs/ensembl/ensembl-30/ensembl-variation/modules";
use lib "/home/smontgom/sNPeGEn/modules";
use lib "/home/smontgom/sNPeGEn/dependencies/chinook/perl/modules";

use Data::Dumper;
use Bio::EnsEMBL::Registry;
use SOAP::Lite;
use Snpegen::Utils::GenomeCoordinateLocator;

#######################################################################################################
##LOAD EACH COMMAND-LINE PARAMETER INTO THE APPROPRIATE VARIABLE#######################################
#######################################################################################################
my $oregid;
my $ensembl_gene_map = 0; #default false
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-i/) {
        $oregid = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-g/) {
		$ensembl_gene_map = $ARGV[++$i];
    } else {
        print "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($oregid)) {
        print "\nSome parameters not defined, ";
        print "usage ./oregMap.pl ";
        print "-i OREG00000001\n";
        exit(1);      
} 


#######################################################################################################
##GET ID FROM OREGANNO#################################################################################
#######################################################################################################
my $osa = SOAP::Lite
          -> uri('OregannoServerImpl')
          -> proxy('http://www.bcgsc.ca:8080/oregano/soap/');

my $response = $osa->searchRecords(
				SOAP::Data->name(field => SOAP::Data->type(string => "stable_id")),
                SOAP::Data->name(query => SOAP::Data->type(string => $oregid)));

my $record;
if ($response->fault) {
		die $response->faultstring;
} else {
      	my @results = @{$response->result};
		if (scalar(@results) != 1) {
			die "Could not find record";
		} else {
			$record = @results[0]
        }
}

if ($record->{'record'}->{'speciesName'} ne "Homo sapiens") {
	die "Script only supports human mapping";
}
print STDERR "Found " . $record->{'record'}->{'stableId'} . " " . $record->{'record'}->{'type'} . "\n";

#######################################################################################################
##ENSEMBL GENE MAP#####################################################################################
#######################################################################################################

##If not specified and mapping required, find the corresponding ensembl_gene_id from a Entrez Gene ID
my $ens_gene_stable_id;
if ($ensembl_gene_map) {
	if ($record->{'record'}->{'geneSource'} eq "NCBI") {
		my $reg = "Bio::EnsEMBL::Registry";
		$reg->load_all();

		my $ga = $reg->get_adaptor("human", "core", "Gene");
		my @genes = @{$ga->fetch_all_by_external_name($record->{'record'}->{'geneName'})};

		if (defined(@genes)) {
			if (scalar(@genes) == 1) {
				$ens_gene_stable_id = @genes[0]->stable_id;
			} else {
				print STDERR scalar(@genes) . " Could not find EnsEMBL coordinates for " . $record->{'record'}->{'geneName'} . 
							 " " . $record->{'record'}->{'stableId'}  . "\n";
			}
		}
		
	}
}


#######################################################################################################
##BLAST SEQUENCE ######################################################################################
#######################################################################################################

my $hits;
if (($record->{'record'}->{'type'} eq "REGULATORY REGION") || ($record->{'record'}->{'type'} eq "TRANSCRIPTION FACTOR BINDING SITE")) {
	if (!defined($record->{'record'}->{'sequenceWithFlank'})) {
		die "No sequence was found for record";
	}
	my $gcl = new Snpegen::Utils::GenomeCoordinateLocator(	'blast_db' => "hg17.fa");
	$hits = $gcl->locateSequence($record->{'record'}->{'sequenceWithFlank'});
} elsif ($record->{'record'}->{'type'} eq "REGULATORY POLYMORPHISM") {
	my @variations = @{$record->{'record'}->{'variationSet'}->{'variation'}};
	if (!defined(@variations[0]->{'referenceSequence'})) {
		die "No sequence was found for record";
	}
	my $gcl = new Snpegen::Utils::GenomeCoordinateLocator(	'blast_db' => "hg17.fa");
	$hits = $gcl->locateSequence($oregid, @variations[0]->{'referenceSequence'});
} else {
	die "Unsupported OREGANNO type: " . $record->{'record'}->{'type'};
}

#######################################################################################################
##GET AND OUTPUT COORDINATES ##########################################################################
#######################################################################################################

if (($record->{'record'}->{'type'} eq "REGULATORY REGION") || ($record->{'record'}->{'type'} eq "TRANSCRIPTION FACTOR BINDING SITE")) {
	foreach my $hit (@$hits) {
		print $oregid . "\t" . $hit->{'chromosome'} . "\t" . $hit->{'start'} . "\t" . 
			$hit->{'end'} . "\t" . $hit->{'strand'} . "\t.\t" . "ensGeneId " . $ens_gene_stable_id . ";geneId " .
			$record->{'record'}->{'geneId'} . ";geneSource " . $record->{'record'}->{'geneSource'}  . 
			";score " .  $hit->{'evalue'}  . "\n";
	}
} elsif ($record->{'record'}->{'type'} eq "REGULATORY POLYMORPHISM") {
	my $index = 1;
	foreach my $hit (@$hits) {
		my $variant_map = &findVariant($record, $hit);
		my $start;
		my $end;

		if ($hit->{'strand'} eq "-1" || $hit->{'strand'} eq "-") {
			$end = $hit->{'end'} - $variant_map->{'start'};
			$start = $hit->{'end'} - $variant_map->{'end'};

		} else {
			$start = $variant_map->{'start'} + $hit->{'start'};
			$end = $variant_map->{'end'} + $hit->{'start'};
		}

		print "REFERENCE-" . $index . "\t" . $hit->{'chromosome'} . "\tVARIANT\t" . $start . "\t" . 
				$end . "\t.\t" . $hit->{'strand'} . "\t.\tseq " .  $variant_map->{'reference'} . ";" .
				"ensGeneId " . $ens_gene_stable_id . ";geneId " .
				$record->{'record'}->{'geneId'} . ";geneSource " . $record->{'record'}->{'geneSource'}  . 
				";score " .  $hit->{'evalue'}  . "\n";
		print "VARIANT-" . $index . "\t" . $hit->{'chromosome'} . "\tVARIANT\t" . $start . "\t" . 
				$end . "\t.\t" . $hit->{'strand'} . "\t.\tseq " .  $variant_map->{'variant'} . ";" .
				"ensGeneId " . $ens_gene_stable_id . ";geneId " .
				$record->{'record'}->{'geneId'} . ";geneSource " . $record->{'record'}->{'geneSource'}  . 
				";score " .  $hit->{'evalue'}  . "\n";
	}
}

print STDERR "Done\n";
exit(0);

#######################################################################################################
##SUBROUTINES #########################################################################################
#######################################################################################################

##NEEDS TO BE MODIFIED FOR INDELS, ONLY WORKS ON SNPS AND EQUAL LENGTH POLYMORPHISMS
sub findVariant {
	my $record = shift;
	my $hit = shift;

	my @variations = @{$record->{'record'}->{'variationSet'}->{'variation'}};
	my $variation = @variations[0];

	if (length($variation->{'referenceSequence'}) eq length($variation->{'variantSequence'})) {
		my @referenceNts = split //, $variation->{'referenceSequence'};
		my @variantNts = split //, $variation->{'variantSequence'};
	
		my $open = 0;
		my $count = 0;
		my $start;
		my $end;
		my $reference = "";
		my $variant = "";

		for (my $i = 0; $i < @referenceNts; $i++) {
			if (@referenceNts[$i] eq @variantNts[$i]) {
				if ($open) {
					$end = $i - 1;
					$open = 0;
				}
				##ignore
			} else {
				if (!$open) {
					$start = $i;
					$open = 1;
				}
				$reference .= @referenceNts[$i];
				$variant .= @variantNts[$i];
				$count++;
			}
		}
		if ($count > 1) {
			print STDERR "DEBUG CHECK ON (MULTILENGTH HIT) " .  $record->{'record'}->{'stableId'} . "\n";
		}
		return {'reference' => $reference, 'variant' => $variant, 'start' => $start, 'end' => $end };

	} else {
		print STDERR "DEBUG CHECK ON (LENGTH) " .  $record->{'record'}->{'stableId'} . "\n";
		print $record->{'record'}->{'stableId'} . " does not have equivalent variant and reference sequences, cannot handle, yet\n";
	}

}

