#!/usr/bin/perl

use strict;
use Bio::EnsEMBL::Registry;
use Chum::Statistics::AnalysisResult;
require Chum::Utils::ExpectancyManager;
use Data::Dumper;

############################################################CONSTANTS
##
my $STARSON = 1;
my $NORMALIZE_ALL_EXPECTANCIES = 1;
my $EXPECTANCY_LOADER = undef;
if ($NORMALIZE_ALL_EXPECTANCIES) { $EXPECTANCY_LOADER = new Chum::Utils::ExpectancyManager(); }
Bio::EnsEMBL::Registry->load_all();

############################################################SETUP PROGRAM
##
my $results_file;
my $columns_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $results_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-c/) {
        $columns_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($results_file)) { die("No input file defined"); }
unless (-e $results_file) { die("Input file does not exist"); }
if (!defined($columns_file)) { die("No output columns file defined"); }

############################################################RUN TESTS
##
my @results = @{&loadFile($results_file)};
my $index = 1;
my %tests = (	"transfac" => $index++,
		"weeder-diff" => $index++,
		"weeder-max" => $index++,
		"motifsampler-diff" => $index++,
		"motifsampler-max" => $index++,
		"bend" => $index++,
		"curve" => $index++,
		"gc" => $index++,
		"temperature" => $index++,
		"distance" => $index++,
		"ln-distance" => $index++,
		"kaks-mouse" => $index++,
		"kaks-cow" => $index++,
		"kaks-dog" => $index++,
		"kaks-rat" => $index++,
		"kaks-chimp" => $index++,
		"repeat-density" => $index++,
		"in-repeat" => $index++,
		"repeat-short-events" => $index++,
		"repeat-long-events" => $index++,
		"maf" => $index++,
		"daf" => $index++,
		# "maf-rare" => $index++,
		# "maf-common" => $index++,
		# "maf-unknown" => $index++,
		"coexpression-jaspar" => $index++,
		"in-cpg" => $index++,
		"nhgri-dnase" => $index++,
		"noble-dnase" => $index++,
		"phastcons" => $index++,
		"regulatory-potential-x5" => $index++);

############################################################AVERAGE ALLELE FREQUENCY
##
my $average_maf = 0;
if (defined($tests{'maf'})) {
    my $known = 0;
    foreach my $result (@results) {
	my $allelefreq_hashref = $result->{'allele_frequency'};
	if ($allelefreq_hashref) {
	    for my $key (keys %$allelefreq_hashref) {
		if ($result->{'allele_frequency'}->{$key} < 0.5) {
		    $average_maf += $result->{'allele_frequency'}->{$key};
		    $known++;
		    last;
		}
	    }
	}
    }
    $average_maf = $average_maf / $known;
}
if ($average_maf == 0) { $average_maf = 0.5; }

############################################################OUTPUT DATA
##
print "corner";
foreach my $test (sort { $tests{$a} <=> $tests{$b} } keys %tests) {
	print "\t" . $test;
}
print "\n";

open(RES, ">" . $columns_file) || die ("Cannot open $columns_file file");
print RES "corner\trsnp\n";
foreach my $result (@results) {
    if (!defined($result->{'bend'}->{'score'}) || length($result->{'bend'}->{'score'}) == 0) {
	next;  ##PROBABLY NOT A SNP
    }
   
    my $closest_gene;
    my $closest_distance;
    my $closest_ln_distance;
    my $distance_hashref = $result->{'distance'};
    for my $key (keys %$distance_hashref) {
            if (!defined($closest_gene)) {
                    $closest_gene = $result->{'distance'}->{$key}->{'gene'};
                    $closest_distance = $result->{'distance'}->{$key}->{'distance'};
                    $closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
            } elsif ($result->{'distance'}->{$key}->{'distance'} < $closest_distance) {
                    $closest_gene = $result->{'distance'}->{$key}->{'gene'};
                    $closest_distance = $result->{'distance'}->{$key}->{'distance'};
                    $closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
            }
    }
    if (abs($closest_distance) > 6001) {
        next; ##NOT WITHIN SEARCH REGION WE ARE INTERESTED IN
    }

    
    if ($result->{'id'} =~ /OREG[0-9]+/) {
	print RES $result->{'id'} . "\t1\n";
    } elsif ($result->{'id'} =~ /rs[0-9]+/) {
	print RES $result->{'id'} . "\t0\n";
    }

    my $OUTPUT_LINE = "";

    $OUTPUT_LINE .= $result->{'id'} . "\t";
    if (defined($tests{'transfac'})) {
	if ($result->{'transfac'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'transfac'}->{'score'} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'weeder-diff'})) {
	if ($result->{'weeder'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'weeder'}->{'score'} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'weeder-max'})) {
	if ($result->{'weeder'}->{'max_score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'weeder'}->{'max_score'} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'motifsampler-diff'})) {
	if ($result->{'motifsampler'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'motifsampler'}->{'score'} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'motifsampler-max'})) {
	if ($result->{'motifsampler'}->{'max_score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'motifsampler'}->{'max_score'} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'bend'})) {
	if ($result->{'bend'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'bend'}->{'score'} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'curve'})) {
	if ($result->{'curve'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'curve'}->{'score'} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'gc'})) {
	if ($result->{'gc'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'gc'}->{'score'} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'temperature'})) {
	if ($result->{'temperature'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'temperature'}->{'score'} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
		
    ###DISTANCE
    if (defined($tests{'distance'})) {
	if (defined($closest_distance)) {
	    $OUTPUT_LINE .=  $closest_distance . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'ln-distance'})) {
	if (defined($closest_ln_distance)) {
	    $OUTPUT_LINE .=  $closest_ln_distance . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }

    ###KAKS
    my %resultskaks;
    my $kaks_hashref1 = $result->{'kaks'}->{$closest_gene};
    for my $key1 (keys %$kaks_hashref1) {
	my $kaks_hashref2 = $result->{'kaks'}->{$closest_gene}->{$key1};
	for my $key2 (keys %$kaks_hashref2) {
	    if (defined($result->{'kaks'}->{$closest_gene}->{$key1}->{$key2})) {
		if ($key1 eq "Mus musculus" || $key2 eq "Mus musculus") {
		    $resultskaks{"mumu"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
		} elsif ($key1 eq "Bos taurus" || $key2 eq "Bos taurus") {
		    $resultskaks{"bota"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
		} elsif ($key1 eq "Canis familiaris" || $key2 eq "Canis familiaris") {
		    $resultskaks{"cafa"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
		} elsif ($key1 eq "Rattus norvegicus" || $key2 eq "Rattus norvegicus") {
		    $resultskaks{"rano"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
		} elsif ($key1 eq "Pan troglodytes" || $key2 eq "Pan troglodytes") {
		    $resultskaks{"patr"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
		} else {
		    die("Should add test for $key1 $key2 in KAKS\n");
		}
	    }
        }
    }
    if (defined($tests{'kaks-mouse'})) {
	if (defined($resultskaks{"mumu"})) {
	    $OUTPUT_LINE .=  $resultskaks{"mumu"} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'kaks-cow'})) {
	if (defined($resultskaks{"bota"})) {
	    $OUTPUT_LINE .=  $resultskaks{"bota"} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'kaks-dog'})) {
	if (defined($resultskaks{"cafa"})) {
	    $OUTPUT_LINE .=  $resultskaks{"cafa"} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'kaks-rat'})) {
	if (defined($resultskaks{"rano"})) {
	    $OUTPUT_LINE .=  $resultskaks{"rano"} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'kaks-chimp'})) {
	if (defined($resultskaks{"patr"})) {
	    $OUTPUT_LINE .=  $resultskaks{"patr"} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    
    my $repeat_density = 0;
    if ($result->{'repeat'} != 0) {
	    $repeat_density = $result->{'repeat'}->{'repeat_length'} / $result->{'repeat'}->{'search_length'};
    }
    my $CHROMOSOME = &getChromosomeForGene($closest_gene);
    my $expectancy_adjustment_in_repeat_value = 0;
    my $expectancy_adjustment_short_event = 0;
    my $expectancy_adjustment_long_event = 0;
    if ($NORMALIZE_ALL_EXPECTANCIES) {
	$expectancy_adjustment_in_repeat_value = $EXPECTANCY_LOADER->get_in_repeat_expectancy_value($CHROMOSOME, (0 - $closest_distance));
	$expectancy_adjustment_short_event = $EXPECTANCY_LOADER->get_short_event_expectancy_value($CHROMOSOME, (0 - $closest_distance));
	$expectancy_adjustment_long_event = $EXPECTANCY_LOADER->get_long_event_expectancy_value($CHROMOSOME, (0 - $closest_distance));
    }

		
    if (defined($tests{'repeat-density'})) {
	$OUTPUT_LINE .=  $repeat_density . "\t";
    }
    if (defined($tests{'in-repeat'})) {
	my $adjvalue = $result->{'repeat'}->{'in_repeat'} - $expectancy_adjustment_in_repeat_value;
	$OUTPUT_LINE .=  $adjvalue . "\t";
    }
    if (defined($tests{'repeat-short-events'})) {
	my $adjvalue = $result->{'repeat'}->{'short_events'} - $expectancy_adjustment_short_event;
	$OUTPUT_LINE .=  $adjvalue . "\t";
    }
    if (defined($tests{'repeat-long-events'})) {
	my $adjvalue = $result->{'repeat'}->{'long_events'} - $expectancy_adjustment_long_event;
	$OUTPUT_LINE .=  $adjvalue . "\t";
    }
    

    ###MAF
    if (defined($tests{'maf'})) {
	my $allelefreq_hashref = $result->{'allele_frequency'};
	if ($allelefreq_hashref) {
	    my $printed = 0;
	    for my $key (keys %$allelefreq_hashref) {
			    #	if ($result->{'allele_frequency'}->{$key} < 0.05) {
			    #		if (defined(%tests->{'maf-rare'})) { print "1\t"; }
			    #		if (defined(%tests->{'maf-common'})) { print "0\t"; }
			    #		if (defined(%tests->{'maf-unknown'})) { print "0\t"; }
			    #		last;
			    #	} elsif ($result->{'allele_frequency'}->{$key} < 0.5) {
			    #		if (defined(%tests->{'maf-rare'})) { print "0\t"; }
			    #		if (defined(%tests->{'maf-common'})) { print "1\t"; }
			    #		if (defined(%tests->{'maf-unknown'})) { print "0\t"; }
			    #		last;
			    #	}
		if ($result->{'allele_frequency'}->{$key} < 0.5) {
		    if (defined($tests{'maf'})) {
			$OUTPUT_LINE .=  $result->{'allele_frequency'}->{$key} . "\t";
			$printed = 1;
		    }
		}
	    }
	    unless ($printed) {
		if (defined($tests{'maf'})) {
		    if ($STARSON) {
			$OUTPUT_LINE .=  "*" . "\t";
		    } else {
			$OUTPUT_LINE .=  "0.5" . "\t";
		    }
		}
	    }
	} else {
	    if (defined($tests{'maf'})) {
		if ($STARSON) {
		    $OUTPUT_LINE .=  "*" . "\t";
		} else {
		    $OUTPUT_LINE .=  "0.5" . "\t";
		}	
	    }
	    #if (defined(%tests->{'maf-rare'})) { print "0\t"; }
	    #if (defined(%tests->{'maf-common'})) { print "0\t"; }
	    #if (defined(%tests->{'maf-unknown'})) { print "1\t"; }
	}
    }

    if (defined($tests{'daf'})) {
	if ($result->{'daf'} ne "UNDEFINED" && defined($result->{'daf'})) {
	    $OUTPUT_LINE .=  $result->{'daf'} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0.5\t";
	    }
	}
    }
    if (defined($tests{'coexpression-jaspar'})) {
	if ($result->{'coexpression-jaspar'}->{'score'} ne "UNDEFINED") {
	    $OUTPUT_LINE .=  $result->{'coexpression-jaspar'}->{'score'} . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    
    my $expectancy_adjustment_in_cpg = 0;
    my $expectancy_adjustment_nhgri = 0;
    my $expectancy_adjustment_noble = 0;
    my $expectancy_adjustment_phastcons = 0;
    my $expectancy_adjustment_regulatory_potential = 0;
    if ($NORMALIZE_ALL_EXPECTANCIES) {
	$expectancy_adjustment_in_cpg = $EXPECTANCY_LOADER->get_in_cpg_expectancy_value($CHROMOSOME, (0 - $closest_distance));
	$expectancy_adjustment_nhgri = $EXPECTANCY_LOADER->get_nhgri_expectancy_value($CHROMOSOME, (0 - $closest_distance));
	$expectancy_adjustment_noble = $EXPECTANCY_LOADER->get_noble_expectancy_value($CHROMOSOME, (0 - $closest_distance));
	$expectancy_adjustment_phastcons = $EXPECTANCY_LOADER->get_phastcons_expectancy_value($CHROMOSOME, (0 - $closest_distance));
	$expectancy_adjustment_regulatory_potential = $EXPECTANCY_LOADER->get_regulatory_potential_expectancy_value($CHROMOSOME, (0 - $closest_distance));
    }
    if (defined($tests{'in-cpg'})) {
	if ($result->{'cpg'}->{'score'} ne "UNDEFINED") {
	    my $adjvalue = $result->{'cpg'}->{'score'} - $expectancy_adjustment_in_cpg;
	    $OUTPUT_LINE .=  $adjvalue . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'nhgri-dnase'})) {
	if ($result->{'nhgri-dnase'}->{'score'} ne "UNDEFINED") {
	    my $adjvalue = $result->{'nhgri-dnase'}->{'score'} - $expectancy_adjustment_nhgri;
	    $OUTPUT_LINE .=  $adjvalue . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'noble-dnase'})) {
	if ($result->{'noble-dnase'}->{'score'} ne "UNDEFINED") {
	    my $adjvalue = $result->{'noble-dnase'}->{'score'} - $expectancy_adjustment_noble;
	    $OUTPUT_LINE .=  $adjvalue . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'phastcons'})) {
	if ($result->{'phastcons'}->{'score'} ne "UNDEFINED") {
	    my $adjvalue = $result->{'phastcons'}->{'score'} - $expectancy_adjustment_phastcons;
	    $OUTPUT_LINE .=  $adjvalue . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    if (defined($tests{'regulatory-potential-x5'})) {
	if ($result->{'rp'}->{'score'} ne "UNDEFINED") {
	    my $adjvalue = $result->{'rp'}->{'score'} - $expectancy_adjustment_phastcons;
	    $OUTPUT_LINE .=  $adjvalue . "\t";
	} else {
	    if ($STARSON) {
		$OUTPUT_LINE .=  "*\t";
	    } else {
		$OUTPUT_LINE .=  "0\t";
	    }
	}
    }
    chomp($OUTPUT_LINE);
    $OUTPUT_LINE .=  "\n";
    print $OUTPUT_LINE;
}
close (RES);
exit(0);


############################################################PRIVATE SUBROUTINES
##
sub loadFile {
	my $file = shift;
	my @results = ();
	open(FILE, $file) || die ("Cannot open $file");
	foreach my $line (<FILE>) {
		chomp($line);
		push @results, new Chum::Statistics::AnalysisResult( 'file' => $line);
	}
	close(FILE);
	return \@results;
}

sub getChromosomeForGene() {
    my $stable_id = shift;
    my $ga = Bio::EnsEMBL::Registry->get_adaptor("human", "core", "Gene");
    my $gene = $ga->fetch_by_stable_id($stable_id);
    return $gene->slice->seq_region_name;
}
