#!/usr/bin/perl

use strict;
use Bio::EnsEMBL::Registry;
use Chum::Statistics::AnalysisResult;
require Chum::Utils::ExpectancyManager;
use Data::Dumper;

############################################################CONSTANTS
##
my $STARSON = 1;
my $NORMALIZE_ALL_EXPECTANCIES = 1;
my $EXPECTANCY_LOADER = undef;
if ($NORMALIZE_ALL_EXPECTANCIES) { $EXPECTANCY_LOADER = new Chum::Utils::ExpectancyManager(); }
Bio::EnsEMBL::Registry->load_all();

############################################################SETUP PROGRAM
##
my $groups_file;
my $raw_results_directory;
my $columns_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-g/) {
        $groups_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-d/) {
        $raw_results_directory = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-c/) {
        $columns_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($groups_file)) { die("No input group file defined"); }
unless (-e $groups_file) { die("Input group file does not exist"); }
if (!defined($raw_results_directory)) { die("No raw results directory defined"); }
unless (-d $raw_results_directory) { die("Raw results directory does not exist"); }
if (!defined($columns_file)) { die("No output columns file defined"); }


############################################################RUN PROGRAM
##
my $results_hashref = &loadAnalysisResults($groups_file, $raw_results_directory);
my %results = %$results_hashref;

############################################################LOAD AVERAGES FOR STATISTICAL ANALYSIS
##
my %averages;
for my $gene (keys %results) {
	$averages{$gene} = &loadAverages($results{$gene});  
}

############################################################HAVE AVERAGES, RUN TESTS
##
my $index = 1;
my %tests = (	"transfac" => $index++,
		"weeder-diff" => $index++,
		"weeder-max" => $index++,
		"motifsampler-diff" => $index++,
		"motifsampler-max" => $index++,
		"bend" => $index++,
		"curve" => $index++,
		"gc" => $index++,
		"temperature" => $index++,
		#"distance" => $index++,
		#"ln-distance" => $index++,
		"repeat-density" => $index++,
		"in-repeat" => $index++,
		"repeat-short-events" => $index++,
		"repeat-long-events" => $index++,
		"maf" => $index++,
		"daf" => $index++,
		"coexpression-jaspar" => $index++,
		"in-cpg" => $index++,
		"nhgri-dnase" => $index++,
		"noble-dnase" => $index++,
		"phastcons" => $index++,
		"rp" => $index++);


############################################################BEGIN ANALYSES
##
print "corner";
foreach my $test (sort { $tests{$a} <=> $tests{$b} } keys %tests) {
	print "\t" . $test;
}
print "\n";
open(RES, ">" . $columns_file) || die ("Cannot open $columns_file file");
print RES "corner\trsnp\n";
for my $gene (keys %results) {
	my $results_hashref = $results{$gene};
	my %analysis_results_hash = %$results_hashref;
	for my $id (keys %analysis_results_hash) {
		my $result = $analysis_results_hash{$id};
		
		################################################################
		######################################@TODO PROCESS ONLY SNPS
		################################################################
		if (!defined($result->{'bend'}->{'score'}) || length($result->{'bend'}->{'score'}) == 0) {
		    next; ##PROBABLY NOT A SNP
		}
		my $closest_gene = undef;
                my $closest_distance = undef;
                my $closest_ln_distance = undef;
                my $distance_hashref = $result->{'distance'};
                for my $key (keys %$distance_hashref) {
                    if (!defined($closest_gene)) {
                        $closest_gene = $result->{'distance'}->{$key}->{'gene'};
                        $closest_distance = $result->{'distance'}->{$key}->{'distance'};
                        $closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
                    } elsif ($result->{'distance'}->{$key}->{'distance'} < $closest_distance) {
                        $closest_gene = $result->{'distance'}->{$key}->{'gene'};
                        $closest_distance = $result->{'distance'}->{$key}->{'distance'};
                        $closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
                    }
                }
		if (abs($closest_distance) > 6001) {
			next; ##NOT IN OUR SEARCH REGION
		}		


		my $OUTPUT_LINE = "";
		
		if ($result->{'id'} =~ /OREG[0-9]+/) {
			print RES $result->{'id'} . "\t1\n";
		} elsif ($result->{'id'} =~ /rs[0-9]+/) {
			print RES $result->{'id'} . "\t0\n";
		}
		$OUTPUT_LINE .= $result->{'id'} . "\t";
		
		if (defined($tests{'transfac'})) {
		    if ($result->{'transfac'}->{'score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'transfac'}->{'score'} - $averages{$gene}{'transfac'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'weeder-diff'})) {
		    if ($result->{'weeder'}->{'score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'weeder'}->{'score'} - $averages{$gene}{'weeder-diff'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'weeder-max'})) {
		    if ($result->{'weeder'}->{'max_score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'weeder'}->{'max_score'} - $averages{$gene}{'weeder-max'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'motifsampler-diff'})) {
		    if ($result->{'motifsampler'}->{'score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'motifsampler'}->{'score'} - $averages{$gene}{'motifsampler-diff'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'motifsampler-max'})) {
		    if ($result->{'motifsampler'}->{'max_score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'motifsampler'}->{'max_score'} - $averages{$gene}{'motifsampler-max'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'bend'})) {
		    if ($result->{'bend'}->{'score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'bend'}->{'score'} - $averages{$gene}{'bend'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'curve'})) {
		    if ($result->{'curve'}->{'score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'curve'}->{'score'} - $averages{$gene}{'curve'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'gc'})) {
		    if ($result->{'gc'}->{'score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'gc'}->{'score'} - $averages{$gene}{'gc'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'temperature'})) {
		    if ($result->{'temperature'}->{'score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'temperature'}->{'score'} - $averages{$gene}{'temperature'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		
		###DISTANCE
		if (defined($tests{'distance'})) {
		    if (defined($closest_distance)) {
			my $divergence_from_average = $closest_distance - $averages{$gene}{'distance'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'ln-distance'})) {
		    if (defined($closest_ln_distance)) {
			my $divergence_from_average = $closest_ln_distance - $averages{$gene}{'ln-distance'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		my $CHROMOSOME = &getChromosomeForGene($closest_gene);

		###REPEAT DENSITY
		my $repeat_density = 0;
		if ($result->{'repeat'} != 0) {
			$repeat_density = $result->{'repeat'}->{'repeat_length'} / $result->{'repeat'}->{'search_length'};
		}
		my $expectancy_adjustment_in_repeat_value = 0;
		my $expectancy_adjustment_short_event = 0;
		my $expectancy_adjustment_long_event = 0;
		if ($NORMALIZE_ALL_EXPECTANCIES) {
		    $expectancy_adjustment_in_repeat_value = $EXPECTANCY_LOADER->get_in_repeat_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_short_event = $EXPECTANCY_LOADER->get_short_event_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_long_event = $EXPECTANCY_LOADER->get_long_event_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    
		}
		if (defined($tests{'repeat-density'})) {
		    my $divergence_from_average = $repeat_density - $averages{$gene}{'repeat-density'};
		    $OUTPUT_LINE .=  $divergence_from_average . "\t";
		}
		if (defined($tests{'in-repeat'})) {
		    if (defined($result->{'repeat'}->{'in_repeat'})) {
			my $divergence_from_average = $result->{'repeat'}->{'in_repeat'} - $averages{$gene}{'in-repeat'} - $expectancy_adjustment_in_repeat_value;
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'repeat-short-events'})) {
		    if (defined($result->{'repeat'}->{'short_events'})) {
			my $divergence_from_average = $result->{'repeat'}->{'short_events'} - $averages{$gene}{'repeat-short-events'} - $expectancy_adjustment_short_event;
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'repeat-long-events'})) {
		    if (defined($result->{'repeat'}->{'long_events'})) {
			my $divergence_from_average = $result->{'repeat'}->{'long_events'} - $averages{$gene}{'repeat-long-events'} - $expectancy_adjustment_long_event;
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		
	
		###MAF
		my $allelefreq_hashref = $result->{'allele_frequency'};
		if ($allelefreq_hashref) {
		    my $printed = 0;
		    for my $key (keys %$allelefreq_hashref) {
			if ($result->{'allele_frequency'}->{$key} < 0.5 && length($key) == 1) {
			    if (defined($tests{'maf'})) {
				my $divergence_from_average = $result->{'allele_frequency'}->{$key} - $averages{$gene}{'maf'};
				$OUTPUT_LINE .=  $divergence_from_average . "\t";
			    }
			    $printed = 1;
			    last;
			}
		    }
		    unless ($printed) {
			if (defined($tests{'maf'})) {
			    if ($STARSON) {
				$OUTPUT_LINE .=  "*\t";    
			    } else {
				$OUTPUT_LINE .=  "0\t";
			    }   
			}
		    }
		} else {
		    if (defined($tests{'maf'})) {
			if ($STARSON) {
				$OUTPUT_LINE .=  "*\t";    
		        } else {
		        	$OUTPUT_LINE .=  "0\t";
			}   
		    }
		}
		
		if (defined($tests{'daf'})) {
		    if ($result->{'daf'} ne "UNDEFINED" && defined($result->{'daf'})) {
			my $divergence_from_average = $result->{'daf'} - $averages{$gene}{'daf'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}

		###EVERYTHING ELSE, ANNOTATION BASED FEATURES
		if (defined($tests{'coexpression-jaspar'})) {
		    if ($result->{'coexpression-jaspar'}->{'score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'coexpression-jaspar'}->{'score'} - $averages{$gene}{'coexpression-jaspar'};
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}

		my $expectancy_adjustment_in_cpg = 0;
		my $expectancy_adjustment_nhgri = 0;
		my $expectancy_adjustment_noble = 0;
		my $expectancy_adjustment_phastcons = 0;
		my $expectancy_adjustment_regulatory_potential = 0;
		if ($NORMALIZE_ALL_EXPECTANCIES) {
		    $expectancy_adjustment_in_cpg = $EXPECTANCY_LOADER->get_in_cpg_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_nhgri = $EXPECTANCY_LOADER->get_nhgri_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_noble = $EXPECTANCY_LOADER->get_noble_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_phastcons = $EXPECTANCY_LOADER->get_phastcons_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_regulatory_potential = $EXPECTANCY_LOADER->get_regulatory_potential_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		}
		
		if (defined($tests{'in-cpg'})) {
		    if ($result->{'cpg'}->{'score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'cpg'}->{'score'} - $averages{$gene}{'in-cpg'} - $expectancy_adjustment_in_cpg;
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'nhgri-dnase'})) {
		    if ($result->{'nhgri-dnase'}->{'score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'nhgri-dnase'}->{'score'} - $averages{$gene}{'nhgri-dnase'} - $expectancy_adjustment_nhgri;
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'noble-dnase'})) {
		    if ($result->{'noble-dnase'}->{'score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'noble-dnase'}->{'score'} - $averages{$gene}{'noble-dnase'} - $expectancy_adjustment_noble;
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'phastcons'})) {
		    if ($result->{'phastcons'}->{'score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'phastcons'}->{'score'} - $averages{$gene}{'phastcons'} - $expectancy_adjustment_phastcons;
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		if (defined($tests{'rp'})) {
		    if ($result->{'rp'}->{'score'} ne "UNDEFINED") {
			my $divergence_from_average = $result->{'rp'}->{'score'} - $averages{$gene}{'rp'} - $expectancy_adjustment_regulatory_potential;
			$OUTPUT_LINE .=  $divergence_from_average . "\t";
		    } else {
			if ($STARSON) {
			    $OUTPUT_LINE .=  "*\t";    
			} else {
			    $OUTPUT_LINE .=  "0\t";
			}
		    }
		}
		chomp($OUTPUT_LINE);
		$OUTPUT_LINE .=  "\n";
		print $OUTPUT_LINE;
	}
}
close (RES);
exit(0);

#############################################################################SUBROUTINES

##############################################################################################
##################################################################LOAD ALL AVERAGES FOR GROUPS

sub loadAverages() {
	my $results_hashref = shift;
	my %results_hash = %$results_hashref;
	
	#######################################################################SETUP TOTALS AND COUNTS HASHES
	my %totals = {"transfac" => 0, "weeder-diff" => 0, "weeder-max" => 0, "motifsampler-diff" => 0, "motifsampler-max" => 0,
			"bend" => 0, "curve" => 0, "gc" => 0, "temperature" => 0, "distance" => 0, "ln-distance" => 0,
			"kaks-mouse" => 0, "kaks-cow" => 0, "kaks-dog" => 0, "kaks-rat" => 0, "kaks-chimp" => 0, "repeat-density" => 0, "in-repeat" => 0,
			"repeat-short-events" => 0, "repeat-long-events" => 0, "maf-rare" => 0, "maf-common" => 0, "maf-unknown" => 0,
			"coexpression-jaspar" => 0, "in-cpg" => 0, "nhgri-dnase" => 0, "noble-dnase" => 0, "phastcons" => 0,
			"rp" => 0, "maf" => 0, "daf" => 0};
	my %counts = {	"transfac" => 0, "weeder-diff" => 0, "weeder-max" => 0, "motifsampler-diff" => 0, "motifsampler-max" => 0,
			"bend" => 0, "curve" => 0, "gc" => 0, "temperature" => 0, "distance" => 0, "ln-distance" => 0,
			"kaks-mouse" => 0, "kaks-cow" => 0, "kaks-dog" => 0, "kaks-rat" => 0, "kaks-chimp" => 0, "repeat-density" => 0, "in-repeat" => 0,
			"repeat-short-events" => 0, "repeat-long-events" => 0, "maf-rare" => 0, "maf-common" => 0, "maf-unknown" => 0,
			"coexpression-jaspar" => 0, "in-cpg" => 0, "nhgri-dnase" => 0, "noble-dnase" => 0, "phastcons" => 0,
			"rp" => 0, "maf" => 0, "daf" => 0};
	
	
	#######################################################################PROCESS ALL RESULT FILES
	for my $id (keys %results_hash) {
		my $result = $results_hash{$id};


		if (!defined($result->{'bend'}->{'score'}) || length($result->{'bend'}->{'score'}) == 0)) {
		    ##probable not a snp
		    next;
		}	


		my $closest_gene;
                my $closest_distance;
                my $closest_ln_distance;
                my $distance_hashref = $result->{'distance'};
                for my $key (keys %$distance_hashref) {
                        if (!defined($closest_gene)) {
                                $closest_gene = $result->{'distance'}->{$key}->{'gene'};
                                $closest_distance = $result->{'distance'}->{$key}->{'distance'};
                                $closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
                        } elsif ($result->{'distance'}->{$key}->{'distance'} < $closest_distance) {
                                $closest_gene = $result->{'distance'}->{$key}->{'gene'};
                                $closest_distance = $result->{'distance'}->{$key}->{'distance'};
                                $closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
                        }
                }
		if (abs($closest_distance) > 6001) {
			next; ##NOT WITHIN OUR SEARCH REGION
		}

		################################################################
		######################################@TODO PROCESS ONLY SNPS
		################################################################

		##############################################ADD ANALYSIS VALUES TO AVERAGE HASH
		if ($result->{'transfac'}->{'score'} ne "UNDEFINED") {
		    $totals{'transfac'} += $result->{'transfac'}->{'score'};
		    $counts{'transfac'} += 1;
		}
		if ($result->{'weeder'}->{'score'} ne "UNDEFINED") {
		    $totals{'weeder-diff'} += $result->{'weeder'}->{'score'};
		    $counts{'weeder-diff'} += 1;
		}
		if ($result->{'weeder'}->{'max_score'} ne "UNDEFINED") {
		    $totals{'weeder-max'} += $result->{'weeder'}->{'max_score'};
		    $counts{'weeder-max'} += 1;
		}
		if ($result->{'motifsampler'}->{'score'} ne "UNDEFINED") {
		    $totals{'motifsampler-diff'} += $result->{'motifsampler'}->{'score'};
		    $counts{'motifsampler-diff'} += 1;
		}
		if ($result->{'motifsampler'}->{'max_score'} ne "UNDEFINED") {
		    $totals{'motifsampler-max'} += $result->{'motifsampler'}->{'max_score'};
		    $counts{'motifsampler-max'} += 1;
		}
		if ($result->{'bend'}->{'score'} ne "UNDEFINED") {
		    $totals{'bend'} += $result->{'bend'}->{'score'};
		    $counts{'bend'} += 1;
		}
		if ($result->{'curve'}->{'score'} ne "UNDEFINED") {
		    $totals{'curve'} += $result->{'curve'}->{'score'};
		    $counts{'curve'} += 1;
		}
		if ($result->{'gc'}->{'score'} ne "UNDEFINED") {
		    $totals{'gc'} += $result->{'gc'}->{'score'};
		    $counts{'gc'} += 1;
		}
		if ($result->{'temperature'}->{'score'} ne "UNDEFINED") {
		    $totals{'temperature'} += $result->{'temperature'}->{'score'};
		    $counts{'temperature'} += 1;
		}
		if ($result->{'coexpression-jaspar'}->{'score'} ne "UNDEFINED") {
		    $totals{'coexpression-jaspar'} += $result->{'coexpression-jaspar'}->{'score'};
		    $counts{'coexpression-jaspar'} += 1;
		}

		##############################################GET CLOSEST GENE INFORMATION
		$totals{'distance'} += $closest_distance;
		$counts{'distance'} += 1;
		$totals{'ln-distance'} += $closest_ln_distance;
		$counts{'ln-distance'} += 1;
		
		my $CHROMOSOME = getChromosomeForGene($closest_gene);
		
		##############################################GET CLOSEST KAKS INFORMATION
		my %resultskaks;
		my $kaks_hashref1 = $result->{'kaks'}->{$closest_gene};
		for my $key1 (keys %$kaks_hashref1) {
			my $kaks_hashref2 = $result->{'kaks'}->{$closest_gene}->{$key1};
			for my $key2 (keys %$kaks_hashref2) {
				if (defined($result->{'kaks'}->{$closest_gene}->{$key1}->{$key2})) {
					if ($key1 eq "Mus musculus" || $key2 eq "Mus musculus") {
						$resultskaks{"mumu"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} elsif ($key1 eq "Bos taurus" || $key2 eq "Bos taurus") {
						$resultskaks{"bota"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} elsif ($key1 eq "Canis familiaris" || $key2 eq "Canis familiaris") {
						$resultskaks{"cafa"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} elsif ($key1 eq "Rattus norvegicus" || $key2 eq "Rattus norvegicus") {
						$resultskaks{"rano"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} elsif ($key1 eq "Pan troglodytes" || $key2 eq "Pan troglodytes") {
						$resultskaks{"patr"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} else {
						die("Should add test for $key1 $key2 in KAKS\n");
					}
				}
			}
		}
		if (defined($resultskaks{"mumu"})) {
		    $totals{'kaks-mouse'} += $resultskaks{"mumu"};
		    $counts{'kaks-mouse'} += 1;
		}
		if (defined($resultskaks{"bota"})) {
		    $totals{'kaks-cow'} += $resultskaks{"bota"};
		    $counts{'kaks-cow'} += 1;
		}
		if (defined($resultskaks{"cafa"})) {
		    $totals{'kaks-dog'} += $resultskaks{"cafa"};
		    $counts{'kaks-dog'} += 1;
		}
		if (defined($resultskaks{"rano"})) {
		    $totals{'kaks-rat'} += $resultskaks{"rano"};
		    $counts{'kaks-rat'} += 1;
		}
		if (defined($resultskaks{"patr"})) {
		    $totals{'kaks-chimp'} += $resultskaks{"patr"};
		    $counts{'kaks-chimp'} += 1;
		}

		##############################################GET REPEAT INFORMATION
		my $repeat_density = 0;
		if ($result->{'repeat'} != 0) {
			$repeat_density = $result->{'repeat'}->{'repeat_length'} / $result->{'repeat'}->{'search_length'};
		}
		$totals{'repeat-density'} += $repeat_density;
		$counts{'repeat-density'} += 1;
		
		my $expectancy_adjustment_in_repeat_value = 0;
		my $expectancy_adjustment_short_event = 0;
		my $expectancy_adjustment_long_event = 0;
		if ($NORMALIZE_ALL_EXPECTANCIES) {
		    $expectancy_adjustment_in_repeat_value = $EXPECTANCY_LOADER->get_in_repeat_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_short_event = $EXPECTANCY_LOADER->get_short_event_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_long_event = $EXPECTANCY_LOADER->get_long_event_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    
		}
		$totals{'in-repeat'} += $result->{'repeat'}->{'in_repeat'} - $expectancy_adjustment_in_repeat_value;
		$counts{'in-repeat'} += 1;
		$totals{'repeat-short-events'} += $result->{'repeat'}->{'short_events'} - $expectancy_adjustment_short_event;
		$counts{'repeat-short-events'} += 1;
		$totals{'repeat-long-events'} += $result->{'repeat'}->{'long_events'} - $expectancy_adjustment_long_event;
		$counts{'repeat-long-events'} += 1;

		##############################################GET ALLELE FREQUENCY INFORMATION
		my $allelefreq_hashref = $result->{'allele_frequency'};
		if ($allelefreq_hashref) {
			for my $key (keys %$allelefreq_hashref) {
					if ($result->{'allele_frequency'}->{$key} < 0.5) {
						$totals{'maf'} += $result->{'allele_frequency'}->{$key};
						$counts{'maf'} += 1;
						last;
					}
			}
		}
		
		##############################################GET DERIVED ALLELE FREQUENCY INFORMATION
		if ($result->{'daf'} ne "UNDEFINED") {
		    $totals{'daf'} += $result->{'daf'};
		    $counts{'daf'} += 1;
		}
		
		
		##############################################GET EXPECTANCY ADJUSTED VALUES FOR ANNOTATION FEATURES
		my $expectancy_adjustment_in_cpg = 0;
		my $expectancy_adjustment_nhgri = 0;
		my $expectancy_adjustment_noble = 0;
		my $expectancy_adjustment_phastcons = 0;
		my $expectancy_adjustment_regulatory_potential = 0;
		if ($NORMALIZE_ALL_EXPECTANCIES) {
		    $expectancy_adjustment_in_cpg = $EXPECTANCY_LOADER->get_in_cpg_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_nhgri = $EXPECTANCY_LOADER->get_nhgri_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_noble = $EXPECTANCY_LOADER->get_noble_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_phastcons = $EXPECTANCY_LOADER->get_phastcons_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		    $expectancy_adjustment_regulatory_potential = $EXPECTANCY_LOADER->get_regulatory_potential_expectancy_value($CHROMOSOME, (0 - $closest_distance));
		}
		if ($result->{'in-cpg'}->{'score'} ne "UNDEFINED") {
		    $totals{'in-cpg'} += $result->{'cpg'}->{'score'} - $expectancy_adjustment_in_cpg;
		    $counts{'in-cpg'} += 1;
		}
		if ($result->{'nhgri-dnase'}->{'score'} ne "UNDEFINED") {
		    $totals{'nhgri-dnase'} += $result->{'nhgri-dnase'}->{'score'} - $expectancy_adjustment_nhgri;
		    $counts{'nhgri-dnase'} += 1;
    		}
		if ($result->{'noble-dnase'}->{'score'} ne "UNDEFINED") {
		    $totals{'noble-dnase'} += $result->{'noble-dnase'}->{'score'} - $expectancy_adjustment_noble;
		    $counts{'noble-dnase'} += 1;
		}
		if ($result->{'phastcons'}->{'score'} ne "UNDEFINED") {
		    $totals{'phastcons'} += $result->{'phastcons'}->{'score'} - $expectancy_adjustment_phastcons;
		    $counts{'phastcons'} += 1;
		}
		if ($result->{'rp'}->{'score'} ne "UNDEFINED") {
		    $totals{'rp'} += $result->{'rp'}->{'score'} - $expectancy_adjustment_regulatory_potential;
		    $counts{'rp'} += 1;
		}
	}
	for my $key (keys %totals) {
		eval {
		    $totals{$key} = $totals{$key} / $counts{$key};
		}; if (@$) {
		    $totals{$key} = 0;
		}
	}
	return \%totals;
}

##############################################################################################
#####################################################################LOAD ALL ANALYSIS RESULTS
sub loadAnalysisResults() {
	my $groups_file = shift;
	my $results_directory = shift;
	my %result_hash;
	open(GROUPS, $groups_file) || die("Cannot open groups file");
	foreach my $line (<GROUPS>) {
		chomp($line);
		$line =~ s/://;
		my @ids = split(/[\s]+/, $line);
		my $geneid;
		foreach my $id (@ids) {
			eval {
				my $ar = new Chum::Statistics::AnalysisResult( 'file' => $results_directory . "/" . $id . ".results");
				my $gene = &getClosestGene($ar);
				if (!defined($geneid)) {
					$geneid = $gene;
				}
				if ($gene ne $geneid) {
					warn("$id doesnt match group: " . $ids[0]);
				} else {
					$result_hash{$geneid}{$id} = $ar;
				}
			};
			if (@$) {
				print STDERR "Error detected: " . @$ . "\n";
			}
		}
		
	}
	close(GROUPS);
	return \%result_hash;
}

##############################################################################################
###################################################################GET CLOSEST GENE SUBROUTINE
sub getClosestGene() {
	my $result = shift;
	my $closest_gene;
	my $closest_distance;
	my $closest_ln_distance;
	my $distance_hashref = $result->{'distance'};
	for my $key (keys %$distance_hashref) {
		if (!defined($closest_gene)) {
			$closest_gene = $result->{'distance'}->{$key}->{'gene'};
			$closest_distance = $result->{'distance'}->{$key}->{'distance'};
			$closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
		} elsif ($result->{'distance'}->{$key}->{'distance'} < $closest_distance) {
			$closest_gene = $result->{'distance'}->{$key}->{'gene'};
			$closest_distance = $result->{'distance'}->{$key}->{'distance'};
			$closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
		}
	}
	return $closest_gene;
}

##############################################################################################
#####################################################################GET CHROMOSOME FROM ENS
sub getChromosomeForGene() {
    my $stable_id = shift;
    my $ga = Bio::EnsEMBL::Registry->get_adaptor("human", "core", "Gene");
    my $gene = $ga->fetch_by_stable_id($stable_id);
    return $gene->slice->seq_region_name;
}
