#!/usr/bin/perl

use strict;
use Data::Dumper;
use Snpegen::Backgroundbuilder;
use Snpegen::Stackbuilder;
use Snpegen::VariationTools();
use Snpegen::Analysis::Motifsampler;
use Snpegen::Analysis::Transfac;
use Snpegen::Analysis::Weeder;
use Snpegen::Result::ScoredResult;
use Snpegen::Result::Interpretter::MotifsamplerInterpretter;
use Snpegen::Result::Interpretter::TransfacInterpretter;
use Snpegen::Result::Interpretter::WeederInterpretter;

#######################################################################################################
##LOAD EACH COMMAND-LINE PARAMETER INTO THE APPROPRIATE VARIABLE#######################################
#######################################################################################################

print STDERR "Type thePipe.pl -help for usage information\n";

for (my $i = 0; $i < @ARGV; $i++) {
	if ($ARGV[$i] =~ /-help/) {
		print "Usage: thePipe.pl -gene ENSGXXXXXXXXX -f variants.gff -min 0.1 -max 0.4 -bg 0 -iga 0 -mbgf background.txt -xt T,W,M\n";
		print " gene: optional parameter, calculates regulatory variants around a gene\n";
		print " f: optional parameter, add variants to the analysis using gff\n";
		print " min: optional parameter, restrict snps to those within a minimum allele frequency\n";
		print " max: optional parameter, restrict snps to those within a maximum allele frequency\n";
		print " bg: optional parameter, generate background, default true (non-zero)\n";
		print " mbgf: optional parameter, motifsampler background file\n";
		print " iga: optional parameter, run the gene analysis, default true (non-zero)\n";
		print " xt: optional parameter, excludes particular tests T: transfac, W: weeder, M: motifsampler\n";
		print " gff file format:\n";
		print "VARIANT-1\tCHROMOSOME\tVARIANT\t100000\t100001\t.\t+\t.\tT\n";
		print "REFERENCE-1\tCHROMOSOME\tVARIANT\t100000\t100001\t.\t+\t.\tC\n";
	}
}


my $ensgid;
my $variation_file;
my $min_maf = 0;
my $max_maf = 0;
my $generate_background = 1; #true;
my $run_gene_analysis = 1; #true
my $background_output_file;
my $exclusion_string;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-gene/) {
        $ensgid = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-f/) {
        $variation_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-min/) {
        $min_maf = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-max/) {
        $max_maf = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-bg/) {
        $generate_background = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-iga/) {
        $run_gene_analysis = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-mbgf/) {
        $background_output_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-xt/) {
	$exclusion_string = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if ($min_maf > $max_maf) {
        print STDERR "\nThe min minor allele frequency must be less than the max minor allele frequency\n";
        exit(1);
}

###2. Set-up test exclusion string
my $exclude_transfac = 0; #false
my $exclude_weeder = 0; #false
my $exclude_motifsampler = 0; #false
my @exclusion_arr = split(/,/, $exclusion_string);
foreach my $exclusion_token (@exclusion_arr) {
	if ($exclusion_token =~ /[Tt]{1}/) { print STDERR "Excluding transfac\n"; $exclude_transfac = 1; }
	elsif ($exclusion_token =~ /[Ww]{1}/) { print STDERR "Excluding weeder\n"; $exclude_weeder = 1; }
	elsif ($exclusion_token =~ /[Mm]{1}/) { print STDERR "Excluding motifsampler\n"; $exclude_motifsampler = 1; }
	else { print STDERR "Unknown exclusion token: " . $exclusion_token . "\n"; }
}

######################################################################################################
##BUILD THE STACKS####################################################################################
######################################################################################################

my $stackbuilder = new Snpegen::Stackbuilder(   'minmaf' => $min_maf,
                                                'maxmaf' => $max_maf);
my @stacks = ();

if (defined($variation_file)) {
	push @stacks, @{$stackbuilder->generate_stacks_from_variation_file($variation_file)};
}
if ($run_gene_analysis) {
	if (defined($ensgid)) {
		push @stacks, @{$stackbuilder->generate_stacks_from_ensgid($ensgid)};
	}
}

if (!defined(@stacks) || scalar(@stacks) == 0) {
	print STDERR "No sequence stacks to analyze\n";
	exit(1);
}

######################################################################################################
##BUILD THE BACKGROUND################################################################################
######################################################################################################

my $background;
if ($generate_background) {
	if (!defined($ensgid)) {
		print STDERR "\nThe background requires an ENSEMBL GENE ID to be generated\n";
		exit(1);
	}
	my $backgroundbuilder = new Snpegen::Backgroundbuilder(	'ensgid' => $ensgid,
															'minmaf' => $min_maf,
															'maxmaf' => $max_maf);
	$background = $backgroundbuilder->generate_background;
}


######################################################################################################
##PUMP THROUGH ANALYSIS PROGRAMS AND SCORE STACKS#####################################################
######################################################################################################

my $vtools = new Snpegen::VariationTools();

#1. Run through analysis program, Transfac ###########################################################

if (!$exclude_transfac) {
my $transfac = new Snpegen::Analysis::Transfac('transfac_dir' => "/home/pubseq/databases/TransFac/current/www/biobase/transfac/7.2/data");
my $tinterpretter = new Snpegen::Result::Interpretter::TransfacInterpretter();
my %transfac_results_hash = ();
my $size = scalar(@stacks);
my $tcounter = 0;
print STDERR "PROCESSING $size STACKS\n";
foreach  my $stack (@stacks) {
	my $variation_name = $stack->get_variation_name;
	print STDERR "processing: " . $tcounter++ . "/" . $size . "\t" . $variation_name . "\n"; 
	
	if (!defined(%transfac_results_hash->{$variation_name})) {
		my $results = $transfac->run_variation_analysis($stack);
		my $score = $tinterpretter->interpret($results);
		my $scored_result = new Snpegen::Result::ScoredResult(	'score' => $score,
																'alleles' => $stack->alleles,
																'species' => $stack->get_ids_string_in_stack,
																'variation_name' => $variation_name);
		%transfac_results_hash->{$variation_name} = $scored_result;
		print STDERR $stack->refseq->seq . "\n";
		foreach my $varseq (@{$stack->variantseqs}) {
			print STDERR $varseq->seq . "\n";
		}
		print STDERR "SNPEGENRES:\t" . $variation_name . "\tTRANSFAC\t" . $score .  "\t" . $stack->get_ids_string_in_stack . "\t" . $vtools->generate_allele_string($stack->alleles) . "\n";
		print "SNPEGENRES:\t" . $variation_name . "\tTRANSFAC\t" . $score .  "\t" . $stack->get_ids_string_in_stack . "\t" . $vtools->generate_allele_string($stack->alleles) . "\n";
	}
}
$transfac->destroy; ##Reclaim the memory held by the transfac database
}

#2. Run through analysis program, Weeder #############################################################
if (!$exclude_weeder) {
my $weeder = new Snpegen::Analysis::Weeder();
my %weeder_results_hash = ();
my @wservices = @{$weeder->get_weeder_services};
my $wrandom_index = rand @wservices;
my $wservice = @wservices[$wrandom_index];
print STDERR "Weeder running at " . $wservice->getLocation . "\n";
if (scalar(@wservices) > 0) {
	my @wparameters = @{$weeder->get_input_parameters($wservice)};
	my $winterpretter = new Snpegen::Result::Interpretter::WeederInterpretter();	
	my $size = scalar(@stacks);
	print STDERR "PROCESSING $size STACKS\n";
	my $counter = 0;
	foreach  my $stack (@stacks) {
		my $variation_name = $stack->get_variation_name;
		print STDERR "processing: " . $counter++ . "/" . $size . "\t" . $variation_name . "\n"; 
		
		if (!defined(%weeder_results_hash->{$variation_name})) {
			my $results = $weeder->run_variation_analysis($wservice, $stack, \@wparameters);
			my $score = $winterpretter->interpret($results);
			my $scored_result = new Snpegen::Result::ScoredResult(	'score' => $score,
																	'alleles' => $stack->alleles,
																	'species' => $stack->get_ids_string_in_stack,
																	'variation_name' => $variation_name);
			%weeder_results_hash->{$variation_name} = $scored_result;
			print STDERR $stack->refseq->seq . "\n";
			foreach my $varseq (@{$stack->variantseqs}) {
				print STDERR $varseq->seq . "\n";
			}
			print STDERR "SNPEGENRES:\t" . $variation_name . "\tWEEDER\t" . $score .  "\t" . $stack->get_ids_string_in_stack . "\t" . $vtools->generate_allele_string($stack->alleles) . "\n";
			print "SNPEGENRES:\t" . $variation_name . "\tWEEDER\t" . $score .  "\t" . $stack->get_ids_string_in_stack . "\t" . $vtools->generate_allele_string($stack->alleles) . "\n";
		}
	}
} else {
	print STDERR "Not running weeder\n";
}
}

######################################################################################################

if (!$exclude_motifsampler) {
#3. Run through analysis program, Motifsampler########################################################
my $motifsampler = new Snpegen::Analysis::Motifsampler();

#a. Generate Motifsampler background file, if not specified
if (!defined $background_output_file) {
	if (!defined($background)) {
		die "Set bg parameter to true to activate background generator for Motifsampler";
	}

	$background_output_file = "/home/smontgom/sNPeGEn/background/" . $ensgid . ".background.txt";
	my @mbservices = @{$motifsampler->get_motifsampler_background_services};

	my $mbstatus;
	if (defined @mbservices) {
		if (scalar(@mbservices) > 0) {
			my $mbrandom_index = rand @mbservices;
			my $mbservice = @mbservices[$mbrandom_index];
			my @mbparameters = @{$motifsampler->get_input_parameters($mbservice)};
			$mbstatus = $motifsampler->run_motifsampler_background_service(	$mbservice,
																			$background, 
																			\@mbparameters, 
																			$background_output_file);
		}
	}
	if ($mbstatus) { die("Could not obtain background"); }
	if (!defined($background_output_file)) { #Check that a background has been created
		die "Background file not specified for Motifsampler, cannot run, so exiting..."
	}
	print STDERR "Background generated at: " . $background_output_file . "\n";
} 

#b. Run Motifsampler
my %motifsampler_results_hash = ();
my @mservices = @{$motifsampler->get_motifsampler_services};
my $mrandom_index = rand @mservices;
my $mservice = @mservices[$mrandom_index];
print STDERR "Motifsampler running at " . $mservice->getLocation . "\n";

if (scalar(@mservices) > 0) {
	my @mparameters = @{$motifsampler->get_input_parameters($mservice)};
	my $minterpretter = new Snpegen::Result::Interpretter::MotifsamplerInterpretter();
	my $size = scalar(@stacks);
	print STDERR "PROCESSING $size STACKS\n";
	my $counter = 0;
	foreach  my $stack (@stacks) {
		my $variation_name = $stack->get_variation_name;
		print STDERR "processing: " . $counter++ . "/" . $size . "\t" . $variation_name . "\n"; 

		print STDERR $stack->refseq->seq . "\n";
        foreach my $varseq (@{$stack->variantseqs}) {
			print STDERR $varseq->seq . "\n";
        }

		if (!defined(%motifsampler_results_hash->{$variation_name})) {
			my $results = $motifsampler->run_variation_analysis($mservice, $stack, \@mparameters, $background_output_file);
			my $score = $minterpretter->interpret($results);
			my $scored_result = new Snpegen::Result::ScoredResult(	'score' => $score,
																	'alleles' => $stack->alleles,
																	'species' => $stack->get_ids_string_in_stack,
																	'variation_name' => $variation_name);
			
			%motifsampler_results_hash->{$variation_name} = $scored_result;
			print STDERR "SNPEGENRES:\t" . $variation_name . "\tMOTIFSAMPLER\t" . $score .  "\t" . $stack->get_ids_string_in_stack . "\t" . $vtools->generate_allele_string($stack->alleles) . "\n";
			print "SNPEGENRES:\t" . $variation_name . "\tMOTIFSAMPLER\t" . $score .  "\t" . $stack->get_ids_string_in_stack . "\t" . $vtools->generate_allele_string($stack->alleles) . "\n";
		} 
	}
} else {
	print STDERR "Not running Motifsampler\n";
}
}
######################################################################################################

######################################################################################################
##POSTPROCESS/OUTPUT RESULTS##########################################################################
######################################################################################################
